#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include "aoeproto.h"
#include "etherproto.h"
#include "ata.h"
#include "aoedisk.h"
#include "disk.h"
#include "enet.h"
#include "aoedev.h"
#include "aoedevint.h"

typedef uint64_t blkno_t;
#define	BADLBA	((blkno_t)~0)

static int atanop(Aoe, char *, char *, int);
static int ataident(Aoe, char *, char *, int);
static int ataread(Aoe, char *, char *, int);
static int atawrite(Aoe, char *, char *, int);
static int putataerr(Aoe, char *, char *, blkno_t);
static blkno_t aoelba(Aoe, char *);
static void clearlba(char *, int);
static void toatastr(char *, int, char *, int);

int
aoeata(ap, buf, len)
Aoe ap;
void *buf;
int len;
{
	char *p, *q;
	int ndata, af;

	p = buf;
	p += LEN_ETHER;
	if (len < LEN_ETHER + MINLEN_AOEA) {
		(*ap->eout)("--- truncated ATA command");
		return (aoeputerr(ap, AOEBADARG, 0));
	}
	q = ap->outbuf;
	q += LEN_ETHER;
	memcpy(q + AOE_ARG, p + AOE_ARG, MINLEN_AOEA - AOE_ARG);
	af = fraoechar(p, AOEA_AF);
	ndata = fraoechar(p, AOEA_NSEC) * AOEASEC;
	if (af & AOEAFW) {
		if (ndata > len - (LEN_ETHER + MINLEN_AOE)) {
			(*ap->eout)("--- write: missing data: %d/%d",
				len - (LEN_ETHER + MINLEN_AOE), ndata);
			return (aoeputerr(ap, AOEBADARG, 0));
		}
	} else {
		if (ndata > ap->outlen - (LEN_ETHER + MINLEN_AOE)) {
			(*ap->eout)("--- read: too much data requested: %d/%d",
				ndata, ap->outlen - (LEN_ETHER + MINLEN_AOE));
			return (aoeputerr(ap, AOEBADARG, 0));
		}
	}
	if (af & AOEAFD)
		return (atanop(ap, p, q, 0));
	switch (fraoechar(p, AOEA_CS)) {
	case ATANOP:
		return (atanop(ap, p, q, 1));

	case ATAIDENTIFY:
		return (ataident(ap, p, q, ndata));

	case ATARDSEC:
		if ((af & (AOEAFW|AOEAFE)) != 0)
			return (aoeputerr(ap, AOEBADARG, 0));
		return (ataread(ap, p, q, ndata));

	case ATARDXSEC:
		if ((af & (AOEAFW|AOEAFE)) != AOEAFE)
			return (aoeputerr(ap, AOEBADARG, 0));
		return (ataread(ap, p, q, ndata));

	case ATAWRSEC:
		if ((af & (AOEAFW|AOEAFE)) != AOEAFW)
			return (aoeputerr(ap, AOEBADARG, 0));
		return (atawrite(ap, p, q, ndata));

	case ATAWRXSEC:
		if ((af & (AOEAFW|AOEAFE)) != (AOEAFW|AOEAFE))
			return (aoeputerr(ap, AOEBADARG, 0));
		return (atawrite(ap, p, q, ndata));

	default:
		return (aoeputerr(ap, AOEBADVER, 0));
	}
}

/*
 * ATA no-op:
 * standard says always reports an error,
 * with status
 *
 * drvok means the drive should be ready;
 * otherwise it's not
 * we may call this routine for some other error aborts too
 */
static int
atanop(ap, p, q, drvok)
Aoe ap;
char *p, *q;
int drvok;
{
	toaoechar(q, AOEA_EF, ATAER_ABRT);
	toaoechar(q, AOEA_NSEC, 0);
	clearlba(q, fraoechar(p, AOEA_AF));
	if (drvok)
		toaoechar(q, AOEA_CS, ATAST_ERR|ATAST_DF|ATAST_DRDY);
	else
		toaoechar(q, AOEA_CS, ATAST_ERR|ATAST_DF);
	return (enetsend(ap->ep, ap->outbuf, LEN_ETHER + MINLEN_AOEA));
}

/*
 * ATA identify
 * make up constant data
 */
static int
ataident(ap, p, q, ndata)
Aoe ap;
char *p, *q;
int ndata;
{
	disksize_t size;
	char serial[LEN_ATASERIAL];

	if ((fraoechar(p, AOEA_AF) & AOEAFW) != 0
	||  ndata < ATAIDSECS * ATASEC) {
		(*ap->eout)("ATA IDENTIFY: not a write, or too little data requested");
		return (aoeputerr(ap, AOEBADARG, 0));
	}
	if (ap->eflags & AFE_IDENT)
		return (putataerr(ap, p, q, 0));
	size = disksize(ap->dp) / AOEASEC;
	toaoechar(q, AOEA_EF, 0);
	toaoechar(q, AOEA_NSEC, ATAIDSECS);
	clearlba(q, fraoechar(p, AOEA_AF));
	toaoechar(q, AOEA_CS, ATAST_DRDY);
	q += AOEA_DATA;
	memset(q, 0, ATAIDSECS * ATASEC);
	/* ATAID_TYPE == 0 */
	snprintf(serial, sizeof(serial), "%d", getpid());
	toatastr(q, ATAID_SERIAL, serial, LEN_ATASERIAL);
	toatastr(q, ATAID_FWVER, "1", LEN_ATAFWVER);
	toatastr(q, ATAID_MODEL, "aoedisk", LEN_ATAMODEL);
	/* ATAID_MAXMULT? */
	toataword(q, ATAID_CAPAB, ATAC_LBA);
	if (size > 0xffffffffUL)
		toatalong(q, ATAID_MAX28LBN, 0xffffffff);
	else
		toatalong(q, ATAID_MAX28LBN, size);
	toatalong(q, ATAID_CMDSUPP, ATAC_LBA48|ATAC_NOP);
	toatalong(q, ATAID_CMDENAB, ATAC_LBA48|ATAC_NOP);
	toatall(q, ATAID_MAX48LBN, size);
	ndata = LEN_ETHER + AOEA_DATA + (ATAIDSECS*ATASEC);
	if (ap->flags & AF_LOGALL) {
		(*ap->eout)("=== ATA IDENTIFY response:");
		aoedump(ap, ap->outbuf, ndata);
	}
	return (enetsend(ap->ep, ap->outbuf, ndata));
}

/*
 * general syntax issues cleared in advance;
 * just do the I/O
 */

static int
ataread(ap, p, q, ndata)
Aoe ap;
char *p, *q;
int ndata;
{
	blkno_t bno;
	int nact;
	int e;

	if ((bno = aoelba(ap, p)) == BADLBA)
		return (aoeputerr(ap, AOEBADARG, 0));
	if ((ap->eflags & (AFE_HARD|AFE_SOFT)) != 0
	&&  (ap->emod == 0 || (bno % ap->emod) == 0))
		return (putataerr(ap, p, q, bno));
	nact = diskread(ap->dp, bno * ATASEC, q + AOEA_DATA, ndata);
	if (ap->flags & AF_LOGALL)
		(*ap->eout)("ATA LBA %lld: read %d -> %d", bno, ndata, nact);
	if ((ap->flags & (AF_VERBOSE|AF_LOGALL)) != 0 && nact < 0)
		(*ap->eout)("ATA LBA %lld: read error: %s",
			bno, strerror(errno));
	if (nact == ndata)
		e = 0;
	else if (nact >= 0 || errno == ENXIO) {
		e = ATAER_ABRT|ATAER_IDNF;
		nact = 0;
	} else {
		e = ATAER_ABRT|ATAER_UNC;
		nact = 0;
	}
	toaoechar(q, AOEA_EF, e);
	if (e == 0)
		toaoechar(q, AOEA_CS, ATAST_DRDY);
	else
		toaoechar(q, AOEA_CS, ATAST_DRDY|ATAST_ERR);
	if (ap->flags & AF_LOGALL) {
		(*ap->eout)("=== ATA read reply:");
		aoedump(ap, ap->outbuf, LEN_ETHER + AOEA_DATA + nact);
	}
	return (enetsend(ap->ep, ap->outbuf, LEN_ETHER + AOEA_DATA + nact));
}

static int
atawrite(ap, p, q, ndata)
Aoe ap;
char *p, *q;
int ndata;
{
	blkno_t bno;
	int nact;
	int e;

	if ((bno = aoelba(ap, p)) == BADLBA)
		return (aoeputerr(ap, AOEBADARG, 0));
	if ((ap->eflags & (AFE_HARD|AFE_SOFT)) != 0
	&&  (ap->emod == 0 || (bno % ap->emod) == 0))
		return (putataerr(ap, p, q, bno));
	nact = diskwrite(ap->dp, bno * ATASEC, p + AOEA_DATA, ndata);
	if (ap->flags & AF_LOGALL)
		(*ap->eout)("ATA LBA %lld: write %d -> %d", bno, ndata, nact);
	if ((ap->flags & (AF_VERBOSE|AF_LOGALL)) != 0 && nact < 0)
		(*ap->eout)("ATA LBA %lld: write error: %s",
			bno, strerror(errno));
	if (nact == ndata)
		e = 0;
	else if (nact >= 0 || errno == ENXIO)
		e = ATAER_ABRT|ATAER_IDNF;
	else
		e = ATAER_ABRT|ATAER_UNC;
	toaoechar(q, AOEA_EF, e);
	if (e == 0)
		toaoechar(q, AOEA_CS, ATAST_DRDY);
	else
		toaoechar(q, AOEA_CS, ATAST_DRDY|ATAST_ERR);
	if (ap->flags & AF_LOGALL) {
		(*ap->eout)("=== ATA write reply:");
		aoedump(ap, ap->outbuf, LEN_ETHER + AOEA_DATA);
	}
	return (enetsend(ap->ep, ap->outbuf, LEN_ETHER + AOEA_DATA));
}

static int
putataerr(ap, p, q, bno)
Aoe ap;
char *p, *q;
blkno_t bno;
{
	switch (fraoechar(p, AOEA_CS)) {
	case ATARDSEC:
	case ATARDXSEC:
	case ATAWRSEC:
	case ATAWRXSEC:
		if (ap->eflags & AFE_HARD) {
			toaoechar(q, AOEA_CS, ATAST_ERR);
			toaoechar(q, AOEA_EF, ATAER_ABRT|ATAER_UNC);
		} else {
			toaoechar(q, AOEA_CS, ATAST_ERR|ATAST_CORR);
			toaoechar(q, AOEA_EF, ATAER_ABRT);
		}
		break;

	default:
		toaoechar(q, AOEA_CS, ATAST_ERR);
		toaoechar(q, AOEA_EF, ATAER_ABRT);
		break;
	}
	toaoechar(q, AOEA_NSEC, 0);
	if (ap->flags & AF_LOGALL) {
		(*ap->eout)("=== ATA error response:");
		aoedump(ap, ap->outbuf, LEN_ETHER + AOEA_DATA);
	}
	return (enetsend(ap->ep, ap->outbuf, LEN_ETHER + AOEA_DATA));
}

static blkno_t
aoelba(ap, p)
Aoe ap;
char *p;
{
	blkno_t bno;
	int n;

	if (fraoechar(p, AOEA_AF) & AOEAFE) {	/* LBA48 */
		bno = fraoechar(p, AOEA_LBA5);
		bno <<= 8;
		bno |= fraoechar(p, AOEA_LBA4);
		bno <<= 8;
		bno |= fraoechar(p, AOEA_LBA3);
		bno <<= 8;
	} else if (((n = fraoechar(p, AOEA_LBA3)) & ATAHD_MB1) == 0) {
		(*ap->eout)("ATA invalid: LBA28 and MB1 not set");
		return (BADLBA);
	} else if ((n & ATAHD_LBA) == 0) {
		(*ap->eout)("ATA invalid: not LBA addressing");
		return (BADLBA);
	} else {
		bno = (n & ATAHD_ADDR);
		bno <<= 8;
	}
	bno |= fraoechar(p, AOEA_LBA2);
	bno <<= 8;
	bno |= fraoechar(p, AOEA_LBA1);
	bno <<= 8;
	bno |= fraoechar(p, AOEA_LBA0);
	return (bno);
}

static void
clearlba(q, af)
char *q;
int af;
{
	toaoechar(q, AOEA_LBA0, 0);
	toaoechar(q, AOEA_LBA1, 0);
	toaoechar(q, AOEA_LBA2, 0);
	if (af & AOEAFE) {
		toaoechar(q, AOEA_LBA3, 0);
		toaoechar(q, AOEA_LBA4, 0);
		toaoechar(q, AOEA_LBA5, 0);
	}
}

/*
 * put string in Intel-word-style byte order
 */
static void
toatastr(p, off, s, maxlen)
char *p;
int off;
char *s;
int maxlen;
{
	int i;

	p += off;
	memset(p, ' ', maxlen);
	if (maxlen % 2)
		maxlen--;	/* safety */
	for (i = 0; i < maxlen; i += 2) {
		if (s[i] == 0)
			break;
		p[i+1] = s[i];
		if (s[i+1] == 0)
			break;
		p[i] = s[i+1];
	}
}
