#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "aoeproto.h"
#include "etherproto.h"
#include "aoedisk.h"
#include "disk.h"
#include "enet.h"
#include "aoedev.h"
#include "aoedevint.h"

#define	FWVER	1		/* phony firmware version */

static int aoemkconf(Aoe, unsigned long);
static int aoewanted(Aoe, void *, int);
static int aoepedant(Aoe, void *, int);
static void aoemkrsp(Aoe, void *);
static char *eaddr(char *, char *);
static char *aoecmdname(int);
static char *cstrname(int);
static void hexdump(Aoe, void *, int);
static void nullerr(char *, ...);

Aoe
aoeinit(ep, dp, cf, eout)
Enet ep;
Disk dp;
struct aoeconf *cf;
void (*eout)(char *, ...);
{
	Aoe ap;
	int n, m;
	char *p;

	if ((ap = malloc(sizeof(struct aoedev))) == NULL) {
		if (eout)
			(*eout)("no memory for Aoe");
		return (NULL);
	}
	ap->outlen = enetmtu(ep) + LEN_ETHER;
	if ((ap->outbuf = malloc(ap->outlen)) == NULL) {
		if (eout)
			(*eout)("no memory for Aoe output buffer");
		free(ap);
		return (NULL);
	}
	ap->ep = ep;
	ap->dp = dp;
	if ((ap->eout = eout) == NULL)
		ap->eout = nullerr;
	ap->eflags = cf->eflags;
	ap->emod = cf->emod;
	ap->maj = cf->maj;
	ap->min = cf->min;
	ap->nbuf = cf->nbuf;
	ap->flags = cf->flags;
	ap->eproto = cf->eproto;
	ap->cstr = strdup(cf->cstr ? cf->cstr : "");
	if (ap->cstr && strlen(ap->cstr) > AOEMAXSTR)
		ap->cstr[AOEMAXSTR] = 0;
	if ((ap->flags & AF_NOBCAST) == 0) {
		n = aoemkconf(ap, 0);
		p = ap->outbuf;
		memset(p + ETHER_DST, 0xff, LEN_EADDR);
		toetherprot(p, ETHER_PROT, ap->eproto);
		if (ap->flags & AF_LOGALL) {
			(*eout)("=== broadcast config:");
			aoedump(ap, ap->outbuf, n);
		}
		if ((m = enetsend(ap->ep, ap->outbuf, n)) != n)
			(*eout)("warning: cannot broadcast config: %d/%d: %s",
				m, n, strerror(errno));
	}
	return (ap);
}

void
aoeclose(ap)
Aoe ap;
{
	if (ap->outbuf)
		free(ap->outbuf);
	if (ap->cstr)
		free(ap->cstr);
	free(ap);
}

/*
 * return
 * >0	handled command, sent response, even if an AoE error
 * 0	ignored command, probably because it wasn't ours
 * <0	couldn't respond for some reason
 */

int
aoecmd(ap, buf, len)
Aoe ap;
void *buf;
int len;
{
	char *p;
	int e, n;
	char cstr[AOEMAXSTR + 1];

	if (len < LEN_ETHER + MINLEN_AOE) {
		(*ap->eout)("=== Runt message ignored:");
		aoedump(ap, buf, len);
		return (0);
	}
	p = buf;
	p += LEN_ETHER;
	if (aoeversion(fraoechar(p, AOE_VF)) != AOEV1) {
		if (ap->flags & (AF_VERBOSE|AF_LOGALL)) {
			(*ap->eout)("=== Wrong AoE version:");
			aoedump(ap, buf, len);
		}
		/*
		 * should we send back wrong-proto-version error?
		 * how can we?  if we don't know the protocol version,
		 * how can we decide whether it was addressed to us,
		 * or compose an error reply?
		 */
		return (0);
	}
	if (ap->flags & AF_LOGALL) {
		(*ap->eout)("=== Command arrived:");
		aoedump(ap, buf, len);
	}
	if ((e = aoewanted(ap, buf, len)) <= 0) {
		if (ap->flags & AF_LOGALL)
			(*ap->eout)("--- not for us, ignored");
		else if (e < 0) {
			(*ap->eout)("=== invalid command:");
			aoedump(ap, buf, len);
		}
		return (0);
	}
	aoemkrsp(ap, buf);
	if (aoepedant(ap, buf, len) == 0) {
		if (ap->flags & AF_LOGALL)
			aoedump(ap, buf, len);
	}
	switch (fraoechar(p, AOE_COMMAND)) {
	case AOECID:
		n = fraoeshort(p, AOEI_CLEN);
		if (n > AOEMAXSTR) {
			(*ap->eout)("config string length too big: %d", n);
			return (aoeputerr(ap, AOEBADARG, 0));
		}
		if (n > len - (LEN_ETHER + AOEI_CSTR)) {
			(*ap->eout)("config string longer than command: %d/%d",
				n, len - (LEN_ETHER + AOEI_CSTR));
			return (aoeputerr(ap, AOEBADARG, 0));
		}
		memcpy(cstr, p + AOEI_CSTR, n);
		cstr[n] = 0;
		switch (aoeicc(fraoechar(p, AOEI_VCC))) {
		case AOEICGET:
			break;

		case AOEICMATCH:
			if (ap->cstr == NULL
			||  strcmp(ap->cstr, cstr) != 0)
				return (1);	/* no response */
			break;

		case AOEICPREFIX:
			if (ap->cstr == NULL
			||  strstr(ap->cstr, cstr) != ap->cstr)
				return (1);
			break;

		case AOEICSET:
			if (ap->cstr && ap->cstr[0])
				return (aoeputerr(ap, AOEBADSTR, 0));
			/* fall through */
		case AOEICFORCE:
			if (ap->cstr)
				free(ap->cstr);
			if ((ap->cstr = strdup(cstr)) == NULL)
				return (aoeputerr(ap, AOEBADDEV, 0));
			break;

		default:
			return (aoeputerr(ap, AOEBADARG, 0));
		}
		n = aoemkconf(ap, fraoelong(p, AOE_TAG));
		if (ap->flags & AF_LOGALL) {
			(*ap->eout)("=== config response:");
			aoedump(ap, ap->outbuf, n);
		}
		return (enetsend(ap->ep, ap->outbuf, n));

	case AOECATA:
		return (aoeata(ap, buf, len));
		break;

	}
	/*
	 * if we fall through to here,
	 * we didn't understand the command
	 */
	if (ap->flags & AF_LOGALL)
		(*ap->eout)("--- unknown command ignored");
	else {
		(*ap->eout)("=== unknown AoE command:");
		aoedump(ap, buf, len);
	}
	return (aoeputerr(ap, AOEBADCMD, 0));
}

static int
aoemkconf(ap, tag)
Aoe ap;
unsigned long tag;
{
	char *p;
	int n;

	p = ap->outbuf;
	p += LEN_ETHER;
	memset(p, 0, MINLEN_AOEI);
	toaoechar(p, AOE_VF, aoemkvf(AOEF_R));
	toaoeshort(p, AOE_MAJOR, ap->maj);
	toaoechar(p, AOE_MINOR, ap->min);
	toaoechar(p, AOE_COMMAND, AOECID);
	toaoelong(p, AOE_TAG, tag);
	toaoeshort(p, AOEI_MAXBUF, ap->nbuf);
	toaoeshort(p, AOEI_FV, FWVER);
	toaoechar(p, AOEI_VCC, aoeimkvcc(AOEV1, AOEICGET));
	if (ap->cstr == NULL)
		n = 0;
	else {
		n = strlen(ap->cstr);
		if (n > AOEMAXSTR)
			n = AOEMAXSTR;
		memcpy(p + AOEI_CSTR, ap->cstr, n);
	}
	toaoeshort(p, AOEI_CLEN, n);
	return (LEN_ETHER + AOEI_CSTR + n);
}

/*
 * is this message for our drive?
 * very basic header-valid check,
 * then match against maj/min
 */
static int
aoewanted(ap, buf, len)
Aoe ap;
void *buf;
int len;
{
	char *p;
	int maj, min;

	p = buf;
	p += LEN_ETHER;
	if (fraoechar(p, AOE_VF) & AOEF_R)
		return (0);
	maj = fraoeshort(p, AOE_MAJOR);
	min = fraoechar(p, AOE_MINOR);
	if (min != ap->min && min != AOEMINWILD)
		return (0);
	if (maj != ap->maj && maj != AOEMAJWILD)
		return (0);
	return (1);
}

/*
 * pedantic validity check:
 * log errors such as non-zero MBZ fields
 * return
 * > 0	all is well
 * 0	something is awry; perhaps message should be dumped out
 *
 * it is known that this is a command
 * aoewanted has already been called,
 * so some initial validity checks may be skipped
 */

#define	AOEAFALL	(AOEAFE|AOEAFD|AOEAFA|AOEAFW)

static int
aoepedant(ap, buf, len)
Aoe ap;
void *buf;
int len;
{
	char *p;
	int n, m;
	int anom;

	p = buf;
	p += LEN_ETHER;
	len -= LEN_ETHER;
	anom = 0;
	/*
	 * common header
	 */
	if ((n = fraoechar(p, AOE_VF)) & ~(AOEF_VER)) {
		(*ap->eout)("pedant: unexpected flags: 0x%x", n);
		anom++;
	}
	if ((n = fraoechar(p, AOE_ERROR))) {
		(*ap->eout)("pedant: nonzero error in command: 0x%x", n);
		anom++;
	}
	/*
	 * per command checks
	 * --invalid command code is diagnosed anyway
	 */
	switch (fraoechar(p, AOE_COMMAND)) {
	case AOECATA:
		n = fraoechar(p, AOEA_AF);
		m = fraoechar(p, AOEA_NSEC);
		if (n & ~AOEAFALL) {
			(*ap->eout)("pedant: ATA: nonzero MBZ Aflags: 0x%x", n);
			anom++;
		}
		if ((n & AOEAFW) != 0 && m == 0) {
			(*ap->eout)("pedant: ATA: flag W set with zero sector count");
			anom++;
		}
		if ((n & (AOEAFW|AOEAFA)) == AOEAFA) {
			(*ap->eout)("pedant: ATA: flag A set without W");
			anom++;
		}
		if ((n & (AOEAFE|AOEAFD)) == AOEAFD) {
			(*ap->eout)("pedant: ATA: flag D set without E");
			anom++;
		}
		if ((n & AOEAFE) == 0
		&&  (fraoechar(p, AOEA_LBA5) != 0
		 ||  fraoechar(p, AOEA_LBA4) != 0
		 ||  (fraoechar(p, AOEA_LBA3) & 0x0f) != 0)) {
			(*ap->eout)("pedant: ATA: LBA > 28 bits, but flag E not set");
			anom++;
		}
		if ((n & AOEAFW) != 0
		&&  m != 0 && m * AOEASEC != len - AOEA_DATA) {
			(*ap->eout)("pedant: ATA: nsec says %d bytes data, message has %d",
				m * AOEASEC, len - AOEA_DATA);
			anom++;
		}
		break;

	case AOECID:
		/*
		 * we know this is a command!
		 */
		if (fraoeshort(p, AOEI_MAXBUF) != 0) {
			(*ap->eout)("pedant: QC: maxbuf nonzero");
			anom++;
		}
		if (fraoeshort(p, AOEI_FV) != 0) {
			(*ap->eout)("pedant: QC: fwver nonzero");
			anom++;
		}
		if (fraoechar(p, AOEI_MAXSEC) != 0) {
			(*ap->eout)("pedant: QC: maxsec nonzero");
			anom++;
		}
		if (aoeiver(fraoechar(p, AOEI_VCC)) != 0) {
			(*ap->eout)("pedant: QC: AoE version nonzero");
			anom++;
		}
		n = fraoeshort(p, AOEI_CLEN);
		if (n > AOEMAXSTR) {
			(*ap->eout)("pedant: QC: string length %d too big", n);
			anom++;
		}
		if (n > len - AOEI_CSTR) {
			(*ap->eout)("pedant: QC: string length %d, message has only %d",
				n, len - AOEI_CSTR);
			anom++;
		}
		break;
	}
	if (anom)
		return (0);
	return (1);
}

/*
 * init outbuf: set up header for response to buf
 */
static void
aoemkrsp(ap, buf)
Aoe ap;
void *buf;
{
	char *p, *q;

	p = buf;
	q = ap->outbuf;
	memset(q, 0, MINLEN_AOE);
	memcpy(q + ETHER_DST, p + ETHER_SRC, LEN_EADDR);
	toetherprot(q, ETHER_PROT, fretherprot(p, ETHER_PROT));
	p += LEN_ETHER;
	q += LEN_ETHER;
	toaoechar(q, AOE_VF, aoemkvf(AOEF_R));
	toaoeshort(q, AOE_MAJOR, ap->maj);
	toaoechar(q, AOE_MINOR, ap->min);
	toaoechar(q, AOE_COMMAND, fraoechar(p, AOE_COMMAND));
	toaoelong(q, AOE_TAG, fraoelong(p, AOE_TAG));
}

/*
 * return AoE error err,
 * using the header aoemkrsp made in outbuf
 * if len == 0, just send the header;
 * if nonzero, indicates amount of data
 */
int
aoeputerr(ap, err, len)
Aoe ap;
int err;
int len;
{
	char *p;
	int n;

	p = ap->outbuf;
	p += LEN_ETHER;
	toaoechar(p, AOE_ERROR, err);
	p[AOE_VF] |= AOEF_E;
	if (len == 0)
		len = LEN_ETHER + MINLEN_AOE;
	if (ap->flags & AF_LOGALL) {
		(*ap->eout)("=== put error response:");
		aoedump(ap, ap->outbuf, len);
	}
	if ((n = enetsend(ap->ep, ap->outbuf, len)) != len) {
		(*ap->eout)("write error");
		return (-1);
	}
	return (1);
}

/*
 * dump the contents of an AoE message
 * control data only (for now)
 */
void
aoedump(ap, buf, len)
Aoe ap;
void *buf;
int len;
{
	char *p;
	char *bp;
	char b1[LEN_EADDR * 3], b2[LEN_EADDR * 3];	/* xx- or xx\0 per octet */
	int n, m;
	uint64_t lba;

	if (ap->eout == nullerr)
		return;
	p = buf;
	if (len < LEN_ETHER) {
		(*ap->eout)("truncated MAC header:");
		hexdump(ap, p, len);
		return;
	}
	(*ap->eout)("dst %s src %s proto 0x%x",
		eaddr(p + ETHER_DST, b1), eaddr(p + ETHER_SRC, b2),
		fretherprot(p, ETHER_PROT));
	p += LEN_ETHER;
	len -= LEN_ETHER;
	if (len < MINLEN_AOE) {
		(*ap->eout)("truncated AoE header:");
		hexdump(ap, p, len);
		return;
	}
	n = fraoechar(p, AOE_VF);
	bp = b1;
	if (n & AOEF_R)
		*bp++ = 'R';
	if (n & AOEF_E)
		*bp++ = 'E';
	*bp = 0;
	(*ap->eout)("AoEV%d vers+flags 0x%x [%s] error %d",
		aoeversion(n), n, b1, fraoechar(p, AOE_ERROR));
	n = fraoechar(p, AOE_COMMAND);
	(*ap->eout)("target %d.%d command %d [%s] tag 0x%lx",
		fraoeshort(p, AOE_MAJOR), fraoechar(p, AOE_MINOR),
		n, aoecmdname(n), (unsigned long)fraoelong(p, AOE_TAG));
	switch (n) {
	case AOECATA:
		if (len < MINLEN_AOEA) {
			(*ap->eout)("truncated ATA arg:");
			hexdump(ap, p + AOE_ARG, len - AOE_ARG);
			return;
		}
		n = fraoechar(p, AOEA_AF);
		bp = b1;
		if (n & AOEAFE)
			*bp++ = 'E';
		if (n & AOEAFD)
			*bp++ = 'D';
		if (n & AOEAFA)
			*bp++ = 'A';
		if (n & AOEAFW)
			*bp++ = 'W';
		*bp = 0;
		(*ap->eout)("ATA ef %02x nsec %02x cs %02x flags %02x [%s]",
			fraoechar(p, AOEA_EF), fraoechar(p, AOEA_NSEC),
			fraoechar(p, AOEA_CS), n, b1);
		if (n & AOEAFE) {	/* LBA48 */
			lba = fraoechar(p, AOEA_LBA5);
			lba <<= 8;
			lba |= fraoechar(p, AOEA_LBA4);
			lba <<= 8;
			lba |= fraoechar(p, AOEA_LBA3);
			lba <<= 8;
		} else {
			lba = (fraoechar(p, AOEA_LBA3) & 0xf);
			lba <<= 8;
		}
		lba |= fraoechar(p, AOEA_LBA2);
		lba <<= 8;
		lba |= fraoechar(p, AOEA_LBA1);
		lba <<= 8;
		lba |= fraoechar(p, AOEA_LBA0);
		(*ap->eout)("LBA %lld (0x%llx)", lba, lba);
		if (len > AOEA_DATA)
			(*ap->eout)("%d bytes of data included", len - AOEA_DATA);
		break;

	case AOECID:
		if (len < MINLEN_AOEI) {
			(*ap->eout)("truncated Query-Config arg:");
			hexdump(ap, p + AOE_ARG, len - AOE_ARG);
			return;
		}
		n = fraoeshort(p, AOEI_FV);
		m = fraoechar(p, AOEI_VCC);
		(*ap->eout)("AoEV%d maxbuf %d maxsec %d fwvers %d.%d",
			aoeiver(m), fraoeshort(p, AOEI_MAXBUF),
			fraoechar(p, AOEI_MAXSEC),
			aoefvmaj(n), aoefvmin(n));
		(*ap->eout)("cstr cmd %d [%s]", aoeicc(m), cstrname(aoeicc(m)));
		n = fraoeshort(p, AOEI_CLEN);
		if (n > len - AOEI_CSTR) {
			(*ap->eout)("string truncated: given len %d");
			n = len - AOE_ARG;
		}
		(*ap->eout)("cstr len %d [%.*s]", n, n, p + AOEI_CSTR);
		break;
	}
}

static char digits[16] = "0123456789abcdef";

static char *
eaddr(p, buf)
char *p;
char *buf;
{
	char *bp;
	int i;

	bp = buf;
	for (i = 0; i < LEN_EADDR; i++) {
		if (i)
			*bp++ = '-';
		*bp++ = digits[(*p>>4) & 0xf];
		*bp++ = digits[*p & 0xf];
		p++;
	}
	*bp = 0;
	return (buf);
}

static char *
aoecmdname(cmd)
int cmd;
{
	switch (cmd) {
	case AOECATA:
		return ("ATA-command");

	case AOECID:
		return ("Query-Config");

	default:
		return ("unknown");
	}
}

static char *
cstrname(cmd)
int cmd;
{
	switch (cmd) {
	case AOEICGET:
		return ("get");

	case AOEICMATCH:
		return ("match");

	case AOEICPREFIX:
		return ("match-prefix");

	case AOEICSET:
		return ("set");

	case AOEICFORCE:
		return ("force-set");

	default:
		return ("unknown");
	}
}

/*
 * dump raw hex bytes
 */
#define	BYTESPERLINE	16

static void
hexdump(ap, buf, len)
Aoe ap;
void *buf;
int len;
{
	char *p;
	char bbuf[BYTESPERLINE * 3];	/* xx(blank) or xx\0 */
	char *bp;
	int i;

	if (ap->eout == nullerr)
		return;
	p = buf;
	while (len > 0) {
		bp = bbuf;
		for (i = 0; i < BYTESPERLINE && len > 0; i++, len--) {
			if (i > 0)
				*bp++ = ' ';
			*bp++ = digits[(*p>>4) & 0xf];
			*bp++ = digits[*p & 0xf];
			p++;
		}
		*bp = 0;
		(*ap->eout)("  %s", bbuf);
	}
}

/*
 * avoid constant eout == null checks
 */
static void
nullerr(char *fmt, ...)
{
}
