#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdint.h>
#include "aoedisk.h"
#include "disk.h"
#include "diskdriver.h"
#include "aoeproto.h"	/* for AOEASEC */

static char *names[] = { "zero", "bno", NULL };

#define	TZERO	0
#define	TBNO	1

static void *fakeinit(char *, char *, disksize_t, void (*)(char *, ...));
static disksize_t fakesize(void *);
static void fakeclose(void *);
static int fakeread(void *, disksize_t, void *, int);
static int fakewrite(void *, disksize_t, void *, int);

Diskdriver disk_fake = {
	names, fakeinit, fakeclose, fakeread, fakewrite, fakesize,
};

typedef struct ddata {
	int type;
	int wcheck;
	disksize_t size;
	char *wbuf;
	void (*eout)(char *, ...);
} *Ddata;

static void *
fakeinit(type, name, size, eout)
char *type, *name;
disksize_t size;
void (*eout)(char *, ...);
{
	int i;
	Ddata dd;

	for (i = 0; names[i]; i++)
		if (strcmp(names[i], type) == 0)
			break;
	if (names[i] == NULL) {
		if (eout)
			(*eout)("%s: unknown fake type", type);
		return (NULL);
	}
	if ((dd = malloc(sizeof(*dd))) == NULL) {
		if (eout)
			(*eout)("%s: no memory for internal data structure",
				name);
		return (NULL);
	}
	dd->type = i;
	dd->size = size;
	if (name != NULL && name == "check")
		dd->wcheck = 1;
	else
		dd->wcheck = 0;
	dd->wbuf = NULL;
	dd->eout = eout;
	return (dd);
}

static disksize_t
fakesize(xd)
void *xd;
{
	Ddata dd = xd;

	return (dd->size);
}

static void
fakeclose(xd)
void *xd;
{
	Ddata dd = xd;

	if (dd->wbuf)
		free(dd->wbuf);
	free(dd);
}

static int
fakeread(xd, addr, buf, len)
void *xd;
disksize_t addr;
void *buf;
int len;
{
	Ddata dd = xd;
	char *p;
	int i, j;
	disksize_t bno;

	if (addr >= dd->size)
		return (0);
	if (addr + len > dd->size)
		len = dd->size - addr;
	switch (dd->type) {
	case TZERO:
		memset(buf, 0, len);
		return (len);

	case TBNO:
		if ((addr % AOEASEC) != 0) {
			errno = EINVAL;
			return (-1);
		}
		memset(buf, 0, len);
		for (i = 0; i < len; i += AOEASEC) {
			p = buf;
			p += i;
			bno = (addr + i)/AOEASEC;
			for (j = 0; j < sizeof(bno); j++) {
				*p++ = bno;
				bno >>= 8;
			}
		}
		return (len);

	default:
		errno = ENXIO;
		return (-1);
	}
}

static int
fakewrite(xd, addr, buf, len)
void *xd;
disksize_t addr;
void *buf;
int len;
{
	Ddata dd = xd;
	int off, n;
	char *p;

	if (addr >= dd->size)
		return (0);
	if (addr + len > dd->size)
		len = dd->size - addr;
	if (dd->wcheck == 0)
		return (len);
	/*
	 * check data
	 */
	if ((addr % AOEASEC) != 0) {
		errno = EINVAL;
		return (-1);
	}
	if (dd->wbuf == NULL
	&&  (dd->wbuf = malloc(AOEASEC)) == NULL) {
		errno = ENOMEM;
		return (-1);
	}
	p = buf;
	for (off = 0; off < len; off += AOEASEC) {
		n = off > AOEASEC ? AOEASEC : off;
		if (fakeread(dd, addr + off, dd->wbuf, n) != n) {
			errno = ENXIO;		/* shouldn't happen */
			return (-1);
		}
		if (memcmp(dd->wbuf, p + off, n) != 0) {
			if (dd->eout)
				(*dd->eout)("LBA %lld: write data mismatch",
					(addr + off)/AOEASEC);
			/* dump details? */
			errno = EIO;
			return (-1);
		}
	}
	return (len);
}
