#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdint.h>
#include <unistd.h>
#include <fcntl.h>	/* open flags */
#include <sys/types.h>
#include <sys/stat.h>
#include "aoedisk.h"
#include "disk.h"
#include "diskdriver.h"

static char *names[] = { "flat", NULL };
static void *flatinit(char *, char *, disksize_t, void (*)(char *, ...));
static disksize_t flatsize(void *);
static void flatclose(void *);
static int flatread(void *, disksize_t, void *, int);
static int flatwrite(void *, disksize_t, void *, int);

Diskdriver disk_flat = {
	names, flatinit, flatclose, flatread, flatwrite, flatsize,
};

typedef struct ddata {
	int fd;
	disksize_t size;
} *Ddata;

static disksize_t getsize(char *);

static void *
flatinit(type, name, size, eout)
char *type, *name;
disksize_t size;
void (*eout)(char *, ...);
{
	Ddata dd;
	int fd;

	if (name == NULL) {
		if (eout)
			(*eout)("disk type %s: name must be specified", type);
		return (NULL);
	}
	if (size == 0 && (size = getsize(name)) == 0) {
		if (eout)
			(*eout)("%s: cannot determine size", name);
		return (NULL);
	}
	if ((fd = open(name, O_RDWR|O_CREAT, 0666)) < 0) {
		if (eout)
			(*eout)("%s: cannot open or create: %s",
				name, strerror(errno));
		return (NULL);
	}
	if ((dd = malloc(sizeof(*dd))) == NULL) {
		if (eout)
			(*eout)("%s: no memory for internal data structure",
				name);
		close(fd);
		return (NULL);
	}
	dd->fd = fd;
	dd->size = size;
	return (dd);
}

static disksize_t
flatsize(xd)
void *xd;
{
	Ddata dd = xd;

	return (dd->size);
}

static void
flatclose(xd)
void *xd;
{
	Ddata dd = xd;

	close(dd->fd);
	free(dd);
}

static int
flatread(xd, addr, buf, len)
void *xd;
disksize_t addr;
void *buf;
int len;
{
	Ddata dd = xd;
	int n;

	if (addr >= dd->size)
		return (0);
	if (addr + len > dd->size)
		len = dd->size - addr;
	if (lseek(dd->fd, (off_t)addr, SEEK_SET) == (off_t)-1)
		return (-1);
	if ((n = read(dd->fd, buf, len)) < 0)
		return (n);
	if (n < len)
		memset((char *)buf + n, 0, len - n);
	return (len);
}

static int
flatwrite(xd, addr, buf, len)
void *xd;
disksize_t addr;
void *buf;
int len;
{
	Ddata dd = xd;

	if (addr >= dd->size)
		return (0);
	if (addr + len > dd->size)
		len = dd->size - addr;
	if (lseek(dd->fd, (off_t)addr, SEEK_SET) == (off_t)-1)
		return (-1);
	return (write(dd->fd, buf, len));
}

/*
 * called only if the size wasn't given explicitly
 * --need a system-dependent bit to probe
 * size of a raw disk partition?
 * --maybe this is inherently system-dependent anyway?
 */
static disksize_t
getsize(name)
char *name;
{
	struct stat st;

	if (stat(name, &st) < 0)
		return (0);
	if (!S_ISREG(st.st_mode))
		return (0);
	return (st.st_size);
}
