/*
 * dispatcher;
 * real work done in per-type drivers
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include "aoedisk.h"
#include "disk.h"
#include "diskdriver.h"

struct disk {
	Diskdriver *dr;
	void *dd;
};

/*
 * driver table
 */
extern Diskdriver disk_fake;
extern Diskdriver disk_flat;

static Diskdriver *drivers[] = {
	&disk_flat,
	&disk_fake,
	NULL
};

static Diskdriver *finddriver(char *);

Disk diskinit(type, file, size, eout)
char *type;
char *file;
disksize_t size;
void (*eout)(char *, ...);
{
	Disk dp;
	Diskdriver *dr;
	void *dd;

	if ((dr = finddriver(type)) == NULL) {
		if (eout)
			(*eout)("%s: unknown disk type", type);
		return (NULL);
	}
	if (type == NULL)		/* clean up default */
		type = dr->names[0];
	if ((dd = (*dr->init)(type, file, size, eout)) == NULL)
		return (NULL);
	if ((dp = malloc(sizeof(struct disk))) == NULL) {
		if (eout)
			(*eout)("no memory for Disk");
		(*dr->close)(dd);
		return (NULL);
	}
	dp->dr = dr;
	dp->dd = dd;
	return (dp);
}

disksize_t
disksize(dp)
Disk dp;
{
	return ((*dp->dr->size)(dp->dd));
}

static Diskdriver *
finddriver(type)
char *type;
{
	Diskdriver **rp;
	char **p;

	if (type == NULL)
		return (drivers[0]);
	for (rp = drivers; *rp; rp++)
		for (p = (*rp)->names; *p; p++)
			if (strcmp(type, *p) == 0)
				return (*rp);
	return (NULL);
}

int
diskclose(dp)
Disk dp;
{
	(*dp->dr->close)(dp->dd);
	free(dp);
}

int
diskread(dp, pos, buf, len)
Disk dp;
disksize_t pos;
void *buf;
int len;
{
	return ((*dp->dr->read)(dp->dd, pos, buf, len));
}

int
diskwrite(dp, pos, buf, len)
Disk dp;
disksize_t pos;
void *buf;
int len;
{
	return ((*dp->dr->write)(dp->dd, pos, buf, len));
}
