/*
 * Copyright 2005 Coraid, Inc.
 * See accompanying file doc/LICENSE for conditions
 */

/*
 * messages exchanged between aoecomm
 * and its user-mode handler;
 * also used for commands sent to aoectl
 * integers use the macros defined in aoeproto.h
 */

/*
 * general header
 */

#define	AC_TYPE		0	/* (c) message type */
#define	AC_ZERO		1	/* (c) ignored */
#define	AC_LEN		2	/* (s) length of entire message */
#define	AC_CHANNEL	4	/* (s) channel number */

#define	LEN_ACHEAD	6	/* length of the header */

#define	ACHWILD	30000		/* global/wildcard channel number */

/*
 * types
 */
#define	ACINITCHAN	0x01	/* channel init */
#define	ACSEND		0x02	/* send to network */
#define	ACDEVENAB	0x03	/* mark device active or inactive */

#define	ACINITACK	0x41	/* reply to INITCHAN */
#define	ACLOG		0x42	/* log message to monitor */

/*
 * body of ACINITCHAN
 */

#define	AC_PROTO	(LEN_ACHEAD)	/* (s) Ethernet proto used */
#define	AC_COOKIE	(LEN_ACHEAD+2)	/* (l) confirmation cookie */
#define	AC_LOCADDR	(LEN_ACHEAD+6)	/* (n) local MAC address */
#define	AC_MAXDATA	(AC_LOCADDR+LEN_EADDR) /* (s) max ATA data segment */

#define	LEN_ACINITCHAN	(AC_MAXDATA+2)

/*
 * body of ACINITACK
 */
#define	AC_STATUS	(LEN_ACHEAD)	/* (c) 0 or an errno value */

#define	LEN_ACINITACK	(LEN_ACHEAD+1)

/*
 * body of ACSEND and ACLOG
 */

#define	AC_LTYPE	(LEN_ACHEAD)	/* (c) ACLOG type */
#define	AC_LJUNK	(LEN_ACHEAD+1)	/* (c) ACLOG MBZ */
#define	AC_NLOST	(LEN_ACHEAD+2)	/* (s) ACLOG missing messages */
#define	AC_MESSAGE	(LEN_ACHEAD+4)	/* (n) datagram */

#define	ACSENDMAX	(1500+LEN_ETHER) /* longest datagram allowed */
#define	ACSENDMIN	LEN_ETHER	/* shortest--at least MAC header! */

#define	MINLEN_ACSEND	(AC_MESSAGE+ACSENDMIN)
#define	LEN_ACSEND	(AC_MESSAGE+ACSENDMAX)
#define	LEN_ACLOG	LEN_ACSEND

/*
 * AC_LTYPE codes
 */
#define	ACLUNSOL	0	/* unsolicited AoE message */
#define	ACLILLFORM	1	/* ill-formed AoE message */
#define	ACLSENDFAIL	2	/* couldn't send */
#define	ACLTIMEOUT	3	/* sent this but received no reply */
#define	ACLAOEERR	4	/* received AoE error report */
#define	ACLATAERR	5	/* received ATA error report */

/*
 * body of ACDEVENAB
 */
#define	AC_MAJOR	(LEN_ACHEAD)	/* (s) AoE major number */
#define	AC_MINOR	(LEN_ACHEAD+2)	/* (c) AoE minor number */
#define	AC_ENABCMD	(LEN_ACHEAD+3)	/* (c) command */

#define	LEN_ACDEVENAB	(AC_ENABCMD+1)

#define	ADDISAB	0
#define	ADENAB	1
#define	ADQUERY	2

#define	LEN_ACMAXSHORT	LEN_ACINITCHAN	/* max sans any datagram */
#define	LEN_ACMAX	LEN_ACSEND	/* max length of header+data */
