/*
 * Copyright 2005 Coraid, Inc.
 * See accompanying file doc/LICENSE for conditions
 */

/*
 * ATA-over-Ethernet protocol
 * defined as byte arrays, not embedded integers,
 * to escape alignment and byte-order issues
 *
 * based on the AoEr10 document
 */

#define	EPROTO_AOE	0x88a2	/* standard Ethernet protocol */

/*
 * access macros
 */

#define	fraoechar(p, off)	((unsigned char *)(p))[(off)]
#define	fraoeshort(p, off)	((((unsigned char *)(p))[(off)]<<8)|((unsigned char *)(p))[(off)+1])
#define	fraoelong(p, off)	((((unsigned char *)(p))[(off)]<<24)|(((unsigned char *)(p))[(off)+1]<<16)|(((unsigned char *)(p))[(off)+2]<<8)|((unsigned char *)(p))[(off)+3])

#define	toaoechar(p, off, v)	(p)[(off)] = (v)&0xff
#define	toaoeshort(p, off, v)	(p)[(off)] = ((v)>>8)&0xff, (p)[(off)+1] = ((v)&0xff)
#define	toaoelong(p, off, v)	(p)[(off)]=((v)>>24)&0xff, (p)[(off)+1]=((v)>>16)&0xff, (p)[(off)+2]=((v)>>8)&0xff, (p)[(off)+3]=(v)&0xff

/*
 * AoE common header
 * --MAC header omitted
 */

#define	AOE_VF		0	/* (c) version, flags */
#define	AOE_ERROR	1	/* (c) error code */
#define	AOE_MAJOR	2	/* (s) major (shelf) id */
#define	AOE_MINOR	4	/* (c) minor (slot) id */
#define	AOE_COMMAND	5	/* (c) command code */
#define	AOE_TAG		6	/* (l) unique ID */
#define	AOE_ARG		10	/* ... see below for format */

#define	MINLEN_AOE	AOE_ARG	/* shortest valid AOE message */

#define	AOEMAJWILD	0xffff
#define	AOEMINWILD	0xff

#define	AOEMAJMAX	0xffff
#define	AOEMINMAX	0xff
/*
 * contents of AOE_VF
 */
#define	AOEV1		1	/* current standard version */

#define	AOEF_VER	0xf0	/* mask for version */
#define	AOEF_R		0x8	/* this is a response */
#define	AOEF_E		0x4	/* error occurred, AOE_ERROR valid */

#define	aoemkvf(f)	((AOEV1<<4)|(f))
#define	aoeversion(vf)	(((vf)&AOEF_VER) >> 4)

/*
 * error codes
 */
#define	AOEOK		0	/* no error */
#define	AOEBADCMD	1	/* invalid command */
#define	AOEBADARG	2	/* bad argument */
#define	AOEBADDEV	3	/* device unavailable */
#define	AOEBADSTR	4	/* config string already set */
#define	AOEBADVER	5	/* wrong protocol version */

/*
 * command codes
 */
#define	AOECATA	0	/* ATA command */
#define	AOECID	1	/* query ID info */

/*
 * arg for AOECATA,
 * expressed as offsets into overall message
 */
#define	AOEA_AF		(AOE_ARG+0)	/* (c) AFlags */
#define	AOEA_EF		(AOE_ARG+1)	/* (c) error/feature */
#define	AOEA_NSEC	(AOE_ARG+2)	/* (c) sector count */
#define	AOEA_CS		(AOE_ARG+3)	/* (c) cmd/status */
#define	AOEA_LBA0	(AOE_ARG+4)	/* (c) */
#define	AOEA_LBA1	(AOE_ARG+5)	/* (c) */
#define	AOEA_LBA2	(AOE_ARG+6)	/* (c) */
#define	AOEA_LBA3	(AOE_ARG+7)	/* (c) */
#define	AOEA_LBA4	(AOE_ARG+8)	/* (c) */
#define	AOEA_LBA5	(AOE_ARG+9)	/* (c) */
/* bytes 10, 11 reserved */
#define	AOEA_DATA	(AOE_ARG+12)	/* 512 or 1024 bytes */

#define	MINLEN_AOEA	AOEA_DATA

#define	AOEASEC		512
#define	AOEAMAXSEC	2	/* default */

/*
 * AFlag values
 */
#define	AOEAFE	0x40
#define	AOEAFD	0x10
#define	AOEAFA	0x2
#define	AOEAFW	0x1

/*
 * arg for AOECID
 */
#define	AOEI_MAXBUF	(AOE_ARG+0)	/* (s) max queued buffers */
#define	AOEI_FV		(AOE_ARG+2)	/* (s) firmware version */
#define	AOEI_MAXSEC	(AOE_ARG+4)	/* (c) max data sectors per command */
#define	AOEI_VCC	(AOE_ARG+5)	/* (c) AOE version, Ccmd */
#define	AOEI_CLEN	(AOE_ARG+6)	/* (s) config string len */
#define	AOEI_CSTR	(AOE_ARG+8)	/* (N) config string */

#define	MINLEN_AOEI	AOEI_CSTR

#define	AOEMAXSTR	1024		/* longest valid config string */

#define	aoeimkvcc(v, cc) (((v)<<4)|(cc&0xf))
#define	aoeiver(vcc)	(((vcc)>>4)&0xf)
#define	aoeicc(vcc)	((vcc)&0xf)

/*
 * config string subcommands
 */
#define	AOEICGET	0	/* fetch string */
#define	AOEICMATCH	1	/* match entire string */
#define	AOEICPREFIX	2	/* match prefix */
#define	AOEICSET	3	/* set if not empty */
#define	AOEICFORCE	4	/* set even if empty */

#define	aoemkfv(maj, min) ((((maj)&0xf)<<12)|((min)&0xfff))
#define	aoefvmaj(v)	(((v)>>12)&0xf)
#define	aoefvmin(v)	((v)&0xfff)
