/*
 * Copyright 2005 Coraid, Inc.
 * See accompanying file doc/LICENSE for conditions
 */

/*
 * standard ATA register hardware defs
 * mostly the subset used by Solaris etherdrive
 */

/*
 * status register
 */
#define	ATAST_ERR	0x01
#define	ATAST_IDX	0x02
#define	ATAST_CORR	0x04
#define	ATAST_DRQ	0x08
#define	ATAST_DSC	0x10
#define	ATAST_DF	0x20
#define	ATAST_DRDY	0x40
#define	ATAST_BSY	0x80

/*
 * device/head register
 */
#define	ATAHD_LBA	0x40	/* LBA addressing */
#define	ATAHD_DEV	0x10	/* second device */
#define	ATAHD_MB1	0xa0	/* bits that must be one */
#define	ATAHD_ADDR	0x0f	/* head number or LBA(27:24) */

/*
 * error register
 */
#define	ATAER_AMNF	0x01
#define	ATAER_TK0NF	0x02
#define	ATAER_ABRT	0x04
#define	ATAER_MCR	0x08
#define	ATAER_IDNF	0x10
#define	ATAER_MC	0x20
#define	ATAER_UNC	0x40

#define	ATASEC	512	/* presumed sector size */

/*
 * command codes
 */

#define	ATANOP		0x00
#define	ATAIDENTIFY	0xec
#define	ATARDSEC	0x20
#define	ATARDXSEC	0x24
#define	ATAWRSEC	0x30
#define	ATAWRXSEC	0x34

/*
 * data-type access macros
 */
#define	frataword(p, o)	((((unsigned char *)(p))[(o)+1]<<8)|\
				 ((unsigned char *)(p))[(o)+0])
#define	fratalong(p, o)	((((unsigned char *)(p))[(o)+3]<<24)|\
				 (((unsigned char *)(p))[(o)+2]<<16)|\
				 (((unsigned char *)(p))[(o)+1]<<8)|\
				 ((unsigned char *)(p))[(o)+0])
#define	fratall(p, o)	(((unsigned long long)(((unsigned char *)(p))[(o)+7])<<56)|\
			 ((unsigned long long)(((unsigned char *)(p))[(o)+6])<<48)|\
			 ((unsigned long long)(((unsigned char *)(p))[(o)+5])<<40)|\
			 ((unsigned long long)(((unsigned char *)(p))[(o)+4])<<32)|\
			 ((unsigned long long)(((unsigned char *)(p))[(o)+3])<<24)|\
			 ((unsigned long long)(((unsigned char *)(p))[(o)+2])<<16)|\
			 ((unsigned long long)(((unsigned char *)(p))[(o)+1])<<8)|\
			 ((unsigned long long)((unsigned char *)(p))[(o)+0]))

#define	toataword(p, o, v)	(p)[(o)+1] = ((v)>>8)&0xff,\
				(p)[(o)+0] = ((v)&0xff)
#define	toatalong(p, o, v)	(p)[(o)+3]=((v)>>24)&0xff,\
				(p)[(o)+2]=((v)>>16)&0xff,\
				(p)[(o)+1]=((v)>>8)&0xff,\
				(p)[(o)+0]=(v)&0xff
#define	toatall(p, o, v)	(p)[(o)+7]=((v)>>54)&0xff,\
				(p)[(o)+6]=((v)>>48)&0xff,\
				(p)[(o)+5]=((v)>>40)&0xff,\
				(p)[(o)+4]=((v)>>32)&0xff, \
				(p)[(o)+3]=((v)>>24)&0xff,\
				(p)[(o)+2]=((v)>>16)&0xff,\
				(p)[(o)+1]=((v)>>8)&0xff,\
				(p)[(o)+0]=(v)&0xff

/*
 * ATAIDENTIFY
 * n.b. byte, not word, offsets
 */
#define	ATAID_TYPE	(0*2)
#define	ATAID_LCYL	(1*2)
#define	ATAID_LHD	(3*2)
#define	ATAID_LSEC	(6*2)
#define	ATAID_SERIAL	(10*2)
#define	 LEN_ATASERIAL	20
#define	ATAID_FWVER	(23*2)
#define	 LEN_ATAFWVER	8
#define	ATAID_MODEL	(27*2)
#define	 LEN_ATAMODEL	40
#define	ATAID_MAXMULT	(47*2)
#define	ATAID_CAPAB	(49*2)
#define	ATAID_CURCYL	(54*2)
#define	ATAID_CURHD	(55*2)
#define	ATAID_CURSEC	(56*2)
#define	ATAID_CAPACITY	(57*2)	/* long */
#define	ATAID_MAX28LBN	(60*2)	/* long */
#define	ATAID_CMDSUPP	(82*2)	/* long */
#define	ATAID_CMDENAB	(85*2)	/* long */
#define	ATAID_MAX48LBN	(100*2)	/* longlong */

#define	ATAIDSECS	1	/* sectors of ID info */

#define	ATAMAX28LBN	0x0fffffff	/* biggest valid 28-bit LBN */
#define	ATAMAX48LBN	0xffffffffffff	/* biggest valid 48-bit LBN */

/*
 * ATAID_TYPE: general configuration
 */
#define	ATAT_PI	0x8000		/* ATAPI, not ATA */
#define	ATAT_RM	0x0080		/* removable media */
#define	ATAT_RI	0x0004		/* response incomplete */

/*
 * ATAID_CAPAB: (some) capabilities
 */
#define	ATAC_LBA 0x200		/* LBA (at least 28) supported */

/*
 * ATAID_CMDSUPP, CMDENAB: command set support
 */
#define	ATAC_LBA48	0x04000000	/* LBA48 support */
#define	ATAC_NOP	0x00004000	/* NOP command supported */
