/*
 * Copyright 2006 Coraid, Inc.
 * See accompanying file doc/LICENSE for conditions
 */

/*
 * AoE disk driver
 * ATA-specific disk ioctls
 */

/*
 * these includes taken from one of the sample drivers
 * Sun's Writing Device Drivers doesn't mention them,
 * but without at least some of them the driver won't compile
 */
#include <sys/types.h>
#include <sys/file.h>
#include <sys/errno.h>
#include <sys/uio.h>
#include <sys/open.h>
#include <sys/cred.h>
#include <sys/kmem.h>
#include <sys/conf.h>
#include <sys/ksynch.h>
#include <sys/modctl.h>
#include <sys/stat.h>
#include <sys/buf.h>

/*
 * these are mentioned in the book
 */
#include <sys/ddi.h>
#include <sys/sunddi.h>

/*
 * these are needed because it's a disk driver
 * or because we may keep statistics
 */
#include <sys/dkio.h>
#include <sys/vtoc.h>
#include <sys/kstat.h>
#include <sys/dktp/dadkio.h>
#include "dadkio32.h"		/* what Sun should supply but don't */

/*
 * these are specific to AoE
 */
#include "etherproto.h"		/* for Adrive */
#include "ata.h"		/* for Adrive */
#include "aoed.h"

static void dadkio32todadkio(struct dadkio32_rwcmd *, struct dadkio_rwcmd *);
static void dadkiotodadkio32(struct dadkio_rwcmd *, struct dadkio32_rwcmd *);
static int dadkio(Adrive *, struct dadkio_rwcmd *, int);
static int dadkmodser(Adrive *, int, int, caddr_t, int *);

int
dadkioctl(ep, cmd, arg, mode)
Adrive *ep;
int cmd;
intptr_t arg;
int mode;
{
	int err;
	caddr_t obuf;
	int len;
	union {
		struct dadkio_rwcmd k;
		struct dadk_ioc_string ms;
	} argval;
	union {
		struct dadkio32_rwcmd k32;
		struct dadk_ioc_string32 m32;
	} arg2;
#define	Ik	argval.k
#define	Ik32	arg2.k32
#define	Ims	argval.ms
#define	Im32	arg2.m32

	switch (cmd) {
	case DIOCTL_RWCMD:
		switch (mode & FMODELS) {
		case DDI_MODEL_LP64:
			if (ddi_copyin((caddr_t)arg, &Ik, sizeof(Ik), mode) < 0)
				return (EFAULT);
			break;

		case DDI_MODEL_ILP32:
			if (ddi_copyin((caddr_t)arg, &Ik32, sizeof(Ik32), mode) < 0)
				return (EFAULT);
			dadkio32todadkio(&Ik32, &Ik);
			break;

		default:
			return (EINVAL);
		}
		if ((err = dadkio(ep, &Ik, mode)) != 0)
			return (err);
		switch (mode & FMODELS) {
		case DDI_MODEL_LP64:
			if (ddi_copyout(&Ik, (caddr_t)arg, sizeof(Ik), mode) < 0)
				return (EFAULT);
			break;

		case DDI_MODEL_ILP32:
			dadkiotodadkio32(&Ik, &Ik32);
			if (ddi_copyout(&Ik32, (caddr_t)arg, sizeof(Ik32), mode) < 0)
				return (EFAULT);
			break;
		}
		return (0);

	case DIOCTL_GETMODEL:
	case DIOCTL_GETSERIAL:
		switch (mode & FMODELS) {
		case DDI_MODEL_LP64:
			if (ddi_copyin((caddr_t)arg, &Ims, sizeof(Ims), mode) < 0)
				return (EFAULT);
			len = Ims.is_size;
			obuf = Ims.is_buf;
			break;

		case DDI_MODEL_ILP32:
			if (ddi_copyin((caddr_t)arg, &Im32, sizeof(Im32), mode) < 0)
				return (EFAULT);
			len = Im32.is_size;
			obuf = (caddr_t)Im32.is_buf;
			break;

		default:
			return (EINVAL);
		}
		if ((err = dadkmodser(ep, cmd, mode, obuf, &len)) != 0)
			return (err);
		switch (mode & FMODELS) {
		case DDI_MODEL_LP64:
			Ims.is_size = len;
			if (ddi_copyout(&Ims, (caddr_t)arg, sizeof(Ims), mode) < 0)
				return (EFAULT);
			break;

		case DDI_MODEL_ILP32:
			Im32.is_size = len;
			if (ddi_copyout(&Im32, (caddr_t)arg, sizeof(Im32), mode) < 0)
				return (EFAULT);
			break;
		}
		return (0);

	default:
		return (ENOTTY);
	}
	/* NOTREACHED */
}

static void
dadkio32todadkio(d32, d64)
struct dadkio32_rwcmd *d32;
struct dadkio_rwcmd *d64;
{
	d64->cmd = d32->cmd;
	d64->flags = d32->flags;
	d64->blkaddr = d32->blkaddr;
	d64->buflen = d32->buflen;
	d64->bufaddr = (caddr_t)(unsigned long)d32->bufaddr;
	d64->status.status = d32->status.status;
	d64->status.resid = d32->status.resid;
	d64->status.failed_blk_is_valid = d32->status.failed_blk_is_valid;
	d64->status.failed_blk = d32->status.failed_blk;
	d64->status.fru_code_is_valid = d32->status.fru_code_is_valid;
	d64->status.fru_code = d32->status.fru_code;
	bcopy(d32->status.add_error_info, d64->status.add_error_info,
		sizeof(d64->status.add_error_info));
}

static void
dadkiotodadkio32(d64, d32)
struct dadkio_rwcmd *d64;
struct dadkio32_rwcmd *d32;
{
	d32->cmd = d64->cmd;
	d32->flags = d64->flags;
	d32->blkaddr = d64->blkaddr;
	d32->buflen = d64->buflen;
	d32->bufaddr = (uint32_t)(unsigned long)d64->bufaddr;
	d32->status.status = d64->status.status;
	d32->status.resid = d64->status.resid;
	d32->status.failed_blk_is_valid = d64->status.failed_blk_is_valid;
	d32->status.failed_blk = d64->status.failed_blk;
	d32->status.fru_code_is_valid = d64->status.fru_code_is_valid;
	d32->status.fru_code = d64->status.fru_code;
	bcopy(d64->status.add_error_info, d32->status.add_error_info,
		sizeof(d32->status.add_error_info));
}

static int
dadkio(ep, d, mode)
Adrive *ep;
struct dadkio_rwcmd *d;
int mode;
{
	int err, resid;

	switch (d->cmd) {
	case DADKIO_RWCMD_READ:
		mutex_exit(&ep->lock);
		err = adrwuser(ep, B_READ, d->blkaddr,
			d->bufaddr, d->buflen, &resid);
		mutex_enter(&ep->lock);
		break;

	case DADKIO_RWCMD_WRITE:
		mutex_exit(&ep->lock);
		err = adrwuser(ep, B_WRITE, d->blkaddr,
			d->bufaddr, d->buflen, &resid);
		mutex_enter(&ep->lock);
		break;

	default:
		return (EINVAL);
	}
	bzero(&d->status, sizeof(d->status));
	switch (err) {
	case 0:
		d->status.status = DADKIO_STAT_NO_ERROR;
		break;

	case ETIME:
		d->status.status = DADKIO_STAT_TIMED_OUT;
		break;

	case EINVAL:
		d->status.status = DADKIO_STAT_ILLEGAL_ADDRESS;
		break;

	default:
		d->status.status = DADKIO_STAT_MEDIUM_ERROR;
		break;
	}
	d->status.resid = resid;
	return (0);
}

static int
dadkmodser(ep, cmd, mode, obuf, olp)
Adrive *ep;
int cmd;
int mode;
caddr_t obuf;
int *olp;
{
	int len;
	char *s;

	switch (cmd) {
	case DIOCTL_GETMODEL:
		s = ep->atamodel;
		break;

	case DIOCTL_GETSERIAL:
		s = ep->ataserial;
		break;

	default:
		return (EINVAL);
	}
	len = strlen(s);
	if (*olp > len)
		*olp = len;
	if (ddi_copyout(s, obuf, *olp, mode) < 0)
		return (EFAULT);
	return (0);
}
