/*
 * Copyright 2005 Coraid, Inc.
 * See accompanying file doc/LICENSE for conditions
 */

/*
 * include only as needed for debugging
 */


/*
 * these includes taken from one of the sample drivers
 * Sun's Writing Device Drivers doesn't mention them,
 * but without at least some of them the driver won't compile
 */
#include <sys/types.h>
#include <sys/file.h>
#include <sys/errno.h>
#include <sys/uio.h>
#include <sys/open.h>
#include <sys/cred.h>
#include <sys/kmem.h>
#include <sys/conf.h>
#include <sys/ksynch.h>
#include <sys/modctl.h>
#include <sys/stat.h>
#include <sys/buf.h>

/*
 * these are mentioned in the book
 */
#include <sys/ddi.h>
#include <sys/sunddi.h>

/*
 * these are needed because it's a disk driver
 * or because we may keep statistics
 */
#include <sys/dkio.h>
#include <sys/vtoc.h>
#include <sys/kstat.h>

/*
 * these are specific to AoE
 */
#include "etherproto.h"	/* for Adrive */
#include "ata.h"
#include "aoed.h"

typedef struct {
	int seq;
	int tag;
	void *p0;
	int i0, i1;
} Trace;

#define	NTRACE	1024

static int adtp;
static Trace adtrace[NTRACE];
static int adtseq;

static int tlockinit;
static kmutex_t tlock;

void
trace(tag, p0, i0, i1)
int tag;
void *p0;
int i0, i1;
{
	Trace *tp;

	if (tlockinit == 0) {
		tlockinit = 1;
		mutex_init(&tlock, NULL, MUTEX_DRIVER, NULL);
	}
	mutex_enter(&tlock);
	tp = &adtrace[adtp++];
	if (adtp >= NTRACE)
		adtp = 0;
	tp->seq = adtseq++;
	tp->tag = tag;
	tp->p0 = p0;
	tp->i0 = i0;
	tp->i1 = i1;
	mutex_exit(&tlock);
}
