/*
 * Copyright 2006 Coraid, Inc.
 * See accompanying file doc/LICENSE for conditions
 */

/*
 * AoE disk driver
 * imitation SCSI ioctls--
 * needed to work around bugs in format(1M)
 */

/*
 * these includes taken from one of the sample drivers
 * Sun's Writing Device Drivers doesn't mention them,
 * but without at least some of them the driver won't compile
 */
#include <sys/types.h>
#include <sys/file.h>
#include <sys/errno.h>
#include <sys/uio.h>
#include <sys/open.h>
#include <sys/cred.h>
#include <sys/kmem.h>
#include <sys/conf.h>
#include <sys/ksynch.h>
#include <sys/modctl.h>
#include <sys/stat.h>
#include <sys/buf.h>

/*
 * these are mentioned in the book
 */
#include <sys/ddi.h>
#include <sys/sunddi.h>

/*
 * these are needed because it's a disk driver
 * or because we may keep statistics
 */
#include <sys/dkio.h>
#include <sys/vtoc.h>
#include <sys/kstat.h>
#include <sys/scsi/scsi.h>
#include <sys/scsi/targets/stdef.h>

/*
 * these are specific to AoE
 */
#include "etherproto.h"		/* for Adrive */
#include "ata.h"		/* for Adrive */
#include "aoed.h"

static int fakeinq(Adrive *, struct uscsi_cmd *, int);
static int fakecap(Adrive *, struct uscsi_cmd *, int);
static int fakeblim(Adrive *, struct uscsi_cmd *, int);
static int putbuf(struct uscsi_cmd *, void *, int, int);

int
aduscsi(ep, cmd, arg, mode)
Adrive *ep;
int cmd;
intptr_t arg;
int mode;
{
	struct uscsi_cmd uc;
	struct uscsi_cmd32 uc32;
	union scsi_cdb cdb;
	int err;

	if (cmd != USCSICMD)
		return (ENOTTY);
	switch (mode & FMODELS) {
	case DDI_MODEL_LP64:
		if (ddi_copyin((caddr_t)arg, &uc, sizeof(uc), mode) != 0)
			return (EFAULT);
		break;

	case DDI_MODEL_ILP32:
		if (ddi_copyin((caddr_t)arg, &uc32, sizeof(uc32), mode) != 0)
			return (EFAULT);
		/* extra parens required by buggy Sun macro */
		uscsi_cmd32touscsi_cmd((&uc32), (&uc));
		break;

	default:
		return (EINVAL);
	}
	if (ddi_copyin(uc.uscsi_cdb, &cdb, CDB_GROUP0, mode) != 0)
		return (EFAULT);
	switch (GETCMD(&cdb)) {
	case SCMD_TEST_UNIT_READY:
		uc.uscsi_status = STATUS_GOOD;
		break;

	case SCMD_INQUIRY:
		if ((err = fakeinq(ep, &uc, mode)) != 0)
			return (err);
		uc.uscsi_status = STATUS_GOOD;
		break;

	case SCMD_READ_CAPACITY:
		if ((err = fakecap(ep, &uc, mode)) != 0)
			return (err);
		uc.uscsi_status = STATUS_GOOD;
		break;

	case SCMD_READ_BLKLIM:
		if ((err = fakeblim(ep, &uc, mode)) != 0)
			return (err);
		uc.uscsi_status = STATUS_GOOD;
		break;

	default:
		admsg(NODEV, ep, "unexpected USCSI command 0x%02x\n",
			GETCMD(&cdb));
		return (EINVAL);
	}
	switch (mode & FMODELS) {
	case DDI_MODEL_LP64:
		if (ddi_copyout(&uc, (caddr_t)arg, sizeof(uc), mode) != 0)
			return (EFAULT);
		break;

	case DDI_MODEL_ILP32:
		/* extra parens required by buggy Sun macro */
		uscsi_cmdtouscsi_cmd32((&uc), (&uc32));
		if (ddi_copyout(&uc32, (caddr_t)arg, sizeof(uc32), mode) != 0)
			return (EFAULT);
		break;
	}
	return (0);
}

/*
 * return fake INQUIRY data
 */
static int
fakeinq(ep, up, mode)
Adrive *ep;
struct uscsi_cmd *up;
int mode;
{
	struct scsi_inquiry iq;
	int iqlen;
	char *p, *q;

	if ((up->uscsi_flags & USCSI_READ) == 0)
		return (ENXIO);
	bzero(&iq, sizeof(iq));
	iqlen = sizeof(iq);
	iq.inq_dtype = DTYPE_DIRECT;
	iq.inq_len = iqlen - offsetof(struct scsi_inquiry, inq_len) + 1;
	/*
	 * if model fits in atamodel, put in pid
	 * else cram into vid + pid
	 * trailing blanks have already been trimmed;
	 * trim leading blanks just to be safe
	 */
	p = ep->atamodel;
	for (p = ep->atamodel; *p == ' '; p++)
		;
	if (strlen(p) > sizeof(iq.inq_pid)) {
		bcopy(p, iq.inq_vid, sizeof(iq.inq_vid));
		p += sizeof(iq.inq_vid);
	}
	strncpy(iq.inq_pid, p, sizeof(iq.inq_pid));
	for (p = iq.inq_vid, q = p + sizeof(iq.inq_vid); p < q && *p; p++)
		;
	while (p < q)
		*p++ = ' ';
	for (p = iq.inq_pid, q = p + sizeof(iq.inq_pid); p < q && *p; p++)
		;
	while (p < q)
		*p++ = ' ';
	/*
	 * vendor-specific string,
	 * but put serial number anyway
	 */
	p = ep->ataserial;
	for (p = ep->ataserial; *p == ' '; p++)
		;
	strncpy(iq.inq_serial, p, sizeof(iq.inq_serial));
	for (p = iq.inq_serial, q = p + sizeof(iq.inq_serial); p < q && *p; p++)
		;
	while (p < q)
		*p++ = ' ';
	if (putbuf(up, &iq, iqlen, mode) < 0)
		return (EFAULT);
	up->uscsi_resid = up->uscsi_buflen - iqlen;
	return (0);
}

/*
 * return fake READ CAPACITY data
 * -- this is the 32-bit-max version
 * -- beware byte-order issues;
 * on SPARC it happens to come out right
 */

#define	MAXCAP	0xffffffffLLU
static int
fakecap(ep, up, mode)
Adrive *ep;
struct uscsi_cmd *up;
int mode;
{
	struct scsi_capacity sc;

	if ((up->uscsi_flags & USCSI_READ) == 0)
		return (ENXIO);
	if (ep->nblocks < MAXCAP)
		sc.capacity = ep->nblocks;
	else
		sc.capacity = MAXCAP;
	sc.lbasize = DEV_BSIZE;
	if (putbuf(up, &sc, sizeof(sc), mode) < 0)
		return (EFAULT);
	return (0);
}

/*
 * the SCSI standard defines READ BLOCK LIMITS
 * only for tape drives;
 * it is a mystery why format calls it for disks
 */

static int
fakeblim(ep, up, mode)
Adrive *ep;
struct uscsi_cmd *up;
int mode;
{
	struct read_blklim blim;

	if ((up->uscsi_flags & USCSI_READ) == 0)
		return (ENXIO);
	bzero(&blim, sizeof(blim));
	blim.max_hi = DEV_BSIZE >> 16;
	blim.max_mid = DEV_BSIZE >> 8;
	blim.max_lo = DEV_BSIZE;
	blim.min_hi = DEV_BSIZE >> 8;
	blim.min_lo = DEV_BSIZE;
	if (putbuf(up, &blim, sizeof(blim), mode) < 0)
		return (EFAULT);
	return (0);
}

static int
putbuf(up, buf, len, mode)
struct uscsi_cmd *up;
void *buf;
int len;
int mode;
{
	if (len > up->uscsi_buflen)
		len = up->uscsi_buflen;
	if (ddi_copyout(buf, up->uscsi_bufaddr, len, mode) < 0)
		return (-1);
	up->uscsi_resid = up->uscsi_buflen - len;
	return (1);
}
