.TL
Solaris ATA-over-Ethernet (AoE) Installation and Operation Guide
.AU
Norman Wilson
2007 August 19
.SH
1.
Introduction
.PP
This manual describes
Version 1.4
of the Solaris ATA-over-Ethernet subsystem,
which makes
EtherDrive\(rg\u\s-11\s+1\d Storage Blades
available as disk devices.
.FS
1.
EtherDrive\(rg
is a registered trademark of
Coraid, Inc.
.FE
It explains how to
install
and
configure
the AoE package;
how the disks appear to the rest of the Solaris system;
and some quirks and implementation details.
It is not a manual on how to install EtherDrive hardware.
.PP
Parts of the AoE subsystem work differently
if the Solaris System Management Facility (SMF)
is in use
(\f2svcs\fP,
\f2svcadm\fP,
and
\f2svccfg\fP(1M)).
Solaris 10 normally uses SMF;
Solaris 9 and earlier systems
don't have it.
When the difference matters,
this manual will refer to SMF and non-SMF systems.
.PP
Parts may also behave differently on the slightly-different
versions of Solaris on SPARC and x86 (Intel/AMD) hardware.
Such differences will be annotated `on SPARC' or `on x86.'
.PP
This document uses the new binary-multiple prefixes
described in the IEEE 1541-2002 standard.
Prefixes like kilo, mega, giga, tera
(k, M, G, T)
refer to powers of ten;
when the near-equivalent powers of two are meant,
they are called kibi, mebi, gibi, tebi
(ki, Mi, Gi, Ti).
For example,
a gigabyte (GB) is
10\u\s-19\s+1\d
bytes,
while
a gibibyte (GiB) is
2\u\s-130\s+1\d
(1073741824).
See
\f(CW\s-1http://physics.nist.gov/cuu/Units/binary.html\s+1\fP
for more details.
.SH
1.1.
Requirements and limitations
.PP
The programs described here
have been tested on Solaris 7, 8, 9, and 10
on 32- and 64-bit SPARC systems,
and on Solaris 10 on 32- and 64-bit
x86 (IA32) systems.
They are unlikely to function correctly
on Solaris 2.6 or older
without additional programming work.
Solaris 9/x86 may work, but has not been tested.
They are likely to work on Open Solaris,
but since that is a moving target,
no promises can be made.
.PP
Coraid EtherDrive\(rg
storage blades and
Coraid SR RAID controllers
have been tested.
Any target device conforming to
version 1 of the
AoE protocol specification
should work,
provided the attached (or emulated) ATA disk
supports logical-block addressing.
Any ATA disk manufactured since 1995
ought to work fine.
.PP
The driver supports both 24 and 48-bit sector addresses,
allowing disks as large as 128 pebibytes.
Older versions of Solaris may be limited to 1TiB;
see below
for the full scoop.
.SH
1.2.
Release notes
.PP
Version 1.4 has these additional features
and carries these warnings:
.IP \(bu
Support has been added for Solaris/x86.
.IP \(bu
DOS partition tables
(\f2fdisk\fP
labels)
are recognized in a limited way,
equivalent to that afforded by other Solaris drivers.
See below
for details.
.IP \(bu
The normal Solaris tools
for reading and writing disk labels
(\f2prtvtoc\fP
and
\f2format\fP(1M))
may not work
on an unlabelled disk larger than 1TiB.
As a workaround,
the AoE package includes
a label-initializing program.
.IP \(bu
Support for DOS labels
may uncover new sorts of confusion
in
\f2format \fP\f2(1M).\fP
In case of trouble,
just use EFI labels
unless a disk already has a VTOC label
and cannot be rewritten from scratch.
.IP \(bu
In early versions of Solaris\010,
\f2format\fP
and
\f2zpool\fP
may emit an odd but harmless message like
.RS
\f(CW\s-1Error occurred with device in use checking: No such device\s+1\fP
.RE
or
.RS
\f(CW\s-1warning: device in use checking failed: No such device\s+1\fP
.RE
when pointed at an AoE disk.
This is probably a Solaris library quirk;
in any case it is mended in the 11/06 release.
.IP \(bu
ZFS on any sort of disk,
not just just AoE,
is
surprisingly fragile
in the face of I/O
errors.
Until Sun have repaired the problems,
we suggest thinking twice before using ZFS.
.IP \(bu
AoE may not be compatible with Sun Clusters;
we haven't had a chance to test.
Field reports are welcome.
.IP \(bu
AoE may not work properly
when run on an older,
slower system through a 10Mbps
\f(CW\s-1le\s+1\fP
network interface.
Using a 100Mbps
\f(CW\s-1hme\s+1\fP
network card
or a faster CPU
(any UltraSPARC should be OK)
works fine.
.SH
1.3.
Background:
AoE and EtherDrive blades
.PP
AoE (ATA over Ethernet)
is a network protocol
for accessing ATA devices over an Ethernet.
An initiator system constructs an AoE message
containing an ATA command
such as `read sector,'
and sends it to the Ethernet address of the desired target device;
the target responds with an AoE message
containing the resulting
device status and data (if any).
EtherDrive Storage Blades
and SR RAID controllers
are two examples of such targets,
but the protocol is more general.
The programs described here will work with any target
that correctly implements the protocol.
.PP
AoE is defined at the Ethernet level,
using an AoE-specific Ethernet protocol type
and Ethernet MAC addresses.
IP is not involved in any way.
There is no inter-network routing:
initiators and targets
must share an Ethernet network.
Neither is there any security built into the protocol:
a target will process commands from any node
on the same Ethernet.
It is usually best to dedicate an Ethernet (or VLAN)
to AoE traffic,
or at least to share only with other secure traffic,
e.g. SNMP or other management protocols.
It is quite unwise to run AoE
on a network
into which a random person may plug his untrustworthy laptop.
.PP
Each target is identified by a pair of numbers,
\f2aoemin\fP
and
\f2aoemaj\fP.
For an
EtherDrive blade,
\f2aoemaj\fP
is the shelf number,
\f2aoemin\fP
the slot;
for an SR RAID
controller,
they are set through the configuration interface.
The Solaris implementation
allows AoE to be active on several network interfaces;
each interface is assigned an
\f2aoechan\fP
number,
and targets are addressed by the triplet
(\f2aoechan\fP,
\f2aoemaj\fP,
\f2aoemin\fP).
Error messages and support programs
often express such target addresses
as three numbers
separated by slashes:
\f(CW\s-10/2/8\s+1\fP
means the target with
\f2aoechan\fP
0,
\f2aoemaj\fP
2,
\f2aoemin\fP
8.
.PP
A special broadcast message
asks every target on the network,
or the target with a specified address,
to respond with a check-in message
reporting its address
and a few other parameters.
When a target is first powered on
(e.g. when an EtherDrive blade is inserted in a shelf)
it broadcasts such a check-in response without being asked.
Thus host software can
find a target's Ethernet address
given its target address,
assemble a list of all targets on a network,
and learn
when a new target appears.
.SH
2.
Installing, removing, and upgrading the software
.SH
2.1.
Installing AoE
.PP
The AoE subsystem is a single Solaris package
distributed as a single (datastream format) file,
usually named
\f(CW\s-1CORDaoe-\s+1\fP\f2version\fP\f(CW\s-1-\s+1\fP\f2arch\fP,
e.g.
\f(CW\s-1CORDaoe-1.4-x86\s+1\fP.
It may arrive compressed,
e.g.
\f(CW\s-1CORDaoe-1.4-x86.Z\s+1\fP;
use
\f2uncompress\fP(1)
to unpack it.
.PP
To install the package,
use
\f2pkgadd\fP(1M)
as super-user:
.RS
\f(CW\s-1pkgadd -d CORDaoe-1.4-x86 CORDaoe\s+1\fP
.RE
.PP
The uncompressed package file is about 900kiB;
the installed subsystem takes about 1000kiB.
.PP
\f2Pkgadd\fP
may warn that scripts are to be run as super-user.
Installing the package runs
\f2add_drv\fP(1M)
to add
three device drivers
to the system,
and uses
\f2awk\fP
to add entries to
\f(CW\s-1/etc/devlink.tab\s+1\fP.
(On Solaris 10,
\f2pkgadd\fP
claims
\f(CW\s-1/etc/devlink.tab\s+1\fP
is itself a set-userid file,
apparently a garbling of the super-user script warning.)
On an SMF system,
installation runs
\f2svccfg\fP(1M)
to import the service manifest,
and
\f2svcadm\fP(1M)
to work around a bug in establishing service dependencies.
.PP
\f2Pkgadd\fP
may also warn that the ownership of the three
\f(CW\s-1/usr/share/man\s+1\fP
directories
is being changed.
The trouble is that these directories may not always exist,
and there is no way both to tell
\f2pkgadd\fP
to create them if necessary
and to tell it to leave existing permissions be.
.SH
2.2.
Removing AoE
.PP
To remove AoE from the system:
.IP 1.
Save any configuration files you may need later,
in particular
\f(CW\s-1/etc/aoe.conf\s+1\fP
and
\f(CW\s-1/usr/kernel/drv/aoed.conf\s+1\fP.
.IP 2.
Ensure that no AoE disks are in use:
unmount any file systems
and
\f(CW\s-1zpool\0\s+1\fP\f(CW\s-1export\s+1\fP
any ZFS pools
on AoE disks;
remove any AoE swap areas.
Then run
\f(CW\s-1svcadm\0\s+1\fP\f(CW\s-1disable\0\s+1\fP\f(CW\s-1aoe\s+1\fP
(SMF system)
or
\f(CW\s-1/etc/init.d/aoe\0\s+1\fP\f(CW\s-1stop\s+1\fP
(non-SMF).
.IP 3.
Ensure that the system won't try to use AoE devices later:
remove or comment out
\f2vfstab\fP
entries referring to AoE disks;
remove or comment out every line of
\f(CW\s-1/etc/aoe.conf\s+1\fP,
or remove or rename the file.
.IP 4.
Run
\f(CW\s-1pkgrm CORDaoe\s+1\fP.
This calls
\f2rem_drv\fP(1M)
to remove the device drivers from the system configuration,
then deletes all files created when the package was installed.
.PP
\f2Rem_drv\fP
removes the device files in
\f(CW\s-1/devices\s+1\fP,
but may leave the symbolic links in
\f(CW\s-1/dev\s+1\fP.
That's just the way some versions of Solaris work.
It's OK to remove the dangling links yourself.
.SH
2.3.
Upgrades and downgrades
.PP
To change from one version of AoE to another,
remove the existing
\f(CW\s-1CORDaoe\s+1\fP
package
and
install the desired version
as explained above.
Remember to save
\f(CW\s-1/usr/kernel/drv/aoed.conf\s+1\fP
if you've customized it;
otherwise it will be reset to its default contents
when the new package is installed.
If using ZFS,
remember to
\f(CW\s-1zpool\0\s+1\fP\f(CW\s-1export\s+1\fP
and
\f(CW\s-1zpool\0\s+1\fP\f(CW\s-1import\s+1\fP
any pools already stored on AoE disks.
.SH
2.4.
What's installed
.PP
The package installs files in the following directories:
.IP
.in -\n(PIu
\f(CW\s-1/usr/kernel/strmod\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/usr/kernel/strmod/sparcv9\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/usr/kernel/strmod/amd64\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/usr/kernel/drv\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/usr/kernel/drv/sparcv9\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/usr/kernel/drv/amd64\s+1\fP
.in
Device driver binaries and configuration files.
There are two device drivers
and one STREAMS module,
each in both 32- and 64-bit
(\f(CW\s-1sparcv9\s+1\fP
or
\f(CW\s-1amd64\s+1\fP)
versions.
.IP
.in -\n(PIu
\f(CW\s-1/usr/sbin\s+1\fP
.in
Configuration and control programs:
\f2aoectl\fP,
\f2aoestart\fP,
\f2aoestop\fP,
\f2aoemkconf\fP,
\f2aoelabinit\fP,
\f2aoemon\fP,
\f2aoeunlabel\fP.
.IP
.in -\n(PIu
\f(CW\s-1/etc/init.d\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/etc/rc2.d\s+1\fP
.in
Non-SMF start-and-stop control script.
\f(CW\s-1/etc/init.d/aoe\s+1\fP
is installed on both SMF and non-SMF systems,
the
\f(CW\s-1/etc/rc2.d/S00aoe\s+1\fP
link only on an non-SMF system.
.IP
.in -\n(PIu
\f(CW\s-1/lib/svc/method\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/var/svc/manifest/device\s+1\fP
.in
SMF start-and-stop control script
\f(CW\s-1/lib/svc/method/device-aoe\s+1\fP
and service manifest
\f(CW\s-1/var/svc/manifest/device/aoe.xml\s+1\fP.
Installed only on an SMF system.
.IP
.in -\n(PIu
\f(CW\s-1/etc/aoe\s+1\fP
.in
Initially empty;
holds one file per active
\f2aoechan\fP,
used
(with
\f2fattach\fP(3))
to hold the channel open.
.IP
.in -\n(PIu
\f(CW\s-1/usr/share/man/man1m\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/usr/share/man/man7d\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/usr/share/man/man7m\s+1\fP
.in
Manual entries describing the programs and drivers.
.IP
.in -\n(PIu
\f(CW\s-1/opt/CORDaoe/doc\s+1\fP
.in
Longer documents
(this one,
for instance)
about the use and workings of AoE.
See
\f(CW\s-1/opt/CORDaoe/doc/README\s+1\fP
for details.
.IP
.in -\n(PIu
\f(CW\s-1/opt/CORDaoe/lib\s+1\fP
.in
Odds and ends used during package installation.
.SH
3.
Configuration and use
.SH
3.1.
Setting up a network channel
.PP
Before AoE targets can be accessed,
the network channels it will use
must be defined in file
\f(CW\s-1/etc/aoe.conf\s+1\fP.
Until this file has been set up,
starting AoE silently does nothing.
.PP
To set up a single channel,
create
\f(CW\s-1aoe.conf\s+1\fP
with a single line declaring the device to be used.
For example:
.RS
\f(CW\s-1aoechan 0 ether /dev/bge2\s+1\fP
.RE
configures a channel using
the third connector
of the four-port gigabit interface
built into some systems.
.PP
Any Ethernet device Solaris supports may be used.
Special configuration may be required
to enable
jumbo frames;
see Sun's documentation for details.
.PP
The Ethernet device need not be dedicated to AoE;
in particular it may be shared with IP,
though as noted
above,
this is often unwise.
.PP
To use more than one network device for AoE,
declare additional channels
as explained below.
.PP
Normally nothing special is needed in
\f(CW\s-1aoe.conf\s+1\fP
to permit jumbo frames,
but jumbo support may need to be enabled
in Solaris for the network device to be used.
See
the discussion of jumbo frames
for the full scoop.
.PP
Once
\f(CW\s-1aoe.conf\s+1\fP
has been set up,
AoE must be started:
.IP \(bu
On an SMF system,
AoE is disabled by default.
To enable it, use
\f2svcadm\fP(1M):
.RS
\f(CW\s-1svcadm enable svc:/device/aoe\s+1\fP
.RE
This both starts AoE right away
and records that it should be started automatically
on subsequent boots.
.IP \(bu
On a non-SMF system,
run
.RS
\f(CW\s-1/etc/init.d/aoe start\s+1\fP
.RE
or just reboot.
Start-on-boot is enabled by default.
.SH
3.2.
Device names
.PP
AoE disk devices
are named similarly to directly attached disks.
A block devices has a name like
\f(CW\s-1/dev/dsk/cad107s2\s+1\fP;
the corresponding raw device is
\f(CW\s-1/dev/rdsk/cad107s2\s+1\fP.
.PP
What does
\f(CW\s-1cad107s2\s+1\fP
mean?
In AoE's use of standard Sun device-name conventions:
.IP
.in -\n(PIu
\f(CW\s-1ca\s+1\fP
.in
Solaris controller ID
\f(CW\s-1a\s+1\fP.
AoE disks are always
\f(CW\s-1ca\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1d107\s+1\fP
.in
Device
\f(CW\s-1107\s+1\fP.
The number is a
decimal-digit encoding of the target address for this disk:
\f2cMMmm\fP
for target
\f2c\fP\f(CW\s-1/\s+1\fP\f2MM\fP\f(CW\s-1/\s+1\fP\f2mm\fP,
with leading zeroes discarded.
Thus
\f(CW\s-1d107\s+1\fP
is target
\f(CW\s-10/1/7\s+1\fP;
\f(CW\s-1d3\s+1\fP
is
\f(CW\s-10/0/3\s+1\fP;
\f(CW\s-1d10319\s+1\fP
is
\f(CW\s-11/3/19\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1s2\s+1\fP
.in
Partition
\f(CW\s-12\s+1\fP.
The eight standard Solaris partitions
\f(CW\s-1s0\s+1\fP - \f(CW\s-1s7\s+1\fP
(sixteen,
\f(CW\s-1s0\s+1\fP - \f(CW\s-1s15\s+1\fP,
on x86)
are allowed.
.LP
If the disk has a DOS (fdisk) partition table,
partition suffixes
\f(CW\s-1p1\s+1\fP - \f(CW\s-1p4\s+1\fP
refer to the four direct DOS partitions.
Only primary DOS partitions are accessible;
extended partitions are not supported.
Unlike standard Sun drivers,
DOS partitions
are allowed even on SPARC systems.
.PP
On x86 systems only,
a Sun VTOC label
may be encapsulated in a DOS partition;
in that case the
\f(CW\s-1s\s+1\fP\f2n\fP
partitions refer to the VTOC partitions
contained within that DOS partition,
as with standard Sun drivers.
.PP
There is also a device name
with no partition suffix,
e.g.
\f(CW\s-1/dev/dsk/cad107\s+1\fP;
this accesses the whole disk
regardless of the disk label,
including the parts normally reserved
for system use.
.PP
By default devices are created for targets
\f(CW\s-10/0/0\s+1\fP - \f(CW\s-10/0/14\s+1\fP
(i.e.
\f(CW\s-1cad0s\(**\s+1\fP - \f(CW\s-1cad14s\(**\s+1\fP)
and
\f(CW\s-10/1/0\s+1\fP - \f(CW\s-10/1/14\s+1\fP
(\f(CW\s-1cad100s\(**\s+1\fP - \f(CW\s-1cad114s\(**\s+1\fP):
enough for
two EtherDrive shelves
or SR RAID devices
numbered 0 and 1,
each with up to fifteen blades.
To make more devices
requires editing a configuration file;
in Solaris 9 and earlier,
the system usually must be rebooted as well.
See
\f3More \fP\f3devices\fP
for details.
.PP
In Solaris,
names in
\f(CW\s-1/dev\s+1\fP
are symbolic links to real device files in
\f(CW\s-1/devices\s+1\fP.
For an AoE disk the real device name
is of the form
.RS
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:\s+1\fP\f2part\fP
.RE
\f2Inst\fP
is the decimal-digit encoding of the target address,
\f2part\fP
the
\f(CW\s-1/devices\s+1\fP-style
partition name.
For example,
\f(CW\s-1/dev/dsk/cad107s2\s+1\fP
is a link to
\f(CW\s-1/devices/pseudo/aoed@107:c\s+1\fP;
\f(CW\s-1/dev/rdsk/cad107\s+1\fP
to
\f(CW\s-1/devices/pseudo/aoed@107:wd,raw\s+1\fP.
.SH
3.3.
Enabling devices
.PP
Before an AoE disk device can be opened,
it must be
\f2enabled\fP.
Normally this is done automatically:
a target's check-in message
is intercepted by
\f2aoemon\fP;
\f2aoemon\fP
enables its devices,
and reports to
\f2syslog\fP
that it did so.
.PP
\f(CW\s-1aoectl\0\s+1\fP\f(CW\s-1list\s+1\fP
lists enabled targets.
\f2Aoectl\fP
may also be used to enable or disable specific targets,
or to repeat the check-in broadcast
so targets will repeat their responses.
See the section on
tools
for details.
.PP
If a target isn't enabled but should be,
try resetting it;
e.g. pull the EtherDrive blade out of the shelf and push it back in,
or put the SR lblade offline and then online.
If all is well,
within a few seconds
\f2aoemon\fP
should report that it has enabled that target.
.SH
3.4.
Using AoE disks
.PP
Once enabled,
AoE
\f(CW\s-1/dev/dsk\s+1\fP
and
\f(CW\s-1/dev/rdsk\s+1\fP
devices work like any other disks.
.PP
\f2Format\fP(1M)
can be used to set up partition tables
(the
\f(CW\s-1partition\s+1\fP
submenu)
and to run simple read and read-write disk tests
(\f(CW\s-1analyze\s+1\fP),
modulo
a few bugs and quirks
described below.
Low-level formatting,
bad-block repair,
and defect-list management
(the
\f(CW\s-1format\s+1\fP,
\f(CW\s-1repair\s+1\fP,
and
\f(CW\s-1defect\s+1\fP
menus)
are not supported.
A partition table may also be written with
\f2fmthard\fP(1M)
or read with
\f2prtvtoc\fP(1M).
On a brand-new huge drive,
or one bearing a spurious old label,
it may be necessary to use
\f2aoelabinit\fP
to initialize the label first.
.PP
If the disk doesn't have a label (partition table),
the driver makes one up.
In the made-up label,
subdevice
\f(CW\s-1s0\s+1\fP
(and
\f(CW\s-1s2\s+1\fP
if the disk supports VTOC labels)
is all of the disk intended for normal use,
excluding parts reserved for use by diagnostics
and the label itself.
.PP
Any file system except the root
or
\f(CW\s-1/usr\s+1\fP
may be stored on an AoE disk.
AoE is started very early in the boot process;
AoE file systems
listed in
\f2vfstab\fP(4)
and mounted automatically when the system boots.
It is not possible to boot from an AoE device.
.PP
AoE disks may belong to ZFS storage pools,
but beware
the reliability problems
(not specific to AoE)
described below.
.PP
AoE disks may be used for swapping.
AoE swap areas listed in
\f2vfstab\fP
will be added automatically.
On a non-SMF system,
swap areas are added twice,
once before AoE has been started and once after.
The first attempt produces
a harmless message like
.RS
\f(CW\s-1/dev/dsk/cad108s1: No such device or address\s+1\fP
.RE
If all is well, the second attempt succeeds.
On an SMF system,
AoE is started earlier
and AoE swap areas are enabled without fuss.
.PP
On an SMF system,
if the
\f(CW\s-1devices/aoe\s+1\fP
service is enabled,
it is automatically started if possible in single-user mode.
To use AoE in single-user mode
on a non-SMF system,
ensure that
\f(CW\s-1/usr\s+1\fP
is mounted if it is a separate file system,
then run
\f(CW\s-1/etc/init.d/aoe\0\s+1\fP\f(CW\s-1start\s+1\fP.
.SH
3.4.1.
Large disks
.PP
Support for disks larger than 1TiB is somewhat complicated,
because of the way Solaris supports such disks.
.SH
3.4.1.1.
What Solaris does
.PP
The following is mostly a summary
of Sun's documentation,
which should be consulted for details.
.PP
The original Solaris disk-labelling scheme (VTOC)
stored the size of each partition as a
32-bit signed sector count.
This limited the size of a single disk subdevice (partition)
to
2\u\s-131\s+1\d
sectors,
or
1 tebibyte.
The UFS file system format
had similar limits:
no file system could span more than 1TiB.
.PP
DOS partition labels are similarly constrained
by 32-bit values to disks no larger than 1TiB.
.PP
The Solaris 9 4/03 release
added a new labelling scheme (EFI),
derived from an Intel specification.
EFI labels store partition parameters in 64-bit unsigned values;
a disk device or any single partition
may span
2\u\s-164\s+1\d
sectors,
or
8 zebibytes,
or
about eight billion tebibytes.
Solaris 9 8/03
updated UFS to match:
a new option to
\f2newfs\fP(1M)
creates a `multi-terabyte
[\f2sic\fP]
file system.'
These upgrades may be installed
on an older Solaris 9 system
by installing the current recommended patch cluster;
they are not available for Solaris 8 or older systems.
.PP
EFI labels afford several benefits besides large-disk handling:
.IP \(bu
The VTOC format is sensitive
to the byte order of the computer accessing the label:
a VTOC written by Solaris/SPARC
makes no sense to Solaris/x86.
EFI labels have a defined byte order,
independent of the host CPU.
.IP \(bu
The VTOC format assumed the disk comprised an array of cylinders,
all of the same size.
Disks ceased to be built that way many years ago;
modern ATA disks no longer report useful
sector, head, or cylinder counts.
EFI labels treat the disk as a simple array of blocks.
.PP
One new constraint is that all partitions
in a single EFI label
must be non-overlapping.
In particular the Solaris tradition
that subdevice
\f(CW\s-1s2\s+1\fP
accesses the entire user-data part of the disk
regardless of other partitions
cannot be honoured under EFI.
.PP
To retain compatibility with existing disks
and older systems,
VTOC labels are still supported.
Sun's standard tools
write VTOC labels to 1TiB or smaller disks,
EFI labels to larger disks.
The system looks for a VTOC label first,
then an EFI label;
thus a small disk may have
either type of label.
.SH
3.4.1.2.
How it works with AoE disks
.PP
The Solaris AoE driver
supports either labelling scheme.
.PP
If an unlabelled disk is no larger than 1TiB,
the AoE driver pretends it has a VTOC label
in which
\f(CW\s-1s0\s+1\fP
and
\f(CW\s-1s2\s+1\fP
both access the entire user-data part of the disk.
The regular Solaris tools read and modify the label
without trouble.
.PP
If an unlabelled disk is larger than 1TiB,
the disk driver makes no pretense.
The whole-disk subdevice
(\f(CW\s-1cad\s+1\fP\f2nnn\fP)
accesses the entire physical device;
none of the standard partition subdevices
(\f(CW\s-1cad\s+1\fP\f2nnn\fP\f(CW\s-1s\s+1\fP\f2n\fP)
may be used.
Sun's standard tools report no label
until one is written;
that is the best that can be done
within Sun's disk-driver interface.
\f2Format\fP(1M)
cannot handle an unlabelled disk
large enough to require EFI labels;
use
\f2aoelabinit\fP
as a workaround.
.PP
If a disk already has a VTOC label,
everything works in the standard way.
The eight standard partitions
\f(CW\s-1s0\s+1\fP-\f(CW\s-1s7\s+1\fP
work as described in the label;
Sun's standard tools display the label and can modify it.
.PP
If a disk already has an EFI label,
and the system accessing it runs a version of Solaris
new enough to have EFI support,
everything also just works.
The eight standard partitions
\f(CW\s-1s0\s+1\fP-\f(CW\s-1s7\s+1\fP
are defined by the first eight EFI partitions.
Sun's standard tools display the label and can modify it,
with
minor bugs.
.PP
If a disk already has an EFI label,
pre-EFI Solaris can access it with limitations.
The AoE driver reads the label and defines the partitions;
the corresponding
\f(CW\s-1s0\s+1\fP-\f(CW\s-1s7\s+1\fP
subdevices work;
but Sun's standard tools don't understand the EFI label
and report errors if asked to display it.
In particular,
\f2format\fP
on a pre-EFI system
doesn't work at all with an EFI-labelled disk.
\f2Aoeunlabel\fP
will wipe out an existing EFI label,
allowing the disk to be partitioned from scratch.
.PP
Pre-EFI Solaris running in 32-bit mode can access
at most 1TiB from a disk partition,
even if the EFI label defines it to be larger.
Pre-EFI Solaris in 64-bit mode can access the whole partition
no matter how big.
Pre-EFI Solaris cannot make a UFS file system
larger than 1TiB,
nor can such a system read a larger file system
written by a newer system.
.SH
3.4.2.
\f2format\fP(1M)
bugs and limitations
.PP
The
\f2format\fP
\f(CW\s-1format\s+1\fP,
\f(CW\s-1repair\s+1\fP,
and
\f(CW\s-1defect\s+1\fP
menus
are not supported
on AoE disks.
.PP
An AoE target is listed in
\f2format\fP's
disk-selection menu
only if it is
enabled.
If an enabled device is unreachable
(broken, removed,
mistakenly enabled by hand),
\f2format\fP
may stall for a few seconds trying to access it,
as it would for a configured-but-inaccessible
SCSI disk.
.PP
\f2Format\fP
exhibits a few bugs when working with
an AoE disk larger than 1TiB:
.IP \(bu
\f2Format\fP
cannot work with an unlabelled AoE disk
larger than 1TiB;
it can't even write a label to such a disk.
To work around this,
use
\f2aoelabinit\fP
to write the first label.
.IP \(bu
ATA devices report a device-model string.
According to the ATA specification it is
allowed to be 40 characters long.
Prior to Solaris 10 11/06,
\f2format\fP
may mangle this string,
truncating it to 24 characters
and inserting a blank after the eighth.
.SH
3.4.3.
ZFS is surprisingly fragile
.PP
We have learned that ZFS is surprisingly non-robust
in the fact of I/O errors:
if a disk reports repeated errors
and the ZFS pool doesn't have enough remaining redundancy
to allow the disk to be set aside,
the whole system panics (crashes).
.PP
For example, the system will panic if:
.IP \(bu
Any disk fails in a pool
with neither RAID nor mirror redundancy.
.IP \(bu
A disk fails in a
\f(CW\s-1raidz1\s+1\fP
pool that is already
\f(CW\s-1DEGRADED\s+1\fP,
i.e.
a disk has already failed
and has not yet been replaced.
.IP \(bu
A disk fails in a
\f(CW\s-1raidz2\s+1\fP
pool in which two disks have already failed
and neither has yet been replaced.
.LP
It is not unreasonable for ZFS to withdraw access to a pool
that has suffered such a severe failure,
but to do so by crashing the whole system
(even if other ZFS pools
or other non-ZFS operations are still have to function)
is,
to put it politely,
excessive.
.PP
This is a problem in Solaris, not in the AoE code;
it has been reported with iSCSI arrays as well.
Rumour has it that Sun are aware of the problem
and a fix is in the works.
.PP
We urge customers to take special care
when using ZFS,
unless it is acceptable
for the entire system to crash because of a single broken disk.
Use redundancy of one sort or another for all pools,
whether by using the mirroring and RAID mechanisms
built into ZFS
or by using an external RAID mechanism
(such as that built into Coraid's RAID appliances).
Whatever redundancy is used,
make use of hot spare mechanisms
so that redundancy will be restored automatically
as quickly as possible.
Check regularly for failures
and replace broken disks
so that the hot-spare supply won't run out.
.PP
All of this is good practice
for any modern storage system.
That ZFS is crash-prone
just sharpens the consequences of error.
.SH
3.4.4.
Metadisks
.PP
AoE disks may belong to metadisks managed by the
Solaris Volume Manager in Solaris 9 and newer systems.
Concatenation, striping, mirroring, RAID 5, and soft partitions
all work just as for any other disk.
Solstice DiskSuite,
the extra-cost predecessor to Volume Manager,
has not been tested;
it probably works but this is not a promise.
Field reports are welcome.
.PP
On an old computer system,
initializing a RAID group
(\f(CW\s-1metainit \s+1\fP\f(CW\s-1d\s+1\fP\f2nn\fP \f(CW\s-1-r\s+1\fP)
with four or more AoE-disk components
may lock up the system.
The system is actually OK,
but so busy initializing RAID data
that it cannot do much else.
All returns to normal when RAID initialization has finished.
A 140MHz Ultra-1 system exhibits the problem;
an Ultra-2 with a single 300MHz processor does not.
Any current system ought to be OK.
.SH
3.5.
Error logging and monitoring
.PP
Kernel drivers in the AoE subsystem
log error messages to
\f2syslog\fP
facility
\f(CW\s-1kern\s+1\fP;
in particular this is how disk errors are reported.
All messages from the AoE disk driver
contain the string
\f(CW\s-1aoed\s+1\fP
and give the device instance number
and target address.
Disk-error messages always contain the string
\f(CW\s-1ATA \s+1\fP\f(CW\s-1error\s+1\fP
and give the ATA error-register value
and an ASCII interpretation of its contents.
Messages about errors in the AoE part of a target
(not the ATA disk part)
say
\f(CW\s-1AoE \s+1\fP\f(CW\s-1error\s+1\fP
and give both the AoE error code
and its conventional meaning.
.PP
User-mode programs
responsible for starting AoE channels
and for monitoring for unusual events
(\f2aoestart\fP
and
\f2aoemon\fP,
described in more detail below)
log to
\f2syslog\fP
facility
\f(CW\s-1daemon\s+1\fP.
This log is where
\f2aoestart\fP
reports that an
\f2aoechan\fP
could not be started,
and where
\f2aoemon\fP
reports that a target was automatically enabled.
\f2Aoemon\fP
also logs a packet dump when an error occurs:
of the message received for an AoE-protocol or ATA error,
of the message that couldn't be sent for a timeout.
.PP
\f2Iostat\fP(1M)
lists AoE disks that have been used at least once
since the last boot.
Additional AoE-specific statistics
may be fetched with
\f2kstat\fP(1M).
.SH
4.
Advanced topics
.PP
.SH
4.1.
More devices
.PP
File
\f(CW\s-1/usr/kernel/drv/aoed.conf\s+1\fP
lists the AoE disk devices to be created.
Only the devices listed when the
\f(CW\s-1aoed\s+1\fP
driver last processed the file
(usually when the system last booted)
will work,
even if additional special files happen to exist,
and even if other targets exist and have been enabled.
.PP
The AoE subsystem comes with a default
\f(CW\s-1aoed.conf\s+1\fP
file
declaring
an initial set of targets.
If more are needed,
the file must be edited
and the system told to reread it.
.SH
4.1.1.
Editing
\f(CW\s-1aoed.conf\s+1\fP
.PP
\f(CW\s-1aoed.conf\s+1\fP
is a Solaris driver configuration file,
in the form described in
\f2driver.conf\fP(4).
The easiest way to add to the file
is to generate new entries with
\f2aoemkconf\fP
and edit them in.
You may also write your own entries.
.PP
To summarize the general Solaris rules:
a driver configuration file
contains lines of text.
Anything following
\f(CW\s-1#\s+1\fP
is a comment.
Empty lines are ignored.
Non-empty lines declare devices
or driver properties
(parameters,
given as
\f2name\fP\f(CW\s-1=\s+1\fP\f2number\fP
or
\f2name\fP\f(CW\s-1="\s+1\fP\f2string\fP\f(CW\s-1"\s+1\fP).
Each line must be terminated with a semi-colon.
.PP
Each line in
\f(CW\s-1aoed.conf\s+1\fP
declares a device instance,
specifying an instance number
and target address:
.RS
\f(CW\s-1name="aoed" parent="pseudo"\s+1\fP
\f(CW\s-1instance=\s+1\fP\f2inst\fP
\f(CW\s-1aoechan=\s+1\fP\f2cno\fP
\f(CW\s-1aoemaj=\s+1\fP\f2majno\fP
\f(CW\s-1aoemin=\s+1\fP\f2minno\fP\f(CW\s-1;\s+1\fP
.RE
The punctuation is important;
in particular the double-quotes
and the terminal semicolon must be present.
The required fields have these meanings:
.IP
.in -\n(PIu
\f(CW\s-1name="aoed" parent="pseudo"\s+1\fP
.in
Constant values
telling the system which driver this is
and how it fits into the system.
.IP
.in -\n(PIu
\f(CW\s-1instance=\s+1\fP\f2inst\fP
.in
\f2Inst\fP,
a decimal number,
is the
\f2instance \fP\f2number\fP
of this device,
used to generate device names:
\f(CW\s-1instance=102\s+1\fP
produces
\f(CW\s-1/devices/pseudo/aoed@102:a\s+1\fP,
\f(CW\s-1/dev/dsk/cad102s0\s+1\fP,
and so on.
.IP
.in -\n(PIu
\f(CW\s-1aoechan=\s+1\fP\f2cno\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1aoemaj=\s+1\fP\f2majno\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1aoemin=\s+1\fP\f2minno\fP
.in
\f2Cno\fP,
\f2majno\fP,
and
\f2minno\fP
are decimal numbers
expressing the the target address:
this is target
\f2cno\fP\f(CW\s-1/\s+1\fP\f2majno\fP\f(CW\s-1/\s+1\fP\f2minno\fP.
.PP
These optional fields are allowed:
.IP
.in -\n(PIu
\f(CW\s-1timeout=\s+1\fP\f2ms\fP
.in
Allow this device
\f2ms\fP
milliseconds
before retrying or expiring a command.
The default is 200ms.
It may need to be lengthened for a long-latency device,
or shortened to make the best of a badly-congested network,
e.g. when running AoE over 10Mbps Ethernet
(allowed but probably not a good idea).
.IP
.in -\n(PIu
\f(CW\s-1hd=\s+1\fP\f2nhead\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1sec=\s+1\fP\f2nsec\fP
.in
Pretend this drive has cylinders of
\f2nhead\fP
heads and
\f2nsec\fP
sectors,
ignoring any information returned by the hardware
or the default numbers in the driver.
Numbers stored in a valid disk label
take precedence.
Both
\f(CW\s-1hd\s+1\fP
and
\f(CW\s-1sec\s+1\fP
must be specified;
if only one is supplied,
it is ignored.
.IP
.in -\n(PIu
\f(CW\s-1maxdata=\s+1\fP\f2size\fP
.in
Limit ATA data transfers for this device
to at most
\f2size\fP
bytes.
Usually needed only when using
jumbo frames
on problematic networks.
Normally it is better to set the size
for the whole
\f2aoechan\fP
rather than for individual targets.
.IP
.in -\n(PIu
\f(CW\s-1maxbuf=\s+1\fP\f2ncmds\fP
.in
Allow at most
\f2ncmds\fP
unacknowledged commands for this target.
Subsequent commands will be held
until a pending command has
been answered or has timed out.
Normally initiator and target negotiate a suitable value,
but it may occasionally be necessary to limit it
on congested networks
or when many initiators access a target concurrently.
.PP
The optional parameters must be added by hand;
\f2aoemkconf\fP
won't put them in.
.PP
The declarations in
\f(CW\s-1aoed.conf\s+1\fP,
and nothing else,
connect instance numbers to target addresses.
That the instance number is a decimal encoding of the target address
is only a convention;
nowhere in the driver subsystem is this assumed.
If you write your own entries
you may use whatever mapping you like,
as long as no
\f2inst\fP
value is used for more than one device,
and none is too large:\u\s-12\s+1\d
20164 on a 32-bit SPARC system,
330382099 on 64-bit SPARC;
12483 and 204522252 on x86.
.FS
2.
The exact rule in
in the versions of Solaris we support:
.IP \(bu
A 32-bit system allows 18 bits for a minor device number.
On a SPARC system,
the driver creates 13 minor devices for each target,
262144/13 = 20164 instances are allowed.
On an x86 system,
there are 21 devices,
so the limit is 262144/21 = 12483.
.IP \(bu
A 64-bit system has 32-bit minor device numbers,
so the calculation is
4294967296/13 = 330382099
or
4294967296/21 = 204522252.
.FE
.SH
4.1.2.
Making the new file effective
.PP
On Solaris 10,
changes to
\f(CW\s-1aoed.conf\s+1\fP
can be made effective while the system is running:
.RS
\f(CW\s-1# update_drv -f aoed\s+1\fP
.RE
unloads the
\f(CW\s-1aoed\s+1\fP
module if possible,
reloads it,
and updates
\f(CW\s-1/devices\s+1\fP
and
\f(CW\s-1/dev\s+1\fP.
If some AoE devices are in use
(mounted,
in use for swapping,
device file open),
\f2update_drv\fP
will complain that the module could not be unloaded;
new devices will be added,
but existing devices whose
\f(CW\s-1aoed.conf\s+1\fP
entries have changed
will not be updated until the next driver reload or system boot.
.PP
On Solaris 9 and older systems,
changes to
\f(CW\s-1aoed.conf\s+1\fP
have no effect until the next time the
\f(CW\s-1aoed\s+1\fP
driver module is loaded,
and
\f(CW\s-1/devices\s+1\fP
and
\f(CW\s-1/dev\s+1\fP
are not updated until Solaris is told to do so.
The simplest way to change the AoE configuration
is to update
\f(CW\s-1aoed.conf\s+1\fP
and then perform
a configuration reboot:
\f(CW\s-1reboot\0\s+1\fP\f(CW\s-1--\0\s+1\fP\f(CW\s-1-r\s+1\fP
if Solaris is running,
\f(CW\s-1boot\0\s+1\fP\f(CW\s-1-r\s+1\fP
to the Open Boot
\f(CW\s-1ok\s+1\fP
prompt.
It is also possible to make changes effective
right away
if no AoE device is in use,
by unloading the
\f(CW\s-1aoed\s+1\fP
module and explicitly reconfiguring:
.KS
.RS
\f(CW\s-1# modinfo | awk '$6 == "aoed"'\s+1\fP
.br
\f(CW\s-1190 780a4000   a570 229   1  aoed (AoE disk driver v1.2)\s+1\fP
.br
\f(CW\s-1# modunload -i 190\s+1\fP
.br
\f(CW\s-1# devfsadm -i aoed\s+1\fP
.br
.RE
.KE
The
\f(CW\s-1-i\s+1\fP
argument to
\f2modunload\fP
is the first field of the line printed by
\f2modinfo\fP.
If
\f2modinfo\fP
doesn't list the
\f(CW\s-1aoed\s+1\fP
module,
the driver wasn't loaded;
skip ahead and run
\f2devfsadm.\fP
.PP
On Solaris 7,
\f2devfsadm\fP
may not exist
(it was added by a patch)
and in any case appears to be undocumented.
The older equivalent,
which works in either case,
is:
.RS
\f(CW\s-1# drvconfig -i aoed\s+1\fP
.br
\f(CW\s-1# devlinks\s+1\fP
.br
.RE
.PP
It is wise to plan ahead,
especially on a non-SMF system
where the driver must be unloaded to reconfigure.
For example,
when installing a new EtherDrive shelf,
add a device instance for each slot,
not just for those you plan to use right away;
when installing an EtherDrive Storage Appliance,
create a few extra devices.
To create hundreds of never-used devices
would be silly,
but the cost of a few extras is modest:
a kibibyte or so inside the operating system per device,
a handful of inodes for special files and symbolic links,
extra entries in the
\f(CW\s-1/dev/dsk\s+1\fP
and
\f(CW\s-1/dev/rdsk\s+1\fP
directories.
.SH
4.2.
Fewer devices
.PP
If a target is removed while the system is running,
it is prudent to disable it manually:
.RS
\f(CW\s-1aoectl disable\s+1\fP \f2target\fP \&.\|.\|.
.RE
Otherwise
attempts to access it will stall for a few seconds
before timing out.
In particular,
\f2format\fP
may stall during its initial search for disks,
just as it sometimes stalls when a SCSI disk has been removed.
If the system reboots,
a missing AoE target won't respond
and will not be enabled.
If the target is plugged back in while the system is running
it will be enabled automatically.
.PP
Removing a target's entry from
\f(CW\s-1/usr/kernel/drv/aoed.conf\s+1\fP
will save a little memory
after the system is next booted.
There's little point in doing this for a single target;
it may make sense if many disks
have been permanently removed.
.PP
When a target is removed from
\f(CW\s-1aoed.conf\s+1\fP
its entries in
\f(CW\s-1/devices\s+1\fP
or
\f(CW\s-1/dev\s+1\fP
may remain,
even after a configuration reboot.
Attempts to use such ghost devices
return errors.
If the same target is later restored to
\f(CW\s-1aoed.conf\s+1\fP
(and the driver reconfigured)
the devices will work again.
This is just the way some versions of Solaris work,
especially older ones.
.SH
4.3.
More channels
.PP
File
\f(CW\s-1/etc/aoe.conf\s+1\fP
assigns AoE network channels
to Solaris Ethernet devices.
Every device through which AoE devices will be accessed
must be listed,
with a distinct
\f2aoechan\fP
number.
.PP
To add network interfaces,
add lines to that file,
in the same format as the first:
.RS
\f(CW\s-1aoechan\s+1\fP \f2channum\fP \f(CW\s-1ether\s+1\fP \f2device\fP
.RE
.PP
Each channel must have a distinct
\f2channum\fP
in the range 0-9.
Channels may not share a network device,
though of course a multiport device like
the Quad Fast Ethernet
may support several channels,
one to a port.
The network device need not be dedicated to AoE;
in particular it may also be configured as an IP interface.
.PP
Changes to
\f(CW\s-1aoe.conf\s+1\fP
take effect when the system is next booted,
or when
\f(CW\s-1svcadm\0\s+1\fP\f(CW\s-1refresh\0\s+1\fP\f(CW\s-1svc:/device/aoe\s+1\fP
(SMF system)
or
\f(CW\s-1/etc/init.d/aoe\0\s+1\fP\f(CW\s-1restart\s+1\fP
(non-SMF)
is run.
Adding an
\f2aoechan\fP
usually requires
adding entries to
\f(CW\s-1aoed.conf\s+1\fP,
so a non-SMF system
usually must be booted anyway.
.SH
4.4.
Jumbo frames
.PP
The Ethernet standard allows at most 1500 bytes per frame.
The AoE header for an ATA command occupies 22 bytes;
transfers must be in whole 512-byte sectors.
Thus only two sectors may be read or written at a time
using standard Ethernet.
.PP
Per-frame overhead is quite significant in gigabit Ethernet,
not just for AoE but for many protocols.
Hence many manufacturers of gigabit network cards and switches
allow larger frames,
so that more data may be packed into a single frame
and fewer frames sent.
Usually the maximum size is about 9000 bytes,
but it varies by product.
.PP
Jumbo frames can speed up AoE I/O quite a bit.
Usually it is necessary to enable them explicitly in Solaris,
and some care may be needed to make everything work right.
.SH
4.4.1.
Requirements for jumbo frames
.PP
For jumbo frames to work,
they must be supported in three places:
.IP 1.
The AoE target must support jumbo frames.
All current gigabit-capable Coraid devices do.
.IP 2.
The host network interface to be used for AoE
must support jumbo frames,
and jumbo support must be enabled.
Not all Sun-supported network devices are jumbo-capable.
In particular jumbo frames are allowed by the
\f(CW\s-1ce\s+1\fP
device
(e.g. the add-on GigaSwift adapter),
but not the
\f(CW\s-1bge\s+1\fP
device
(e.g. the embedded Ethernet devices on
many Sun motherboards).
.RS
.LP
Beware that jumbo support in early versions
of Sun's drivers may not have been well-tested,
especially in the ways AoE uses it.
In particular,
the earliest version of the Solaris 10
\f(CW\s-1ce\s+1\fP
driver
may be prone to crash when used with AoE.
It is wise to patch such drivers to their
most-recent versions before using jumbos.
.RE
.IP 3.
If there are switches in the network
between AoE host and target,
they must all support jumbo frames.
Check the manufacturer's specifications for
the specific models of switch you have.
.PP
.SH
4.4.2.
Enabling jumbo frames
.PP
To make AoE use jumbo frames:
.IP 1.
Enable jumbo frames in the appropriate Solaris device driver.
See Sun's documentation for how to do this,
and for the expected maximum frame size
(usually expressed as MTU,
the maximum Ethernet data payload).
Make sure you set things up so that jumbo frames
are automatically enabled at boot time
for that device.
.IP 2.
If there are any network switches between hosts and targets,
find out the maximum frame size supported by each switch.
If this is smaller than the jumbo size allowed by Solaris,
configure the jumbo frame size by hand,
as explained below.
.IP 3.
If no switches are involved
or all switches support sufficiently large frames,
simply restart AoE or reboot
with jumbo frames enabled in Solaris.
The AoE subsystem will automatically detect
the maximum frame size allowed by the Solaris network driver being used,
reduce it if necessary to match that allowed by the AoE target,
and use that size for subsequent I/O to that target.
.PP
The AoE subsystem can see the frame size
configured in Solaris,
and that available in the AoE target,
but it can't see any limits imposed by switches.
It's up to you to get that right.
.PP
A maximum frame size may be set for each AoE channel.
This may be used only to reduce the frame size,
not to increase it;
the maximum size allowed by Solaris
for the network device used by the channel
cannot be exceeded.
For each target accessed over this channel,
the maximum size is further reduced if necessary
to that supported by that target
(discovered through part of the AoE protocol).
.PP
By default,
AoE uses the largest size supported by the network device.
Manual configuration may be needed
to handle size limits in switches,
or for debugging.
It is not a substitute for properly configuring
Sun's network drivers.
.PP
To specify a maximum frame size,
add any of the following arguments to
the channel's entry in
\f(CW\s-1/etc/aoe.conf\s+1\fP:
.IP
.in -\n(PIu
\f(CW\s-1mtu\s+1\fP \f2n\fP
.in
Frame data-payload size may be no greater than
\f2n\fP
bytes.
If
\f2n\fP
is zero,
the largest size supported by this channel's network device
is used.
.IP
.in -\n(PIu
\f(CW\s-1maxdata\s+1\fP \f2n\fP
.in
AoE ATA data transfers may be no larger than
\f2n\fP
bytes,
rounded down to the nearest multiple of 512.
\f(CW\s-1maxdata\s+1\fP \f2n\fP
means the same as
\f(CW\s-1mtu\s+1\fP \f2n\fP\f(CW\s-1+22\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1jumbo\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1nojumbo\s+1\fP
.in
Shorthand for
\f(CW\s-1mtu \s+1\fP\f(CW\s-10\s+1\fP
(\f(CW\s-1jumbo\s+1\fP)
or
\f(CW\s-1mtu \s+1\fP\f(CW\s-11500\s+1\fP
(\f(CW\s-1nojumbo\s+1\fP).
.PP
The current maximum transfer size
for a particular target
is shown in the
\f(CW\s-1maxdata\s+1\fP
field of the
\f(CW\s-1conf\s+1\fP
\f2kstat\fP
table.
.SH
4.4.3.
Jumbo frames and disk I/O speed
.PP
Is bigger always better?
Apparently not.
.PP
Our tests suggest that
\f(CW\s-1maxdata\s+1\fP
values of at least 3-4 kibibytes
(6-8 sectors)
produce large speed improvements
over the default 1kiB (2 sectors);
reads and writes through the file system
are 2-3 times as fast.
With even larger frames,
the story is not as clear.
For some operations speeds continue to increase,
though not as markedly;
for others there appears to be an optimal size
(varying with type of operation and type of disk)
after which speeds drop a little,
though again not markedly.
.PP
It is not a bad idea just to use the
largest value supported by your hardware and software;
I/O speed will be much better than with standard 1500-byte frames,
and the largest value will cause the least
system overhead within Solaris.
If it's important to squeeze every possible drop of speed
out of your system,
you may want to experiment to find the
\f(CW\s-1maxdata\s+1\fP
value that best matches your network,
device drivers,
disks,
and application mix.
.PP
Those who do run their own tests are invited to
share them with us,
or to contact us for details
about the simple tests we have used so far.
.SH
4.5.
More monitoring
.PP
On Solaris 8 and newer systems,
additional statistics can be read
via a special
\f2kstat\fP(1M)
table called
\f(CW\s-1aoed\s+1\fP\f2inst\fP\f(CW\s-1,stats\s+1\fP
where
\f2inst\fP
is the number appearing in device names
(as
\f(CW\s-1/dev/rdsk/cad\s+1\fP\f2inst\fP\f(CW\s-1s0\s+1\fP).
For example,
to check the statistics for target
\f(CW\s-10/0/5\s+1\fP:
.br
\f(CW\s-1# kstat -n aoed5,stats\s+1\fP
.br
\f(CW\s-1module:\0\s+1\fP\f(CW\s-1aoed\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1instance:\0\s+1\fP\f(CW\s-15\s+1\fP
.br
\f(CW\s-1name:\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1aoed5,stats\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1class:\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1misc\s+1\fP
.br
\f(CW\s-1\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1atarcv\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1316771\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1atasnd\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1316771\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1crtime\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-183.211228318\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1erraoe\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-10\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1errata\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-10\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1errtimeout\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-10\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1errtrunc\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-10\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1errwmsg\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-10\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1retries\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-10\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1snaptime\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1215074.845901558\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1softerr\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-10\s+1\fP
.br
.br
.PP
\f(CW\s-1crtime\s+1\fP
and
\f(CW\s-1snaptime\s+1\fP
are standard values supplied by the system:
when data-gathering began
and when the data shown were last updated,
in seconds since the system booted.
Other values are counters maintained by the AoE disk driver,
separately for each AoE target:
.IP
.in -\n(PIu
\f(CW\s-1atarcv\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1atasnd\s+1\fP
.in
ATA-command responses received and sent.
.IP
.in -\n(PIu
\f(CW\s-1erraoe\s+1\fP
.in
AoE protocol errors.
.IP
.in -\n(PIu
\f(CW\s-1errata\s+1\fP
.in
ATA (disk) errors.
.IP
.in -\n(PIu
\f(CW\s-1errtimeout\s+1\fP
.in
Hard timeouts (no response received
even after several retries).
.IP
.in -\n(PIu
\f(CW\s-1errtrunc\s+1\fP
.in
Truncated AoE responses.
.IP
.in -\n(PIu
\f(CW\s-1errwmsg\s+1\fP
.in
AoE requests that couldn't be sent
(likely a problem with the network channel).
.IP
.in -\n(PIu
\f(CW\s-1retries\s+1\fP
.in
Retransmissions because a response wasn't received
after a standard timeout interval
(that listed as
\f(CW\s-1timeout\s+1\fP
in the
\f(CW\s-1conf\s+1\fP
table,
below).
.IP
.in -\n(PIu
\f(CW\s-1softerr\s+1\fP
.in
Soft (ECC-corrected) errors reported by the drive.
.PP
Static configuration info may be retrieved
from
\f2kstat\fP
table
\f(CW\s-1aoed\s+1\fP\f2inst\fP\f(CW\s-1,conf\s+1\fP:
.br
\f(CW\s-1# kstat -n aoed5,conf\s+1\fP
.br
\f(CW\s-1module:\0\s+1\fP\f(CW\s-1aoed\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1instance:\0\s+1\fP\f(CW\s-15\s+1\fP
.br
\f(CW\s-1name:\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1aoed5,conf\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1class:\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1misc\s+1\fP
.br
\f(CW\s-1\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1crtime\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-183.211229535\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1cylsize\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-156960\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1maxbuf\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-13\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1maxdata\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-11024\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1mibsize\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-139266\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1snaptime\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1215076.003383030\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1timeout\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1200\s+1\fP
.br
.br
The
\f(CW\s-1aoed\s+1\fP-specific
values are:
.IP
.in -\n(PIu
\f(CW\s-1cylsize\s+1\fP
.in
Number of sectors per cylinder for this drive,
discovered from the disk
or (more often) invented by the driver.
.IP
.in -\n(PIu
\f(CW\s-1maxbuf\s+1\fP
.in
Maximum number of unacknowledged AoE commands
allowed by this target.
.IP
.in -\n(PIu
\f(CW\s-1maxdata\s+1\fP
.in
Maximum data-segment size
agreed among
\f2aoestart\fP,
network device,
and target.
.IP
.in -\n(PIu
\f(CW\s-1mibsize\s+1\fP
.in
Size of this disk,
in mebibytes.
.IP
.in -\n(PIu
\f(CW\s-1timeout\s+1\fP
.in
I/O operation timeout,
in milliseconds.
.SH
4.6.
Tools and control programs
.PP
See the manual pages supplied with the software
for details.
Except as noted,
programs are installed in directory
\f(CW\s-1/usr/sbin\s+1\fP,
normally part of the super-user's shell search path.
.SH
4.6.1.
\f2Aoectl\fP
.PP
\f2Aoectl\fP
issues control commands
to the AoE subsystem:
.IP
.in -\n(PIu
\f(CW\s-1aoectl list\s+1\fP \f2address\fP \&.\|.\|.
.in
List enabled
targets at each
\f2address\fP;
if none given,
all enabled targets.
.IP
.in -\n(PIu
\f(CW\s-1aoectl enable\s+1\fP \f2address\fP \&.\|.\|.
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1aoectl disable\s+1\fP \f2address\fP \&.\|.\|.
.in
Enable or disable targets at the
\f2addresses\fP.
.IP
.in -\n(PIu
\f(CW\s-1aoectl probe\s+1\fP \f2address\fP \&.\|.\|.
.in
Broadcast a check-in request
for each target
\f2address\fP;
if none given,
for the wildcard address on every active
\f2aoechan\fP.
Any responding targets are enabled by
\f2aoemon\fP.
.LP
\f2Addresses\fP
are target addresses like
\f(CW\s-10/2/4\s+1\fP.
If an address field is missing or empty,
it matches any value.
.PP
For example:
.IP
.in -\n(PIu
\f(CW\s-1aoectl \s+1\fP\f(CW\s-1list \s+1\fP\f(CW\s-10\s+1\fP
.in
List all enabled targets on
\f2aoechan\fP
0.
.IP
.in -\n(PIu
\f(CW\s-1aoectl \s+1\fP\f(CW\s-1probe \s+1\fP\f(CW\s-10/0/5 \s+1\fP\f(CW\s-10/0/7 \s+1\fP\f(CW\s-10//2\s+1\fP
.in
Probe for
\f(CW\s-10/0/5\s+1\fP,
\f(CW\s-10/0/7\s+1\fP,
and any target on
\f2aoechan\fP
0
with
\f2aoemin\fP
2 no matter what its
\f2aoemaj\fP.
.IP
.in -\n(PIu
\f(CW\s-1aoectl \s+1\fP\f(CW\s-1disable \s+1\fP\f(CW\s-10/1\s+1\fP
.in
Disable all targets with
\f2aoechan\fP
0,
\f2aoemaj\fP
1.
.IP
.in -\n(PIu
\f(CW\s-1aoectl \s+1\fP\f(CW\s-1list\s+1\fP
.in
List all enabled targets.
.IP
.in -\n(PIu
\f(CW\s-1aoectl \s+1\fP\f(CW\s-1probe\s+1\fP
.in
Broadcast a wildcard check-in request
to every active
\f2aoechan\fP.
.PP
\f2Aoectl\fP
need not be used during normal operation,
only in exceptional cases
or for monitoring.
\f(CW\s-1aoectl\0\s+1\fP\f(CW\s-1list\s+1\fP
helps spot targets that should be there but aren't;
\f(CW\s-1aoectl\0\s+1\fP\f(CW\s-1probe\s+1\fP
may be useful if the network is rearranged
while the system is running;
\f(CW\s-1aoectl\0\s+1\fP\f(CW\s-1disable\s+1\fP
will prevent programs like
\f2format\fP
from stalling
because a target has broken or been removed.
.SH
4.6.2.
\f2Aoestart\fP
and
\f2aoestop\fP
.PP
\f2Aoestart\fP
starts a single
\f2aoechan\fP
according to its arguments.
It is called automatically for each channel named in
\f(CW\s-1/etc/aoe.conf\s+1\fP
when
AoE is started;
normally it need not be run by hand.
In fact each line of
\f(CW\s-1/etc/aoe.conf\s+1\fP
is simply used
as arguments to
\f2aoestart\fP.
.PP
\f2Aoestop\fP
stops one,
several,
or all
(\f(CW\s-1-a\s+1\fP)
named
\f2aoechans\fP.
Normally there is no need to do this at all.
.SH
4.6.3.
\f2Aoemkconf\fP
.PP
\f2Aoemkconf\fP
turns target addresses
into device declarations
suitable for inclusion in
\f(CW\s-1aoed.conf\s+1\fP.
It simply prints the entries on the standard output;
it's up to you to edit them into the configuration file.
.PP
An argument like form
\f(CW\s-10/7/3\s+1\fP
names a single target;
\f(CW\s-10/7/9-11\s+1\fP
is shorthand for
\f(CW\s-10/7/9 \s+1\fP\f(CW\s-10/7/10 \s+1\fP\f(CW\s-10/7/11\s+1\fP.
Thus
.RS
\f(CW\s-1aoemkconf 0/10/0-14 0/11/0-14\s+1\fP
.RE
generates entries for two 15-slot EtherDrive shelves numbered 10 and 11.
Instance numbers use the standard decimal encoding.
.PP
Arguments that don't look like target addresses
are taken to be existing configuration files;
\f2aoemkconf\fP
silently omits entries whose instance numbers
would duplicate any in the files.
Hence if
\f(CW\s-1/usr/kernel/drv/aoed.conf\s+1\fP
already has some entries for shelf 10,
.RS
\f(CW\s-1aoemkconf /usr/kernel/drv/aoed.conf 0/10/0-15\s+1\fP
.RE
will print only the missing ones.
.SH
4.6.4.
\f2Aoelabinit\fP
.PP
\f2Aoelabinit\fP
writes a Solaris disk label to the single disk
named as an argument:
.RS
\f(CW\s-1aoelabinit /dev/rdsk/cad209\s+1\fP
.br
.RE
If the disk is already labelled,
option
\f(CW\s-1-f\s+1\fP
(force)
must be given:
.RS
\f(CW\s-1aoelabinit -f /dev/rdsk/cad209\s+1\fP
.br
.RE
.PP
By default,
\f2aoelabinit\fP
invents a label as follows:
.IP \(bu
If the disk is no larger than 1TiB:
VTOC label;
subdevices
\f(CW\s-1s0\s+1\fP
and
\f(CW\s-1s2\s+1\fP
access the whole user-data part of the disk,
excluding system-reserved areas.
.IP \(bu
If the disk is larger:
EFI label;
\f(CW\s-1s0\s+1\fP
spans the whole user-data part of the disk,
excluding disk labels
and a small mandatory system-reserved area.
.PP
The defaults may be overridden in several ways;
see the manual page.
.PP
\f2Aoelabinit\fP
is not meant to be a general-purpose disk labeller,
just a workaround for the
\f2format\fP
bug
forbidding access to large unlabelled disks.
.SH
4.6.5.
\f2Aoemon\fP
.PP
\f2Aoemon\fP
is a daemon
that processes messages
from the kernel AoE subsystem.
It has two main functions:
.IP 1.
When a check-in response arrives,
\f2aoemon\fP
enables the corresponding target,
and logs that it did so.
.IP 2.
When an error or an unexpected event occurs
(e.g. a message arrives when the AoE disk driver is inactive,
or in response to a request the system didn't make,
or appears to be mangled)
\f2aoemon\fP
logs the details.
.PP
\f2Aoemon\fP
uses
\f2syslog\fP
facility
\f(CW\s-1daemon\s+1\fP.
Severity
\f(CW\s-1info\s+1\fP
is used to
report that a target has been enabled;
\f(CW\s-1notice\s+1\fP
when an unexpected or ill-formed AoE message arrives;
\f(CW\s-1error\s+1\fP
for error conditions
that prevent
\f2aoemon\fP
from working at all.
.PP
\f2Aoemon\fP
is started automatically when
AoE is started.
Only one copy is needed for all
\f2aoechans\fP.
If
\f2aoemon\fP
is not running,
AoE disk devices will still work,
but targets will not be automatically enabled
and some errors may go unreported.
.SH
4.6.6.
\f2Aoeunlabel\fP
.PP
\f2Aoeunlabel\fP
destroys any Solaris disk labels
on the devices named as arguments:
.RS
\f(CW\s-1aoeunlabel /dev/rdsk/cad312\s+1\fP
.br
.RE
Both VTOC and EFI labels are destroyed.
.PP
\f2Aoeunlabel\fP
is unlikely to be needed
except when a disk already EFI-labelled
by a newer system
is to be repartitioned
on an older one.
.SH
4.6.7.
Starting and stopping AoE
.PP
.SH
4.6.7.1.
With SMF
.PP
On an SMF system,
AoE is represented as service
\f(CW\s-1svc:/device/aoe\s+1\fP.
Normal SMF tools may be used:
.IP
.in -\n(PIu
\f(CW\s-1svcs\0\s+1\fP\f(CW\s-1svc:/device/aoe\s+1\fP
.in
Display the state of the service:
\f(CW\s-1online\s+1\fP
if all is well;
\f(CW\s-1disabled\s+1\fP
if not enabled;
\f(CW\s-1maintenance\s+1\fP
or
\f(CW\s-1offline\s+1\fP
if the service couldn't be started.
In the latter case, log files
\f(CW\s-1/var/svc/log/device-aoe:default.log\s+1\fP
and
\f(CW\s-1/etc/svc/volatile/device-aoe:default.log\s+1\fP
may offer clues.
.IP
.in -\n(PIu
\f(CW\s-1svcadm\0\s+1\fP\f(CW\s-1enable\0\s+1\fP\f(CW\s-1svc:/device/aoe\s+1\fP
.in
Start AoE if not already running:
start
\f2aoemon\fP,
then call
\f2aoestart\fP
for each channel listed in
\f(CW\s-1aoe.conf\s+1\fP.
Remember that AoE should be started automatically
when the system boots.
.IP
.in -\n(PIu
\f(CW\s-1svcadm\0\s+1\fP\f(CW\s-1disable\0\s+1\fP\f(CW\s-1svc:/device/aoe\s+1\fP
.in
Abruptly stop AoE if it was running:
call
\f2aoestop\fP
for each active channel,
then kill
\f2aoemon\fP.
Remember that AoE should not be started on boot.
All AoE disks become inaccessible,
even if still mounted or open.
.IP
.in -\n(PIu
\f(CW\s-1svcadm\0\s+1\fP\f(CW\s-1clear\0\s+1\fP\f(CW\s-1svc:/device/aoe\s+1\fP
.in
Reset the service from
\f(CW\s-1maintenance\s+1\fP
state,
e.g. if it failed to start when last enabled.
.IP
.in -\n(PIu
\f(CW\s-1svcadm\0\s+1\fP\f(CW\s-1refresh\0\s+1\fP\f(CW\s-1svc:/device/aoe\s+1\fP
.in
Shut down any currently-active AoE channels,
and restart AoE with the channels
now listed in
\f(CW\s-1/etc/aoe.conf\s+1\fP.
Equivalent to
\f(CW\s-1svcadm\0\s+1\fP\f(CW\s-1disable\0\s+1\fP\f(CW\s-1svc:/device/aoe;\s+1\fP
\f(CW\s-1svcadm\0\s+1\fP\f(CW\s-1enable\0\s+1\fP\f(CW\s-1svc:/device/aoe\s+1\fP,
but quicker to type
and less disruptive to active AoE devices.
.PP
Once enabled,
the AoE service is started automatically
early in the boot process.
In particular the services that start swap devices
and mount local file systems
depend on
\f(CW\s-1svc:/device/aoe\s+1\fP,
and AoE is started even for the single-user milestone,
though if AoE fails to start
single-user mode still works.
.PP
Usually it is safe to abbreviate
the service name to
\f(CW\s-1aoe\s+1\fP.
.SH
4.6.7.2.
Without SMF
.PP
On a non-SMF system,
\f(CW\s-1/etc/init.d/aoe\s+1\fP
is a conventional boot-and-shutdown shell script
to start and shut down the AoE subsystem:
.IP
.in -\n(PIu
\f(CW\s-1/etc/init.d/aoe \s+1\fP\f(CW\s-1start\s+1\fP
.in
Start AoE:
start
\f2aoemon\fP,
call
\f2aoestart\fP
for each channel listed in
\f(CW\s-1aoe.conf\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1/etc/init.d/aoe \s+1\fP\f(CW\s-1stop\s+1\fP
.in
Abruptly shut AoE down:
call
\f2aoestop\fP
to stop all channels,
then kill
\f2aoemon\fP.
All AoE disks become inaccessible,
even if still mounted or open.
.IP
.in -\n(PIu
\f(CW\s-1/etc/init.d/aoe \s+1\fP\f(CW\s-1restart\s+1\fP
.in
Shut down any currently-active AoE channels,
start
\f2aoemon\fP
if it is not already running,
then call
\f2aoestart\fP
for each channel listed in
\f(CW\s-1/etc/aoe.conf\s+1\fP.
Equivalent to
\f(CW\s-1aoe\0\s+1\fP\f(CW\s-1stop; \s+1\fP\f(CW\s-1aoe\0\s+1\fP\f(CW\s-1start\s+1\fP
but quicker to type and to execute,
and less disruptive to active AoE devices.
.LP
AoE is started
very early during the boot process,
before any file system but the root and
\f(CW\s-1/usr\s+1\fP
has been mounted;
thus any other file system
may be put on an AoE disk,
but all files used by the
\f(CW\s-1aoe\s+1\fP
script itself
(the kernel drivers,
\f2aoestart\fP,
\f2aoemon\fP)
must be in the root or
\f(CW\s-1/usr\s+1\fP.
.SH
4.6.7.3.
Customization
.PP
With or without SMF,
parts of the initialization script
may be customized
by placing shell-variable declarations in file
\f(CW\s-1/etc/default/aoe.options\s+1\fP.
Here are some of the things that may be set:
.IP
.in -\n(PIu
\f(CW\s-1AOEBIN=\s+1\fP\f2dir\fP
.in
Where to find
\f2aoestart\fP
and
\f2aoemon\fP;
default
\f(CW\s-1/usr/sbin\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1AOECONF=\s+1\fP\f2file\fP
.in
Where to find the file
declaring channels;
default
\f(CW\s-1/etc/aoe.conf\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1AOESTARTOPTS="\s+1\fP\f2options\fP\f(CW\s-1"\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1AOEMONOPTS="\s+1\fP\f2options\fP\f(CW\s-1"\s+1\fP
.in
Additional options for
\f2aoestart\fP
or
\f2aoemon\fP;
default empty.
.IP
.in -\n(PIu
\f(CW\s-1AOEDIR=\s+1\fPdir
.in
Directory for open-channel files;
default
\f(CW\s-1/etc/aoe\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1AOEDIRPERM=\s+1\fP\f2perms\fP
.in
Permissions with which to create
\f(CW\s-1$AOEDIR\s+1\fP
if it doesn't exist;
default
\f(CW\s-1700\s+1\fP,
i.e. only the owner (the super-user)
may look within.
.IP
.in -\n(PIu
\f(CW\s-1AOEPRESTART="\s+1\fP\f2shell-commands\fP\f(CW\s-1"\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1AOEPOSTSTART="\s+1\fP\f2shell-commands\fP\f(CW\s-1"\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1AOEPRESTOP="\s+1\fP\f2shell-commands\fP\f(CW\s-1"\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1AOEPOSTSTOP="\s+1\fP\f2shell-commands\fP\f(CW\s-1"\s+1\fP
.in
Additional shell commands to be executed
just before or just after starting channels,
or just before or just after stopping them.
May be helpful if magic commands are required
to initialize network interfaces.
.SH
4.7.
Driver and device file details
.PP
The AoE subsystem comprises three distinct device drivers.
.PP
A network device is made into an AoE channel
by opening it,
issuing Solaris DLPI commands
to set device unit number and Ethernet protocol type,
and pushing an instance of the
\f2aoecomm\fP
STREAMS module.
There is a little more configuration
to tell
\f2aoecomm\fP
the desired
\f2aoechan\fP
number,
and to protect against denial-of-service attacks
(anybody could push
\f2aoecomm\fP
on a pipe,
for example).
\f2Aoestart\fP
does all this.
.PP
The
\f2aoed\fP
driver affords disk access,
through both block and character devices.
It uses an internal call
to inform
\f2aoecomm\fP
of its interest in incoming AoE messages,
and another call to send messages out.
It also calls
\f2aoecomm\fP
when a device is opened,
to see whether the corresponding target has been enabled;
if not,
the
\f2open\fP
call fails.
\f2Aoed\fP
creates 18
\f(CW\s-1/devices/pseudo/aoed@\(**\s+1\fP\f(CW\s-1\s+1\fP
devices
for each target:
the eight standard block subdevices
\f(CW\s-1:a\s+1\fP-\f(CW\s-1:h\s+1\fP,
their raw counterparts
\f(CW\s-1:a,raw\s+1\fP-\f(CW\s-1:h,raw\s+1\fP,
and whole-disk subdevices
\f(CW\s-1:wd\s+1\fP
and
\f(CW\s-1:wd,raw\s+1\fP.
.PP
The
\f2aoectl\fP
driver provides two control devices:
.IP
.in -\n(PIu
\f(CW\s-1/devices/pseudo/aoectl@0:ctl\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/dev/aoectl\s+1\fP
.in
Written by the
\f2aoectl\fP
program to issue control commands,
by
\f2aoemon\fP
to enable devices,
and by
\f2aoestart\fP
as part of channel configuration.
.IP
.in -\n(PIu
\f(CW\s-1/devices/pseudo/aoectl@0:mon\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/dev/aoemon\s+1\fP
.in
Read by
\f2aoemon\fP.
May not be reopened while open;
hence cannot be opened again
while
\f2aoemon\fP
is running,
not even by another copy of
\f2aoemon\fP.
.LP
The
\f2aoectl\fP
driver calls
\f2aoecomm\fP
to send AoE messages,
to register its interest in unexpected,
ill-formed,
or error-reporting AoE messages
(to be made available through
\f(CW\s-1/dev/aoemon\s+1\fP),
and to query or change the enabled-target list.
.PP
Each of these drivers is loaded into the kernel
as needed:
when the STREAMS module is pushed,
or one of the device files is opened.
Because the
\f2aoed\fP
and
\f2aoectl\fP
drivers depend on
\f2aoecomm\fP,
loading the former
automatically loads the latter first,
and
\f2aoecomm\fP
cannot be unloaded while
either of the other two modules is loaded.
