/*
 * Copyright 2005-2007 Coraid, Inc.
 * See accompanying file doc/LICENSE for conditions
 */

/*
 * generate names for attachment points;
 * attach and detach
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>	/* for creat */
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <stropts.h>
#include "aoecmd.h"

#define	MAXPATH	100
#define	CRMODE	0666	/* for newly-created mount points */

static char *basedir = "/etc/aoe";
static char *prefix = "ch";

static char *achname(int, char *, int);
static int nametoch(char *);
static int achfdetach(char *);
static int ismounted(struct stat *);

/*
 * set or fetch base directory name
 */
char *
achdir(newdir)
char *newdir;
{
	char *olddir;

	olddir = basedir;
	if (newdir)
		basedir = newdir;
	return (olddir);
}

/*
 * attach fd as channel chan
 */
int
achattach(fd, chan)
int fd;
int chan;
{
	char name[MAXPATH];
	struct stat st;
	int cfd;

	if (achname(chan, name, sizeof(name)) == NULL)
		return (-1);
	if (stat(name, &st) >= 0) {
		(void) achfdetach(name);
		if (!S_ISREG(st.st_mode))
			return (-1);
	} else {
		if ((cfd = creat(name, CRMODE)) < 0)
			return (-1);
		close(cfd);
	}
	if (fattach(fd, name) < 0)
		return (-1);
	return (1);
}

/*
 * if chan < 0, detach everything we can find
 * return value is
 *	> 0	detached something
 *	== 0	found nothing to detach
 *	< 0	error
 */
int
achdetach(chan)
int chan;
{
	char name[MAXPATH];
	struct stat st;
	DIR *dp;
	struct dirent *ep;
	int n, ch;

	if (chan >= 0) {
		if (achname(chan, name, sizeof(name)) == NULL)
			return (-1);
		if (stat(name, &st) < 0) {
			if (errno == ENOENT)
				return (0);
			return (-1);
		}
		return (achfdetach(name));
	} else {
		if ((dp = opendir(basedir)) == NULL)
			return (0);
		n = 0;
		while ((ep = readdir(dp)) != NULL) {
			if ((ch = nametoch(ep->d_name)) < 0)
				continue;
			if (verbose || debug)
				fprintf(stderr, "detach %s/%s = aoechan %d\n",
					basedir, ep->d_name, ch);
			if (achdetach(ch) > 0)
				n = 1;
		}
		closedir(dp);
		return (n);
	}
}

static char *
achname(chan, buf, len)
int chan;
char *buf;
int len;
{
	if (chan < 0)
		return (NULL);
	if (snprintf(buf, len, "%s/%s%02d", basedir, prefix, chan) >= len)
		return (NULL);
	return (buf);
}

static int
nametoch(name)
char *name;
{
	char *p;

	for (p = prefix; *p; name++, p++)
		if (*name != *p)
			return (-1);
	for (p = name; *p; p++)
		if (!isdigit(*p))
			return (-1);
	return (atoi(name));
}

/*
 * return value is
 *	> 0	detached something
 *	== 0	found nothing to detach
 *	< 0	error
 */
static int
achfdetach(mpt)
char *mpt;
{
	if (fdetach(mpt) >= 0)
		return (1);
	if (errno == EINVAL)
		return (0);
	return (-1);
}

static int
ismounted(stp)
struct stat *stp;
{
	if (S_ISCHR(stp->st_mode)
	||  S_ISFIFO(stp->st_mode))
		return (1);
	return (0);
}
