/*
 * Copyright 2005 Coraid, Inc.
 * See accompanying file doc/LICENSE for conditions
 */

/*
 * read/write with timeout
 * beware: no protection against
 * partial read or write,
 * if the file in question doesn't work atomically
 */

#include <stdlib.h>
#include <setjmp.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include "aoecmd.h"

static jmp_buf tjmp;

static void msalarm(int);
static void onalarm();

int
tread(fd, buf, len, ms)
int fd;
void *buf;
int len;
int ms;
{
	int n;
	void (*osig)();

	if (setjmp(tjmp)) {
		signal(SIGALRM, osig);
		errno = EINTR;
		return (-1);
	}
	osig = signal(SIGALRM, onalarm);
	msalarm(ms);
	n = read(fd, buf, len);
	msalarm(0);
	signal(SIGALRM, osig);
	return (n);
}

int
twrite(fd, buf, len, ms)
int fd;
void *buf;
int len;
int ms;
{
	int n;
	void (*osig)();

	if (setjmp(tjmp)) {
		signal(SIGALRM, osig);
		errno = EINTR;
		return (-1);
	}
	osig = signal(SIGALRM, onalarm);
	msalarm(ms);
	n = write(fd, buf, len);
	msalarm(0);
	signal(SIGALRM, osig);
	return (n);
}

static void
msalarm(ms)
int ms;
{
	struct itimerval t;

	t.it_interval.tv_sec = 0;
	t.it_interval.tv_usec = 0;
	t.it_value.tv_sec = ms/1000;
	t.it_value.tv_usec = (ms%1000)*1000;
	setitimer(ITIMER_REAL, &t, NULL);
}

static void
onalarm()
{
	longjmp(tjmp, 1);
}
