.TL
A tour of the
Solaris ATA-over-Ethernet (AoE) implementation
.AU
Norman Wilson
2007 August 15
.SH
1.
Introduction
.PP
This is a description of the
Solaris ATA-over-Ethernet (AoE) device-driver implementation,
version 1.4.
It includes:
.IP \(bu
An overview of the design,
listing all components.
.IP \(bu
A description of
how to compile the source code
into binary programs,
and how to assemble the binaries
into a Solaris package for distribution.
.IP \(bu
A detailed tour of the implementation:
notes on coding style
and
an overview of the pieces and how they interact,
followed by a detailed walk-through of
each piece.
.IP \(bu
A collection of notes
on design decisions,
mistakes that should be corrected in future versions,
and other arcana.
.PP
This is not a tutorial on
the Solaris operating system,
device-driver and STREAMS subsystems,
or kernel;
the AoE protocol;
or
C programming.
Neither does it explain how to install and operate
the AoE subsystem.
For background on these topics
and others,
the reader should have access
to the following references:
.IP
.in -\n(PIu
Solaris AoE-over-Ethernet (AoE) Installation and Operation Guide
.in
Companion to the present document;
\f(CW\s-1/opt/CORDaoe/doc/aoe-guide.html\s+1\fP
in the customer distribution.
.IP
.in -\n(PIu
Writing Device Drivers
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
STREAMS Programming Guide
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
Application Packaging Developer's Guide
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
Solaris System Administration Guide
.in
Official Sun reference books,
available in HTML or PDF from
\f(CW\s-1http://docs.sun.com\s+1\fP.
There are different editions
for different versions of Solaris.
.IP
.in -\n(PIu
AoE (ATA over Ethernet)
.in
AoE protocol specification,
available online from
\f(CW\s-1http://www.coraid.com\s+1\fP.
.IP
.in -\n(PIu
Data Link Provider Interface (DLPI) Version 2
.in
Ornate setup protocol
used by Solaris network devices,
available online from
\f(CW\s-1http://www.opengroup.org\s+1\fP.
.IP
.in -\n(PIu
Sundry Solaris manual entries
.in
Cited as needed in the text below,
in the conventional
\f2entry-name\fP(section)
form.
Available on a running Solaris system
or at
\f(CW\s-1http://docs.sun.com\s+1\fP.
.PP
Certain parts of this subsystem
behave differently when
newer Solaris features are present:
.IP \(bu
Driver initialization and shutdown
make use of the Service Management Facility (SMF)
if it was present and active
when the package is installed;
the older
\f2init.d\fP(4)
mechanism if not.
Solaris 10 normally has SMF;
older versions don't.
.IP \(bu
The
newer EFI label format,
which allows disks larger than 1TiB,
is fully supported
only when Solaris contains the corresponding support.
Solaris 9 8/03 and later systems,
or earlier Solaris 9 with appropriate patches,
fully supports large disks;
earlier unpatched Solaris 9 systems
or any earlier versions of the system
do not.
.PP
This document uses the new binary-multiple prefixes
described in the IEEE 1541-2002 standard.
Prefixes like kilo, mega, giga, tera
(k, M, G, T)
refer to powers of ten;
when the near-equivalent powers of two are meant,
they are called kibi, mebi, gibi, tebi
(ki, Mi, Gi, Ti).
For example,
a gigabyte (GB) is
10\u\s-19\s+1\d
bytes,
while
a gibibyte (GiB) is
2\u\s-130\s+1\d
(1073741824).
For more details
see
\f(CW\s-1http://physics.nist.gov/cuu/Units/binary.html\s+1\fP.
.SH
2.
Components
.PP
The AoE subsystem
comprises several modules executing in kernel mode
and several user-mode utility programs.
.PP
These are the kernel parts.
Each is a loadable kernel module,
supplied in both 32- and 64-bit binary versions.
.IP
.in -\n(PIu
\f(CW\s-1aoecomm\s+1\fP
.in
STREAMS module (line discipline).
An instance is pushed on each communication channel
(usually an Ethernet device)
over which AoE messages will be received and sent.
Other AoE kernel modules make internal subroutine calls
to
\f(CW\s-1aoecomm\s+1\fP
to send messages,
to arrange to receive messages,
and to query and update a registry of AoE targets
known to exist.
.IP
.in -\n(PIu
\f(CW\s-1aoectl\s+1\fP
.in
Character device driver
with one statically-configured instance.
Supplies two control devices:
\f(CW\s-1/dev/aoectl\s+1\fP,
to which configuration commands may be written;
\f(CW\s-1/dev/aoemon\s+1\fP,
from which error reports and unsolicted AoE messages
may be read.
.IP
.in -\n(PIu
\f(CW\s-1aoed\s+1\fP
.in
Block-and-character disk device driver.
One instance configured for each possible AoE target disk.
Provides disk devices in the
\f(CW\s-1/dev/dsk\s+1\fP
and
\f(CW\s-1/dev/rdsk\s+1\fP
directories
implementing normal Solaris disk semantics,
sufficient to allow file systems to be created and mounted
and to support standard maintenance programs
like
\f2fsck\fP
and
\f2format\fP(1M).
Implements the required Solaris disk-specific
\f(CW\s-1ioctl\s+1\fP
calls,
including some Sun doesn't bother to document.
.PP
These are the
user-mode parts.
Except as noted,
each is a binary executable program
installed in directory
\f(CW\s-1/sbin\s+1\fP.
Only 32-bit binaries are supplied;
they run fine on 64-bit systems.
.IP
.in -\n(PIu
\f2aoestart\fP
.in
Set up an AoE communication channel
on a specified communication device.
.IP
.in -\n(PIu
\f2aoestop\fP
.in
Shut down a specified channel or channels.
.IP
.in -\n(PIu
\f2aoectl\fP
.in
Perform control operations through the
\f(CW\s-1aoectl\s+1\fP
driver;
e.g. query or tweak the active-target registry,
broadcast a Query-Config request to cause targets
to identify themselves.
.IP
.in -\n(PIu
\f2aoemon\fP
.in
Read the monitoring device supplied by the
\f(CW\s-1aoectl\s+1\fP
driver,
logging errors and unusual events
and updating the registry as Query-Config responses
are received.
.IP
.in -\n(PIu
\f2aoemkconf\fP
.in
Given a list of AoE target addresses,
generate the kernel configuration file entry
required to declare each device to
\f(CW\s-1aoed\s+1\fP.
Entries can be written by hand as well;
this is just a convenience.
.IP
.in -\n(PIu
\f2aoelabinit\fP
.in
Label an empty disk.
Needed to work around a bug in
\f2format\fP(1M).
.IP
.in -\n(PIu
\f2aoeunlabel\fP
.in
Wipe a disk of all labels,
to reuse an EFI-labelled disk
on a pre-EFI system.
.IP
.in -\n(PIu
.IP
.in -\n(PIu
\f2aoe\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1aoe.xml\s+1\fP
.in
Start or stop the AoE subsystem,
calling
\f2aoestart\fP
for each channel named in configuration file
\f(CW\s-1/etc/aoe.conf\s+1\fP.
On a non-SMF system,
\f2aoe\fP
is stored in directory
\f(CW\s-1/etc/init.d\s+1\fP
and linked into
\f(CW\s-1/etc/rc2.d\s+1\fP,
so it will be run when the system boots.
If SMF is installed and active,
\f2aoe\fP
is stored in directory
\f(CW\s-1/lib/svc/manifest/device-aoe\s+1\fP
and
\f(CW\s-1aoe.xml\s+1\fP
added to the SMF inventory,
so that AoE will be started when service
\f(CW\s-1svc:/device/aoe\s+1\fP
is enabled.
.PP
All of these programs,
along with default configuration files
and manual pages and other documentation,
are bundled into a single Solaris package
named
\f(CW\s-1CORDaoe\s+1\fP
for distribution.
To install AoE,
a system administrator runs
\f2pkgadd\fP(1M),
edits a configuration file or two,
and starts the subsystem:
on a non-SMF system,
\f(CW\s-1/etc/init.d/aoe\0\s+1\fP\f(CW\s-1start\s+1\fP
or a reboot;
with SMF,
\f(CW\s-1svcadm\0\s+1\fP\f(CW\s-1enable\0\s+1\fP\f(CW\s-1svc:/device/aoe\s+1\fP.
See the
Installation and Operation Guide
for further operational details.
.SH
3.
Source tree layout
.PP
The source code tree contains these directories:
.IP
.in -\n(PIu
\f(CW\s-1kern\s+1\fP
.in
Source files for kernel modules,
including header files used only by those modules.
Object files
and 32-bit binaries
are kept in the main
\f(CW\s-1kern\s+1\fP
directory,
64-bit binaries
(whether for SPARC or x86)
in subdirectory
\f(CW\s-1kern/bin64\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1user\s+1\fP
.in
Source files for user-mode utility programs.
Object files
and binaries are stored here too.
.IP
.in -\n(PIu
\f(CW\s-1include\s+1\fP
.in
Include files
shared by user-mode and kernel code.
.IP
.in -\n(PIu
\f(CW\s-1man\s+1\fP
.in
Source files for manual entries,
in
\f2troff\fP
\f(CW\s-1-man\s+1\fP
format.
.IP
.in -\n(PIu
\f(CW\s-1doc\s+1\fP
.in
Source files for longer documents
(including this one),
in
\f2fidl\fP
format.
\f2Fidl\fP
is an experimental formatter-independent
document language;
see the author for details.
.IP
.in -\n(PIu
\f(CW\s-1pkg\s+1\fP
.in
Where the installable package is built.
Subdirectory
\f(CW\s-1pkg/data\s+1\fP
contains files to be assembled into the eventual package,
copied there by
\f(CW\s-1make \s+1\fP\f(CW\s-1pkg\s+1\fP
in each of the directories above.
The resulting package is stored in subdirectory
\f(CW\s-1pkg/CORDaoe\s+1\fP
in file system format,
and file
\f(CW\s-1pkg/CORDaoe.Z\s+1\fP
in compressed stream format.
.SH
4.
Building programs and package
.PP
The build process is controlled by
\f2make\fP(1).
\f(CW\s-1Makefile\s+1\fPs
come ready-to-use;
there is no system-dependent configuration process,
automatic or otherwise.
.PP
Sun's compilation tools should be used:
Solaris
\f2make\fP,
included in the standard
\f(CW\s-1SUNWsprot\s+1\fP
package,
as
\f(CW\s-1/usr/ccs/bin/make\s+1\fP;
Sun Studio C,
available at no cost from
\f(CW\s-1www.opensolaris.org\s+1\fP.
\f2Gcc\fP
is not recommended,
and probably cannot correctly compile
the current kernel-mode code.
.PP
At present the code may be compiled
only on Solaris 10,
though the resulting binaries will run on systems
as old as
Solaris 7.
.PP
Each source-file directory
\f(CW\s-1kern\s+1\fP,
\f(CW\s-1user\s+1\fP,
\f(CW\s-1man\s+1\fP,
\f(CW\s-1doc\s+1\fP
has a
\f(CW\s-1Makefile\s+1\fP
with these targets
(except as noted):
.IP
.in -\n(PIu
\f(CW\s-1make\s+1\fP (default target)
.in
Build target files from source files:
compile and load binary programs,
format documents.
.IP
.in -\n(PIu
\f(CW\s-1make \s+1\fP\f(CW\s-1install\s+1\fP
.in
Build targets if necessary,
then install in standard places
on the local system.
Used only during development;
not for customer use,
and not guaranteed to work on any system
save the author's.
Not in
\f(CW\s-1man/Makefile\s+1\fP
or
\f(CW\s-1doc/Makefile\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1make \s+1\fP\f(CW\s-1pkg\s+1\fP
.in
Build targets if necessary,
then copy to the staging area in directory
\f(CW\s-1pkg/data\s+1\fP.
Create subdirectories within
\f(CW\s-1pkg/data\s+1\fP
as needed.
.IP
.in -\n(PIu
\f(CW\s-1make \s+1\fP\f(CW\s-1clean\s+1\fP
.in
Remove generated targets and intermediate files,
leaving only the original source files.
.PP
\f(CW\s-1pkg/Makefile\s+1\fP
has these targets:
.IP
.in -\n(PIu
\f(CW\s-1make \s+1\fP\f(CW\s-1pkg\s+1\fP
.in
Use the current contents of staging area
\f(CW\s-1pkg/Makefile\s+1\fP
to create two copies of the installable package:
one in file system format
(a collection of individual files)
in directory
\f(CW\s-1pkg/CORDaoe\s+1\fP,
one in compressed stream format
(a single file)
in file
\f(CW\s-1CORDaoe.Z\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1make \s+1\fP\f(CW\s-1clean\s+1\fP
.in
Remove intermediate files
(i.e. empty the staging area);
leave installable packages,
if they exist.
.IP
.in -\n(PIu
\f(CW\s-1make \s+1\fP\f(CW\s-1clobber\s+1\fP
.in
Empty the staging area
and remove the packages,
leaving only
\f(CW\s-1Makefile\s+1\fP
and the control files.
.PP
The root directory of the source tree contains an overall
\f(CW\s-1Makefile\s+1\fP
with these targets:
.IP
.in -\n(PIu
\f(CW\s-1make\s+1\fP (default target)
.in
Compile everything:
run
\f(CW\s-1make\s+1\fP
in
\f(CW\s-1kern\s+1\fP,
\f(CW\s-1user\s+1\fP,
\f(CW\s-1man\s+1\fP,
and
\f(CW\s-1doc\s+1\fP,
in that order.
.IP
.in -\n(PIu
\f(CW\s-1make \s+1\fP\f(CW\s-1pkg\s+1\fP
.in
Compile everything, then build packages:
run
\f(CW\s-1make \s+1\fP\f(CW\s-1pkg\s+1\fP
in
\f(CW\s-1kern\s+1\fP,
\f(CW\s-1user\s+1\fP,
\f(CW\s-1man\s+1\fP,
\f(CW\s-1doc\s+1\fP,
and
\f(CW\s-1pkg\s+1\fP,
in that order.
.IP
.in -\n(PIu
\f(CW\s-1make \s+1\fP\f(CW\s-1clean\s+1\fP
.in
Clean up everything:
run
\f(CW\s-1make \s+1\fP\f(CW\s-1clean\s+1\fP
in
\f(CW\s-1kern\s+1\fP,
\f(CW\s-1user\s+1\fP,
\f(CW\s-1man\s+1\fP,
\f(CW\s-1doc\s+1\fP,
and
\f(CW\s-1pkg\s+1\fP,
in that order.
.PP
Thus:
.IP \(bu
To build packages with contents reflecting the current source code,
run
\f(CW\s-1make \s+1\fP\f(CW\s-1pkg\s+1\fP
at the top level.
.IP \(bu
To clean up but leave the packages there,
run
\f(CW\s-1make \s+1\fP\f(CW\s-1clean\s+1\fP
at the top level.
.IP \(bu
To work on a kernel module,
\f(CW\s-1cd \s+1\fP\f(CW\s-1kern\s+1\fP
and edit the files.
To compile what has been changed,
run
\f(CW\s-1make\s+1\fP.
When all seems well
(perhaps after installing the new binary by hand
to test it),
build a new package with
.RS
\f(CW\s-1make pkg && cd ../pkg && make pkg\s+1\fP
.br
.RE
or with
.RS
\f(CW\s-1cd .. && make pkg\s+1\fP
.br
.RE
.PP
All
\f(CW\s-1Makefile\s+1\fPs
assume that required tools are in the shell's search path;
in particular
\f2cc\fP
if C programs are to be compiled,
\f2ld\fP
if kernel object files are to be linked into loadable modules,
\f2fidl\fP
if long documents are to be rendered.
.PP
The stream-format package file resulting from the build process
is named
\f(CW\s-1CORDaoe.Z\s+1\fP.
Societal conventions may dictate giving it another name
in public,
like
\f(CW\s-1aoe-\s+1\fP\f2version\fP\f(CW\s-1.Z\s+1\fP.
The package name
\f(CW\s-1CORDaoe\s+1\fP
is encoded in the package data;
the filename doesn't matter.
A fixed name is used
so that
\f(CW\s-1pkg/Makefile\s+1\fP
need not be edited just because the version number has changed.
.SH
4.1.
Architecture-specific build issues
.PP
\f(CW\s-1Makefile\s+1\fPs
and associated conventions are designed
the distribution to build without fuss
whether on SPARC or x86;
no manual configuration is required.
It is assumed that only one of those architectures
will be built at a time;
there is no provision to keep SPARC- and x86-specific binaries
separate.
To use the same source tree for both,
build one architecture,
save the resulting binaries,
then run
\f(CW\s-1make \s+1\fP\f(CW\s-1clean\s+1\fP
before building the other.
.PP
Within each architecture
32- and 64-bit kernel binaries are kept side-by-side.
Compiler flag
\f(CW\s-1-xarch=generic64\s+1\fP
is used for 64-bit compilations;
this is recognized by Sun's compiler on either architecture.
64-bit kernel modules are created in subdirectory
\f(CW\s-1bin64\s+1\fP.
When
\f(CW\s-1make\0\s+1\fP\f(CW\s-1pkg\s+1\fP
copies files to
\f(CW\s-1pkg/data\s+1\fP
for packaging,
a recursive
\f2make\fP
call selects target directory
\f(CW\s-1usr/kernel/drv/sparcv9\s+1\fP
or
\f(CW\s-1usr/kernel/drv/amd64\s+1\fP
according to the architecture on which
\f2make\fP
is run.
.PP
The user-mode utility programs
are built only as 32-bit binaries,
since those work fine on 64-bit systems.
.PP
\f(CW\s-1pkg/Makefile\s+1\fP
calls
\f2pkgadd\fP(1M)
with variables declaring the current architecture name
and the corresponding subdirectory name
for 64-bit kernel modules,
so that names need not be hard-coded in
\f(CW\s-1pkginfo\s+1\fP
and
\f(CW\s-1prototype\s+1\fP.
.SH
5.
Coding conventions and other background
.PP
Some aspects of the style used in C code
and documentation
are a bit unusual.
The following is meant to aid comprehension,
not as a religious tract,
despite the occasional descent into sermon.
.SH
5.1.
C style
.PP
As a general but not inviolate rule,
code is presented top-down:
within each file,
major routines and entry points come first,
followed by local subroutines they call,
and so on.
Subroutines used only by one or two related major routines
immediately follow those routines;
common code used by many different routines
is grouped
after all uses.
Routines performing related functions
(e.g. different operations on the same type of data structure)
are placed near one another.
.PP
Data declarations and definitions
are placed at the top of the file,
except for tables used by only one or two subroutines,
which are usually placed next to the relevant code.
.PP
Every procedure has a prototype declaration.
A global procedure (one called from another file)
is declared in a header file.
A static procedure is declared
near the top of the source file,
after all data declarations
but before any code.
As a special case,
if a procedure's address
is used as a data initializer
(e.g. device-driver entry points),
its prototype
precedes the relevant data.
.PP
Although every procedure has an ISO-C prototype,
procedure definitions declare arguments in old-C syntax:
.KS
.RS
\f(CW\s-1static mblk_t *aoectlintr(int, mblk_t *);\s+1\fP
.br
\f(CW\s-1\&.\|.\|.\s+1\fP\f(CW\s-1\s+1\fP
.br
\f(CW\s-1static mblk_t *\s+1\fP
.br
\f(CW\s-1aoectlintr(chan, mp)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1mblk_t *mp;\s+1\fP
.br
.RE
.KE
.PP
The original C specification intentionally made
procedure definitions resemble calls,
with type information as a sidebar,
in the belief that this makes the code easier to read.
The author still agrees.
The meaning of such mixed notation is clearly defined
in the ISO standard.
.PP
Variadic procedures are an exception:
their definitions use the ISO syntax,
because that is the only way to express them.
.PP
Types are always declared explicitly:
.KS
.RS
\f(CW\s-1static int\s+1\fP
.br
\f(CW\s-1doquery(ap, maj, min)\s+1\fP
.br
\f(CW\s-1Aoechan *ap;\s+1\fP
.br
\f(CW\s-1int maj, min;\s+1\fP
.br
\f(CW\s-1{\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1unsigned int x;\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\&.\|.\|.\s+1\fP\f(CW\s-1\s+1\fP
.br
.RE
.KE
not
.KS
.RS
\f(CW\s-1static\s+1\fP
.br
\f(CW\s-1doquery(ap, maj, min)\s+1\fP
.br
\f(CW\s-1Aoechan *ap;\s+1\fP
.br
\f(CW\s-1{\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1unsigned x;\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\&.\|.\|.\s+1\fP\f(CW\s-1\s+1\fP
.br
.RE
.KE
.PP
Neither procedures nor data are allowed global scope
unless that is truly required.
(Here the author disagrees with both original and ISO C:
\f(CW\s-1static\s+1\fP
scope should have been the default.)
.PP
Machine-specific pseudo-optimizations
such as the
\f(CW\s-1register\s+1\fP
keyword are not used.
.PP
Source files are not made aware of the source-code directory structure.
In particular,
\f(CW\s-1#include\s+1\fP
directives for AoE-specific header files
are always of the form
\f(CW\s-1#include "\s+1\fP\f2file\fP\f(CW\s-1.h"\s+1\fP;
if
\f2file\fP\f(CW\s-1.h\s+1\fP
might be in directory
\f(CW\s-1../include\s+1\fP,
compiler option
\f(CW\s-1-I../include\s+1\fP
should be used.
.PP
Compile-time conditionals are used sparingly.
There are no `normal' options;
only one version of the code is officially supported.
\f(CW\s-1#ifdef\s+1\fPs
are used only to hide unsupported experimental code,
to make compiler or system-bug band-aids stand out,
or to work around compile-time environment differences
in different versions of Solaris.
.SH
5.2.
Binary data structures
.PP
Externally-defined binary data structures
(e.g. AoE messages,
ATA IDENTIFY data,
EFI disk labels)
are treated as
byte arrays,
not as C structures.
A constant is defined for the offset to each element.
Multi-byte objects other than plain byte arrays
(e.g. integers of various sizes)
are accessed through machine-independent macros
that pack or unpack data
with appropriate attention to byte order.
This allows source code to be ignorant of
per-system byte-order and word-alignment rules.
.PP
These header files,
all stored in directory
\f(CW\s-1include\s+1\fP
in the source tree,
define constants and macros
to implement this scheme.
In the descriptions below,
\f2buf\fP
is always a buffer address,
and may be either
\f(CW\s-1char\0\s+1\fP\f(CW\s-1*\s+1\fP
or
\f(CW\s-1unsigned\0\s+1\fP\f(CW\s-1char\0\s+1\fP\f(CW\s-1*\s+1\fP.
\f2Off\fP
is an integer offset within the buffer addressed by
\f2buf\fP.
\f2Val\fP
is an integer value;
sometimes only the low-order bits are used.
.IP
.in -\n(PIu
\f(CW\s-1aoeproto.h\s+1\fP
.in
AoE protocol:
.RS
.IP \(ci
Offsets for the elements of AoE protocol messages,
relative to the first octet of the AoE header,
i.e. the byte following the Ethernet MAC header
(in case the protocol is someday used over another medium).
.IP \(ci
Constant values used in various protocol fields.
.IP \(ci
These access macros:
.RS
.IP
.in -\n(PIu
\f(CW\s-1fraoechar(buf, off)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1fraoeshort(buf, off)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1fraoelong(buf, off)\s+1\fP
.in
Fetch 8-bit,
16-bit,
or 32-bit
unsigned value from offset
\f2off\fP
in buffer
\f2buf\fP,
using the byte order defined for the AoE protocol.
.IP
.in -\n(PIu
\f(CW\s-1toaoechar(buf, off, val)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1toaoeshort(buf, off, val)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1toaoelong(buf, off, val)\s+1\fP
.in
Store 8-bit,
16-bit,
or 32-bit value
\f2val\fP
at offset
\f2off\fP
in
\f2buf\fP.
.RE
.IP \(ci
Macros to encode or decode some sub-octet values,
such as version-and-flag
or version-and-config-string-op codes.
.RE
.IP
.in -\n(PIu
\f(CW\s-1etherproto.h\s+1\fP
.in
Ethernet MAC header:
.RS
.IP \(ci
Offsets for Ethernet
source, destination,
and protocol (type).
.IP \(ci
Macro
\f(CW\s-1fretherprot(buf, off)\s+1\fP
to fetch a protocol-type value from offset
\f2off\fP
in buffer
\f2buf\fP;
macro
\f(CW\s-1toetherprot(buf, off, val)\s+1\fP
to set protocol-type value
\f2val\fP.
.IP \(ci
These additional constant values:
.RS
.IP
.in -\n(PIu
\f(CW\s-1LEN_EADDR\s+1\fP
.in
Length of an Ethernet MAC address.
Such addresses are treated as opaque fixed-length byte arrays.
.IP
.in -\n(PIu
\f(CW\s-1LEN_ETHER\s+1\fP
.in
Length of the MAC header.
.IP
.in -\n(PIu
\f(CW\s-1MINLEN_ETHER\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1MAXLEN_ETHER\s+1\fP
.in
Minimum and maximum standard lengths of an Ethernet datagram,
including header.
.RE
.RE
.IP
.in -\n(PIu
\f(CW\s-1ata.h\s+1\fP
.in
ATA disk register contents and associated ATA message data:
.RS
.IP \(ci
Values of flags and command codes
read from or written to ATA registers.
Register offsets are not defined;
for our purposes a register is
a location in an AoE message.
.IP \(ci
Offsets and constants for the relevant
(used by this package)
parts of the large data structure returned by the
ATA IDENTIFY command.
.IP \(ci
Access macros for elements within the IDENTIFY structure:
.RS
.IP
.in -\n(PIu
\f(CW\s-1frataword(buf, \s+1\fP\f(CW\s-1off)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1fratalong(buf, \s+1\fP\f(CW\s-1off)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1fratall(buf, \s+1\fP\f(CW\s-1off)\s+1\fP
.in
Fetch 16-bit,
32-bit,
or 64-bit value from offset
\f2off\fP
in
\f2buf\fP,
using the byte order defined by the
ATA specification.
.IP
.in -\n(PIu
\f(CW\s-1toataword(buf, \s+1\fP\f(CW\s-1off, \s+1\fP\f(CW\s-1val)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1toatalong(buf, \s+1\fP\f(CW\s-1off, \s+1\fP\f(CW\s-1val)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1toatall(buf, \s+1\fP\f(CW\s-1off, \s+1\fP\f(CW\s-1val)\s+1\fP
.in
Store 16-bit,
32-bit,
or 64-bit value
\f2val\fP
to offset
\f2off\fP
in
\f2buf\fP.
.RE
.RE
.IP
.in -\n(PIu
\f(CW\s-1aoecommproto.h\s+1\fP
.in
Message protocol
used for control messages
exchanged between user-mode programs
and the
\f(CW\s-1aoecomm\s+1\fP
and
\f(CW\s-1aoectl\s+1\fP
drivers.
This header defines offsets and other constants;
integer values
are formatted using the
\f(CW\s-1fraoe\s+1\fP\f2xxx\fP
and
\f(CW\s-1toaoe\s+1\fP\f2xxx\fP
macros from
\f(CW\s-1aoeproto.h\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1efilabel.h\s+1\fP
.in
Data structures used by the
new EFI label
scheme:
.RS
.IP \(ci
Offsets within the GPT
(GUID Partition Table header,
one per copy of the label).
.IP \(ci
Offsets within each GPE
(GUID Partition Entry,
one per partition per copy of the label).
.IP \(ci
Access macros for elements of either structure:
.RS
.IP
.in -\n(PIu
\f(CW\s-1frefiword(buf, \s+1\fP\f(CW\s-1off)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1frefilong(buf, \s+1\fP\f(CW\s-1off)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1frefill(buf, \s+1\fP\f(CW\s-1off)\s+1\fP
.in
Fetch 16-bit,
32-bit,
or 64-bit value from offset
\f2off\fP
in
\f2buf\fP,
using the byte order defined by the
EFI specification.
.IP
.in -\n(PIu
\f(CW\s-1toefiword(buf, \s+1\fP\f(CW\s-1off, \s+1\fP\f(CW\s-1val)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1toefilong(buf, \s+1\fP\f(CW\s-1off, \s+1\fP\f(CW\s-1val)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1toefill(buf, \s+1\fP\f(CW\s-1off, \s+1\fP\f(CW\s-1val)\s+1\fP
.in
Store 16-bit,
32-bit,
or 64-bit value
\f2val\fP
to offset
\f2off\fP
in
\f2buf\fP.
.RE
.IP \(ci
These additional constants:
.RS
.IP
.in -\n(PIu
\f(CW\s-1MINLEN_GPT\s+1\fP
.in
Length of the meaningful contents of the GPT.
.IP
.in -\n(PIu
\f(CW\s-1LEN_GPT\s+1\fP
.in
Length of the whole GPT sector.
.IP
.in -\n(PIu
\f(CW\s-1EFI_GPT_LOC\s+1\fP
.in
Sector number of the primary copy of the GPT.
.IP
.in -\n(PIu
\f(CW\s-1LEN_GPE\s+1\fP
.in
Length of each GPE.
.IP
.in -\n(PIu
\f(CW\s-1LEN_GUID\s+1\fP
.in
Length of the UUID (128-bit unique ID)
used both in the GPT and in each GPE.
.RE
.RE
The
\f(CW\s-1toefi\s+1\fP\f2xxx\fP
macros are not used at present;
they are included for completeness.
.IP
.in -\n(PIu
\f(CW\s-1dospart.h\s+1\fP
.in
Data structures
used by DOS partition labels:
.RS
.IP \(ci
Offsets for elements
(partition descriptors,
magic number)
within the DOS boot sector.
.IP \(ci
Offsets for the components of each of the four
partition descriptors.
.IP \(ci
Access macros:
.RS
.IP
.in -\n(PIu
\f(CW\s-1frdospchar(buf, \s+1\fP\f(CW\s-1off)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1frdospword(buf, \s+1\fP\f(CW\s-1off)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1frdosplong(buf, \s+1\fP\f(CW\s-1off)\s+1\fP
.in
Fetch 8-bit,
16-bit,
or
32-bit value from offset
\f2off\fP
in
\f2buf\fP,
using the conventional (Intel) byte order.
.IP
.in -\n(PIu
\f(CW\s-1todospchar(buf, \s+1\fP\f(CW\s-1off, \s+1\fP\f(CW\s-1val)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1todospword(buf, \s+1\fP\f(CW\s-1off, \s+1\fP\f(CW\s-1val)\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1todosplong(buf, \s+1\fP\f(CW\s-1off, \s+1\fP\f(CW\s-1val)\s+1\fP
.in
Store 8-bit,
16-bit,
or
32-bit value
\f2val\fP
to offset
\f2off\fP
in
\f2buf\fP.
.RE
The
\f(CW\s-1todosp\s+1\fP\f2xxx\fP
macros are not used at present;
they are included for completeness.
.IP \(ci
Additional constants, including:
.RS
.IP
.in -\n(PIu
\f(CW\s-1LEN_BSEC\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1LEN_DOSP\s+1\fP
.in
Length of the entire boot sector,
and of a single partition descriptor.
.IP
.in -\n(PIu
\f(CW\s-1BSECMAGIC\s+1\fP
.in
Magic number found at the end
of a valid boot sector.
.IP
.in -\n(PIu
\f(CW\s-1SYSID_SUNOS\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1SYSID_SUNOS2\s+1\fP
.in
The two system ID (partition type) values
used when a partition encapsulates a
Sun VTOC label.
.RE
.RE
.PP
Sun supply headers describing EFI and DOS labels,
but they use C structures accompanied by
explicit machine-dependent byte-swapping,
something we would rather avoid.
Having our own headers may also simplify future support
for compiling the AoE code on older versions of Solaris.
.PP
This code fragment tests whether the AoE message at
\f2p\fP
is a response message,
and if so fetches its tag value:
.KS
.RS
\f(CW\s-1if (fraoechar(p, AOE_VF) & AOEF_R)\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1tag = fraoelong(p, AOE_TAG);\s+1\fP
.br
.RE
.KE
This fragment sets the major and minor target numbers
to standard wildcard values:
.KS
.RS
\f(CW\s-1toaoeshort(p, AOE_MAJOR, AOEMAJWILD);\s+1\fP
.br
\f(CW\s-1toaoechar(p, AOE_MINOR, AOEMINWILD);\s+1\fP
.br
.RE
.KE
.SH
5.3.
Control message protocol
.PP
An internal protocol is used for messages exchanged
between user-mode programs and the
\f(CW\s-1aoecomm\s+1\fP
STREAMS module during channel setup,
and for messages
written to
\f(CW\s-1/dev/aoectl\s+1\fP
and read from
\f(CW\s-1/dev/aoemon\s+1\fP.
The message format is defined by constants in
\f(CW\s-1aoecommproto.h\s+1\fP;
messages should be accessed using the macros defined in
\f(CW\s-1aoeproto.h\s+1\fP.
.PP
Each message comprises a fixed-length header
and a variable-length body.
The header contains these fields:
.IP
.in -\n(PIu
\f(CW\s-1AC_TYPE\s+1\fP
.in
Message type;
determines the body format.
.IP
.in -\n(PIu
\f(CW\s-1AC_LEN\s+1\fP
.in
Length of the entire message,
including this header.
.IP
.in -\n(PIu
\f(CW\s-1AC_CHANNEL\s+1\fP
.in
AoE channel number,
if any,
associated with this message.
.PP
These message types are defined:
.IP
.in -\n(PIu
\f(CW\s-1ACINITCHAN\s+1\fP
.in
Initialize a communication channel
using the enclosed channel number,
local MAC address,
Ethernet protocol type,
and maximum data-segment length,
and validating the request with
a 32-bit random cookie.
Written first to
\f(CW\s-1/dev/aoectl\s+1\fP,
then to the channel on which
\f(CW\s-1aoecomm\s+1\fP
was pushed.
.IP
.in -\n(PIu
\f(CW\s-1ACINITACK\s+1\fP
.in
Reply from
\f(CW\s-1aoecomm\s+1\fP
to
\f(CW\s-1ACINITCHAN\s+1\fP,
announcing that channel setup is complete
or giving an
\f2errno\fP
value explaining why it failed.
.IP
.in -\n(PIu
\f(CW\s-1ACSEND\s+1\fP
.in
Command to
\f(CW\s-1/dev/aoectl\s+1\fP:
send the enclosed datagram
(usually an AoE message)
via the channel named.
The MAC header must be included,
but only the destination address is significant;
source address and protocol values are filled in by the system.
.IP
.in -\n(PIu
\f(CW\s-1ACDEVENAB\s+1\fP
.in
Command to
\f(CW\s-1/dev/aoectl\s+1\fP:
query or update the active-target registry.
Body gives an AoE target address
(major and minor numbers)
and a command:
enable target, disable, query.
.IP
.in -\n(PIu
\f(CW\s-1ACLOG\s+1\fP
.in
Message from
\f(CW\s-1/dev/aoemon\s+1\fP:
body contains an unexpected,
ill-formed,
or unsendable AoE message,
as explained by a type code:
.RS
.IP
.in -\n(PIu
\f(CW\s-1ACLUNSOL\s+1\fP
.in
Unsolicited AoE message.
Usually a Query-Config response
from a target that is not open.
.IP
.in -\n(PIu
\f(CW\s-1ACLILLFORM\s+1\fP
.in
Ill-formed AoE message.
.IP
.in -\n(PIu
\f(CW\s-1ACLSENDFAIL\s+1\fP
.in
AoE message that couldn't be sent
(\f2aoecomm_send\fP
failed).
.IP
.in -\n(PIu
\f(CW\s-1ACLTIMEOUT\s+1\fP
.in
AoE message that was sent,
but for which no reply was received
after repeated retries.
The operation that generated the message
has already received an error.
The message may be truncated to the
MAC and AoE headers,
omitting any data sectors.
.IP
.in -\n(PIu
\f(CW\s-1ACLAOEERR\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1ACLATAERR\s+1\fP
.in
AoE message reporting an AoE or ATA error.
.RE
.PP
See the source code
and the supplied
\f2aoecomm\fP(7M)
and
\f2aoectl\fP(7D)
manual entries for details.
.SH
6.
The big picture
.PP
Here is an sketch of major operations
showing
the components and subroutines used.
.SH
6.1.
Initialize channel
(activate AoE on a network device)
.PP
These user-mode steps use a file descriptor
for a network device of suitable type,
suitably initialized
(e.g. configured to receive the desired Ethernet protocol),
to create an active AoE channel:
.IP 1.
Push an instance of the
\f(CW\s-1aoecomm\s+1\fP
STREAMS module onto the file descriptor.
.IP 2.
Invent a 32-bit nonzero random cookie value.
Compose an
\f(CW\s-1ACINITCHAN\s+1\fP
AoE control message
containing the desired channel number,
the cookie,
the local MAC address of the network device
(usually discovered by a device-specific call),
the Ethernet protocol type to be used,
and the maximum data-segment size
(usually computed from the Ethernet device MTU).
.IP 3.
Open
\f(CW\s-1/dev/aoectl\s+1\fP
and write the
\f(CW\s-1ACINITCHAN\s+1\fP
message.
Close
\f(CW\s-1/dev/aoectl\s+1\fP.
The
\f(CW\s-1aoectl\s+1\fP
module calls
\f2aoecomm_initchan\fP
to register the cookie
for the desired channel.
.IP 4.
Write the
\f(CW\s-1ACINITCHAN\s+1\fP
message to the file descriptor.
\f(CW\s-1aoecomm\s+1\fP
compares the cookie stored for the desired channel
with that supplied in the message at hand.
If the cookies match
and the channel number is not already in use,
the protocol number
and MAC address are stored for use when sending messages,
and the channel is made active.
The two-way handshake
using both
\f(CW\s-1aoectl\s+1\fP
and
\f(CW\s-1aoecomm\s+1\fP
is needed to
make it harder for bad guys
to cause trouble.
.LP
The file descriptor must remain open;
when it is closed,
the
\f(CW\s-1aoecomm\s+1\fP
instance for this channel is popped,
and the channel shut down.
.PP
This initialization dance is normally done
by user-mode program
\f2aoestart\fP
within library routine
\f2comminit\fP.
\f2Aoestart\fP
then calls library routine
\f2achattach\fP,
which uses
\f2fattach\fP(3)
to attach the file descriptor to a file in directory
\f(CW\s-1/etc/aoe\s+1\fP
to keep the channel open.
.SH
6.2.
Open AoE disk device
.PP
The
\f(CW\s-1aoed\s+1\fP
disk driver open routine
initializes the device if necessary,
broadcasting an AoE Query-Config message to
discover the target's Ethernet MAC address
and other characteristics.
.PP
Device configuration is semi-automatic.
Every target to be used
must be listed in advance in kernel configuration file
\f(CW\s-1aoed.conf\s+1\fP;
changes to the file
are effective only when the
\f(CW\s-1aoed\s+1\fP
module is reloaded,
or
(Solaris 10 only)
when
\f(CW\s-1update_drv\0\s+1\fP\f(CW\s-1-f\s+1\fP
is run.
The memory overhead for a target that is configured
but not used
is modest;
it is unreasonable to declare every possible device
allowed by the protocol
(255*65535 of them),
but prudent to declare every slot
in a new EtherDrive shelf
even if only a few will be used at first.
.PP
It takes a few seconds to discover
that an AoE device isn't available on the network
even though its device files exist.
If there are many such unavailable devices,
programs that open every possible disk device in the system
(notably
\f2format\fP(1M))
will run quite slowly.
To prevent this,
\f(CW\s-1aoecomm\s+1\fP
maintains a registry of `enabled' devices,
those believed actually to exist.
Daemon
\f2aoemon\fP
listens to
\f(CW\s-1/dev/aoectl\s+1\fP
for unsolicited Query-Config response messages,
generated when a target device is powered up
or in response to a broadcast Query-Config command
from
\f2aoestart\fP
or
\f2aoectl\fP.
When such a message arrives,
\f2aoemon\fP
enables the responding target
by writing an
\f(CW\s-1ACDEVENAB\s+1\fP
control-protocol message
to
\f(CW\s-1/dev/aoectl\s+1\fP,
triggering a call to
\f2aoecomm_devenab\fP
to enable the device in the registry.
.PP
The
\f2aoectl\fP
command
lists and modifies the registry.
.SH
6.3.
Read and write disks
.PP
The
\f(CW\s-1aoed\s+1\fP
driver supplies
read and write entry points
for raw I/O,
and a
strategy routine
for block and file system I/O.
.PP
I/O is done by composing an AoE command
in a STREAMS buffer
and calling
\f2aoecomm_send\fP
to send it to the network device.
When an AoE message arrives
on a channel in which
\f(CW\s-1aoed\s+1\fP
has registered interest
(any on which some disk device
has been opened),
receive routine
\f2adreceive\fP
is called.
Pending commands are stored in a list.
When a message arrives,
the system searches for
a pending command
for the same AoE device,
with the same AoE tag value.
Timer routine
\f2chantimer\fP
is called periodically to scan the pending-command list
for commands which haven't received responses
within a specified time interval.
If a command has timed out only a few times,
it is retransmitted.
After several timeouts
the command is abandoned,
and the I/O request aborted with an error.
.PP
Often an I/O request will involve more data
than can be handled in a single AoE command:
1024 bytes by default,
more if larger (jumbo) Ethernet frames are allowed.
If so,
a single request will generate several AoE commands.
The corresponding several AoE responses
may be returned out of order.
.SH
6.4.
When the system boots
.PP
On a non-SMF system,
startup script
\f(CW\s-1/etc/init.d/aoe\s+1\fP
is run at boot time.
When SMF is present,
AoE is represented by service
\f(CW\s-1svc:/device/aoe\s+1\fP,
initially disabled;
when the service is enabled
and on subsequent reboots,
method script
\f(CW\s-1/lib/svc/method/device-aoe\s+1\fP
is called.
In either case
the script starts the
\f2aoemon\fP
daemon,
then calls the
\f2aoestart\fP
command for each channel
listed in AoE-specific configuration file
\f(CW\s-1/etc/aoe.conf\s+1\fP.
.PP
The startup script is called early in the boot process,
after the root and
\f(CW\s-1/usr\s+1\fP
file systems have been mounted,
but before any others.
Hence the AoE subsystem must not rely on access
to other file systems,
and in particular must not require access to
\f(CW\s-1/var\s+1\fP
or
\f(CW\s-1/opt\s+1\fP.
The payoff is that any other file system,
including
\f(CW\s-1/var\s+1\fP
or
\f(CW\s-1/opt\s+1\fP,
may be stored on an AoE disk.
AoE drivers are normally stored in
\f(CW\s-1/usr/kernel\s+1\fP,
support programs in
\f(CW\s-1/usr/sbin\s+1\fP,
configuration files and the mount points used by
\f2fattach\fP
in
\f(CW\s-1/etc\s+1\fP.
.SH
6.5.
When the system shuts down
.PP
Nothing special need be done
when the system is shut down.
Channels remain active until the very end,
even after all processes have been killed,
because it is the
\f2fattach\fP
operation that keeps them open.
Thus the normal shutdown code
to unmount all file systems
at the last minute
works without fuss.
Notice,
however,
that the file system where AoE channels are attached
may not be unmounted
because the AoE
\f2fattach\fP
calls keep it busy.
That is why AoE uses
\f(CW\s-1/etc/aoe\s+1\fP
rather than
\f(CW\s-1/var/aoe\s+1\fP;
the latter directory is sometimes a separate file system,
the latter rarely if ever.
.PP
AoE may be intentionally shut down
by running
\f(CW\s-1/etc/init.d/aoe\0\s+1\fP\f(CW\s-1stop\s+1\fP
(non-SMF system)
or
\f(CW\s-1svcadm\0\s+1\fP\f(CW\s-1disable\0\s+1\fP\f(CW\s-1svc:/device/aoe\s+1\fP
(SMF).
This is sometimes useful for maintenance purposes,
but is not done by a normal shutdown.
.SH
7.
Kernel component details
.SH
7.1.
\f(CW\s-1aoecomm\s+1\fP
.SH
7.1.1.
Configuration
.PP
An instance of
\f(CW\s-1aoecomm\s+1\fP
is created whenever the module
is pushed on a stream file;
thus each active AoE channel has a separate instance.
.SH
7.1.2.
Source code
.PP
All code is in a single source file,
\f(CW\s-1aoecomm.c\s+1\fP.
It uses AoE-specific header files
\f(CW\s-1aoeproto.h\s+1\fP,
\f(CW\s-1aoecomm.h\s+1\fP,
\f(CW\s-1aoecommproto.h\s+1\fP,
and
\f(CW\s-1etherproto.h\s+1\fP.
.PP
Header file
\f(CW\s-1aoecomm.h\s+1\fP,
stored in the
\f(CW\s-1include\s+1\fP
source directory,
contains prototypes for global entry points
called by other AoE kernel modules,
and declares the
\f(CW\s-1Aoecookie\s+1\fP
data type used by both kernel and user components.
.SH
7.1.3.
Data structures
.PP
None is used outside the
\f(CW\s-1aoecomm\s+1\fP
module.
.PP
Each channel
(active instance of
\f(CW\s-1aoecomm\s+1\fP)
has an associated
\f(CW\s-1Aoechan\s+1\fP
structure.
Its contents include:
.IP \(bu
Channel number:
a small integer
unique among all active channels
.IP \(bu
Local Ethernet MAC address for this channel;
Ethernet protocol type to be used
.IP \(bu
Maximum data-segment size
allowed over this channel
.IP \(bu
Pointer to the STREAMS write queue for this channel
.IP \(bu
Registry of AoE target devices
accessible through this channel
.IP \(bu
Pointer to a single receiver routine,
called when an AoE message arrives
.IP \(bu
\f(CW\s-1krwlock_t\s+1\fP
lock
controlling concurrent access
to this
\f(CW\s-1Aoechan\s+1\fP
.IP \(bu
Channel-is-active flag
.LP
An
\f(CW\s-1Aoechan\s+1\fP
is allocated when an
\f(CW\s-1aoecomm\s+1\fP
instance is created
(when the module is pushed);
its address is stored in the
\f(CW\s-1q_ptr\s+1\fP
member of both queues.
When a channel is shut down
(the module is popped),
its
\f(CW\s-1Aoechan\s+1\fP
is freed.
.PP
Several data structures
provide global context within
\f(CW\s-1aoecomm\s+1\fP,
but are not accessible to the rest of the kernel:
.IP
.in -\n(PIu
\f(CW\s-1static \s+1\fP\f(CW\s-1void \s+1\fP\f(CW\s-1(*logger)(int, \s+1\fP\f(CW\s-1int, \s+1\fP\f(CW\s-1mblk_t \s+1\fP\f(CW\s-1*);\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1static \s+1\fP\f(CW\s-1kmutex_t \s+1\fP\f(CW\s-1loggerlock;\s+1\fP
.in
\f2Logger\fP
points to the logging routine
called by
\f2aoecomm_log\fP,
if any.
\f2Loggerlock\fP
prevents concurrent access to
\f2logger\fP.
.IP
.in -\n(PIu
\f(CW\s-1static \s+1\fP\f(CW\s-1Aoechan \s+1\fP\f(CW\s-1*aoechan[MAXCHAN];\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1static \s+1\fP\f(CW\s-1kmutex_t \s+1\fP\f(CW\s-1chantlock;\s+1\fP
.in
The
\f2aoechan\fP
table contains pointers to
\f(CW\s-1Aoechan\s+1\fP
structures
indexed by channel number
(assigned when the channel is initialized)
for faster lookup.
\f(CW\s-1MAXCHAN\s+1\fP
is defined in
\f(CW\s-1aoecomm.c\s+1\fP;
its present value is 10.
\f2Chantlock\fP
prevents concurrent access to the
\f2aoechan\fP
array,
but not the
\f(CW\s-1Aoechan\s+1\fPs.
.IP
.in -\n(PIu
\f(CW\s-1static \s+1\fP\f(CW\s-1Aoecookie \s+1\fP\f(CW\s-1pendcookie[MAXCHAN];\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1static \s+1\fP\f(CW\s-1kmutex_t \s+1\fP\f(CW\s-1pendlock;\s+1\fP
.in
\f(CW\s-1pendcookie[\s+1\fP\f2i\fP\f(CW\s-1]\s+1\fP
contains a pending initialization cookie
for channel
\f2i\fP,
or
\f(CW\s-1NOCOOKIE\s+1\fP
(zero)
if none has arrived yet.
\f2Pendlock\fP
prevents concurrent access to the
\f2pendcookie\fP
array.
.SH
7.1.4.
Exports
.PP
Standard Solaris loadable-module and STREAMS-module
entry points and data structures are supplied:
in particular
\f2_init\fP,
\f2_fini\fP,
and
\f2_info\fP
routines,
a
\f(CW\s-1modlinkage\s+1\fP
structure (and its many children),
a
\f(CW\s-1streamtab\s+1\fP
structure,
and a pair of
\f(CW\s-1qinit\s+1\fP
structures.
Open and close
(module push and pop),
read put and service,
and write service routines are supplied;
puts to the write queue use
\f2putq\fP(9F).
.PP
Only
\f2_init\fP,
\f2_fini\fP,
and
\f2_info\fP
are global.
\f2_init\fP
calls
\f2mod_install\fP
to tell the system where to find the
\f(CW\s-1modlinkage\s+1\fP
structure
through which the other data structures and routines
can be located.
.PP
Several routines called by other AoE kernel modules
are made available as global entry points.
Prototypes for these routines are declared in
\f(CW\s-1aoecomm.h\s+1\fP,
constant values in
\f(CW\s-1aoecommproto.h\s+1\fP.
.KS
.RS
\f(CW\s-1int aoecomm_initchan(chan, cookie)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1Aoecookie cookie;\s+1\fP
.br
.RE
.KE
.PP
Store
\f2cookie\fP
in
\f(CW\s-1pendcookie[\s+1\fP\f2chan\fP\f(CW\s-1]\s+1\fP.
If an
\f(CW\s-1ACINITCHAN\s+1\fP
message arrives
on the write queue
with the same
\f2chan\fP
and
\f2cookie\fP
values,
the channel becomes active.
Return > 0 if all is well,
< 0 if an error occurred
(\f2chan\fP
out of range).
.KS
.RS
\f(CW\s-1int aoecomm_initdriver(chan, receiver)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1mblk_t *(*receiver)(int, mblk_t *);\s+1\fP
.br
.RE
.KE
.PP
If the receiver routine for channel
\f2chan\fP
is
\f(CW\s-1NULL\s+1\fP,
set it to
\f2receiver\fP.
If
\f2receiver\fP
was already this channel's receiver,
leave it be.
In either case return > 0.
.PP
If
\f2chan\fP
is invalid
or already has a different receiver,
return < 0.
.PP
When an AoE message arrives
on the read queue:
.IP \(bu
If this channel has a receiver routine,
call the receiver
with channel number and complete message,
including the Ethernet MAC header.
.IP \(bu
If this channel has no receiver,
log the unsolicited
message.
.KS
.RS
\f(CW\s-1int aoecomm_initlog(logfunc)\s+1\fP
.br
\f(CW\s-1void (*logfunc)(int, int, mblk_t *);\s+1\fP
.br
.RE
.KE
.PP
If
\f2logger\fP
is
\f(CW\s-1NULL\s+1\fP,
set it to
\f2logfunc\fP
and return > 0.
If
\f2logfunc\fP \f(CW\s-1==\s+1\fP \f2logger\fP,
return > 0.
If
\f2logger\fP
was already nonzero
and is not the same as
\f2logfunc\fP,
leave
\f2logger\fP
unchanged and
return < 0.
.KS
.RS
\f(CW\s-1void aoecomm_log(chan, code, mp)\s+1\fP
.br
\f(CW\s-1int chan, code;\s+1\fP
.br
\f(CW\s-1mblk_t *mp;\s+1\fP
.br
.RE
.KE
.PP
If a logger routine has been registered with
\f2aoecomm_initlog\fP,
call it with the same arguments:
\f(CW\s-1(*logger)(\s+1\fP\f2chan\fP\f(CW\s-1,\s+1\fP\0\f2code\fP\f(CW\s-1,\s+1\fP\0\f2mp\fP\f(CW\s-1)\s+1\fP.
The logger becomes responsible for STREAMS message
\f2mp\fP,
and will free it when finished;
neither
\f2aoecomm_log\fP
nor its caller may use it further.
.PP
If there is no logger,
free
\f2mp\fP.
.KS
.RS
\f(CW\s-1int aoecomm_devenab(chan, maj, min, cmd)\s+1\fP
.br
\f(CW\s-1int chan, maj, min, cmd;\s+1\fP
.br
.RE
.KE
.PP
Query or update the enabled-target registry,
according to
\f2cmd\fP:
.IP
.in -\n(PIu
\f(CW\s-1ADENAB\s+1\fP
.in
Enable (add to the registry)
the specified target.
If
\f2min\fP
is
\f(CW\s-1AOEMINWILD\s+1\fP,
enable every possible target with the
\f2chan\fP
and
\f2maj\fP
given.
Neither
\f2chan\fP
nor
\f2maj\fP
may be wild.
Return
\f(CW\s-1ADENAB\s+1\fP
if all is well,
\f(CW\s-1ADDISAB\s+1\fP
if
\f2chan\fP
or
\f2maj\fP
was invalid.
.IP
.in -\n(PIu
\f(CW\s-1ADDISAB\s+1\fP
.in
Disable (remove from the registry)
all targets matching the arguments.
\f2Maj\fP
value
\f(CW\s-1AOEMAJWILD\s+1\fP
matches any major number,
\f2min\fP
value
\f(CW\s-1AOEMINWILD\s+1\fP,
any minor number.
\f2Chan\fP
may not be wild.
Return
\f(CW\s-1ADDISAB\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1ADQUERY\s+1\fP
.in
Query the registry:
return
\f(CW\s-1ADENAB\s+1\fP
if at least one target matching the arguments
is registered,
\f(CW\s-1ADDISAB\s+1\fP
otherwise.
\f2Chan\fP
value
\f(CW\s-1ACHWILD\s+1\fP
matches any channel,
\f2maj\fP
value
\f(CW\s-1AOEMAJWILD\s+1\fP
any major number,
\f2min\fP
value
\f(CW\s-1AOEMINWILD\s+1\fP
any minor number.
.KS
.RS
\f(CW\s-1int aoecomm_maxdata(chan)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
.RE
.KE
.PP
Return the maximum data-segment size
for channel
\f2chan\fP,
or \-1
if
\f2chan\fP
is invalid.
.KS
.RS
\f(CW\s-1int aoecomm_send(chan, mp)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1mblk_t *mp;\s+1\fP
.br
.RE
.KE
.PP
Send STREAMS message
\f2mp\fP
via channel
\f2chan\fP.
If
\f2chan\fP
is
\f(CW\s-1ACHWILD\s+1\fP,
send a copy to every active channel.
.PP
The message must begin with an Ethernet MAC header
with destination MAC address filled in.
Source address and protocol type
are overwritten
with the values from the
\f(CW\s-1Aoechan\s+1\fP.
.PP
If the message is sent,
return > 0.
If an error occurs
(invalid
\f2chan\fP,
no room downstream),
free the message
and return < 0.
.SH
7.1.5.
Algorithms
.SH
7.1.5.1.
Module pushed
.PP
Open routine
\f2aoecommopen\fP
allocates a new
\f(CW\s-1Aoechan\s+1\fP,
marked inactive;
stores its address in
\f(CW\s-1q_ptr\s+1\fP
for both the read and write queues;
and calls
\f2qprocson\fP(9F)
to enable queue processing.
A channel number is not assigned yet,
so no data may be sent;
the channel is inactive,
so received data are thrown away.
.SH
7.1.5.2.
Message arrives on write queue (from user code)
.PP
Arriving messages are added to the write queue
by
\f2putq\fP(9F).
.PP
Write service routine
\f2aoecommwsrv\fP
loops calling
\f2getq\fP(9F),
processing each message as follows:
.IP \(bu
If the message is of any type but
\f(CW\s-1M_DATA\s+1\fP,
pass it downstream without further interpretation.
.IP \(bu
If the data message comprises
a valid
\f(CW\s-1ACINITCHAN\s+1\fP
message,
and if:
.RS
.IP \(ci
the channel number is valid,
and not that of an already-active channel;
.IP \(ci
a cookie has already been stored for that channel;
.IP \(ci
and the cookie in the message matches the stored value;
.RE
make the channel active:
.RS
.IP 1.
Copy the channel number,
MAC address, Ethernet protocol type,
and maximum data-segment size
from the
\f(CW\s-1ACINITCHAN\s+1\fP
message
to this channel's
\f(CW\s-1Aoechan\s+1\fP.
.IP 2.
Call
\f2strqset\fP(9F)
to adjust the read-queue high water mark
according to the maximum data-segment size.
.IP 3.
Enter the
\f(CW\s-1Aoechan\s+1\fP
in
\f2aoechan\fP.
.IP 4.
Send an
\f(CW\s-1ACINITACK\s+1\fP
reply reporting success.
.RE
If the channel number is invalid,
the cookie is wrong,
or some other error occurs,
send an
\f(CW\s-1ACINITACK\s+1\fP
reply containing an appropriate error code.
.IP \(bu
If the data message is not a valid
\f(CW\s-1ACINITCHAN\s+1\fP,
free it and continue.
.SH
7.1.5.3.
Message arrives on read queue (from communication channel)
.PP
Read put routine
\f2aoecommrput\fP
works as follows:
.IP \(bu
If this is a control message
(\f2mp\fP\f(CW\s-1->db_type\0\s+1\fP\f(CW\s-1>=\0\s+1\fP\f(CW\s-1QPCTL\s+1\fP),
pass the message downstream.
.IP \(bu
If this is a data message
and
the channel is active,
call
\f2putq\fP(9F)
to place the message on our read queue
for later processing by
\f2aoecommrsrv\fP.
.IP \(bu
If this is a data message
and
the channel is not active,
free the message.
.PP
Read service routine
\f2aoecommrsrv\fP
loops calling
\f2getq\fP(9F),
processing each message as follows:
.IP \(bu
If the message is of any type but
\f(CW\s-1M_DATA\s+1\fP,
pass it downstream.
.IP \(bu
If this is a data message
and the channel has a receiver routine,
call the receiver.
.IP \(bu
If this is a data message
and no receiver is registered,
pass it to
\f2aoecomm_log\fP
with code
\f(CW\s-1ACLUNSOL\s+1\fP.
.SH
7.1.5.4.
Module popped
.PP
Close routine
\f2aoecommclose\fP
does the following:
.IP \(bu
Call
\f2qprocsoff\fP(9F)
to suspend message processing on the channel.
.IP \(bu
Remove the
\f(CW\s-1Aoechan\s+1\fP
from the
\f2aoechan\fP
table,
making the channel unavailable to
\f2aoecomm_initdriver\fP
and
\f2aoecomm_send\fP.
.IP \(bu
Discard any pending cookie for this channel.
.IP \(bu
If a receiver was registered for this channel,
call the receiver routine with messsage pointer
\f(CW\s-1NULL\s+1\fP
to announce that the channel is shutting down.
.IP \(bu
Free the
\f(CW\s-1Aoechan\s+1\fP.
.SH
7.2.
\f(CW\s-1aoectl\s+1\fP
.SH
7.2.1.
Configuration
.PP
A single instance of
\f(CW\s-1aoectl\s+1\fP
is configured by driver configuration file
\f(CW\s-1aoectl.conf\s+1\fP,
attached to the
\f(CW\s-1pseudo\s+1\fP
device nexus.
There are two minor devices:
.IP
.in -\n(PIu
\f(CW\s-1/devices/pseudo/aoectl@0:ctl\s+1\fP (\f(CW\s-1/dev/aoectl\s+1\fP)
.in
Write-only control device.
Each
\f2write\fP
system call provides a complete
AoE control protocol
message;
the return value is the size of the buffer written
if the command succeeded,
\-1
with an appropriate
\f2errno\fP
value if it failed,
zero for a few special cases.
Concurrent opens are allowed,
but commands are handled one at a time
without overlap.
.IP
.in -\n(PIu
\f(CW\s-1/devices/pseudo/aoectl@0:mon\s+1\fP (\f(CW\s-1/dev/aoemon\s+1\fP)
.in
Read-only control device;
concurrent opens forbidden.
Each
\f2read\fP
returns one or more rejected AoE messages.
.PP
Neither device supports polling;
neither has any
\f2ioctl\fP
commands.
.SH
7.2.2.
Source code
.PP
All code is in a single source file,
\f(CW\s-1aoecomm.c\s+1\fP.
It uses AoE-specific header files
\f(CW\s-1aoeproto.h\s+1\fP,
\f(CW\s-1aoecomm.h\s+1\fP,
\f(CW\s-1aoecommproto.h\s+1\fP,
and
\f(CW\s-1etherproto.h\s+1\fP.
.SH
7.2.3.
Data structures
.PP
None is used outside the
\f(CW\s-1aoectl\s+1\fP
module.
.PP
.KS
.RS
\f(CW\s-1static Aoectlstate aoectlstate;\s+1\fP
.br
.RE
.KE
Static private context data,
including:
.IP \(bu
Ring of
\f(CW\s-1Aoemon\s+1\fP
structures
used to store messages
pending receipt by
\f(CW\s-1/dev/aoemon\s+1\fP.
.IP \(bu
Two pointers into the ring:
\f(CW\s-1rmon\s+1\fP,
the next message to be read;
\f(CW\s-1wmon\s+1\fP,
the next to be written.
.IP \(bu
Number of
\f(CW\s-1Aoemon\s+1\fPs
in the ring.
.IP \(bu
\f(CW\s-1kmutex_t\s+1\fP
lock to prevent concurrent use of
\f(CW\s-1/dev/aoectl\s+1\fP.
.IP \(bu
\f(CW\s-1kmutex_t\s+1\fP
lock to prevent concurrent access
to the ring buffer.
.IP \(bu
\f(CW\s-1kcondvar_t\s+1\fP
condition variable
on which reads from
\f(CW\s-1/dev/aoemon\s+1\fP
sleep if no messages were available;
a flag indicating that some process is sleeping.
.IP \(bu
Pointer to the Solaris
\f(CW\s-1dev_info_t\s+1\fP
structure for the sole device instance.
.PP
Each
\f(CW\s-1Aoemon\s+1\fP
contains:
.IP \(bu
Data to be logged:
a pointer to a STREAMS message,
an integer log-message type,
the associated AoE channel number.
.IP \(bu
Count of messages lost
before this message was received,
if any.
Incremented if a message arrives
and there's no place to put it.
.IP \(bu
\f(CW\s-1next\s+1\fP,
a pointer to the next
\f(CW\s-1Aoemon\s+1\fP
in the ring.
.LP
The buffer ring
is allocated when the
\f(CW\s-1/dev/aoemon\s+1\fP
device
is opened,
and the
\f(CW\s-1next\s+1\fP-\f(CW\s-1Aoemon\s+1\fP
pointers set to sew the
\f(CW\s-1Aoemon\s+1\fPs
into a ring.
All
\f(CW\s-1Aoemon\s+1\fPs
(and any unread messages)
are freed when the device is closed.
When
\f(CW\s-1aoectlstate.wmon\0\s+1\fP\f(CW\s-1==\0\s+1\fP\f(CW\s-1aoectlstate.rmon\s+1\fP,
the ring is empty;
when
\f(CW\s-1aoectlstate.wmon->next\0\s+1\fP\f(CW\s-1==\0\s+1\fP\f(CW\s-1aoectlstate.rmon\s+1\fP,
the ring is full.
Notice that there is always at least one empty
\f(CW\s-1Aoemon\s+1\fP
following that pointed to by
\f(CW\s-1aoectlstate.rmon\s+1\fP.
.PP
When a message arrives:
.IP \(bu
If the ring is full,
increment the lost-messages counter in
\f(CW\s-1\(**\s+1\fP\f(CW\s-1aoectlstate.wmon\s+1\fP.
.IP \(bu
Otherwise
store the message in
\f(CW\s-1\(**\s+1\fP\f(CW\s-1aoectlstate.wmon\s+1\fP,
and set
\f(CW\s-1aoectlstate.wmon\s+1\fP
to
\f(CW\s-1aoectlstate.wmon->next\s+1\fP.
.PP
When it is desired to read a message:
.IP \(bu
If
\f(CW\s-1aoectlstate.wmon\0\s+1\fP\f(CW\s-1==\0\s+1\fP\f(CW\s-1aoectlstate.rmon\s+1\fP,
no message is available.
.IP \(bu
Otherwise
consume the message stored in
\f(CW\s-1\(**\s+1\fP\f(CW\s-1aoectlstate.rmon\s+1\fP
and set
\f(CW\s-1aoectlstate.rmon\s+1\fP
to
\f(CW\s-1aoectlstate.rmon->next\s+1\fP.
.SH
7.2.4.
Exports
.PP
Standard Solaris loadable-module and device-driver
entry points and data structures are supplied:
in particular
\f2_init\fP,
\f2_fini\fP,
and
\f2_info\fP
routines,
a
\f(CW\s-1modlinkage\s+1\fP
structure (and its many children),
\f(CW\s-1dev_ops\s+1\fP
and
\f(CW\s-1cb_ops\s+1\fP
structures.
A
\f(CW\s-1_depends_on\s+1\fP
string declares a dependency on
\f(CW\s-1aoecomm\s+1\fP,
since
\f(CW\s-1aoectl\s+1\fP
calls procedures
supplied by that module.
.PP
Only
\f2_init\fP,
\f2_fini\fP,
and
\f2_info\fP
are global.
\f2_init\fP
calls
\f2mod_install\fP
to tell the system where to find the
\f(CW\s-1modlinkage\s+1\fP
structure
through which the other data structures and routines
can be located.
Apparently
\f(CW\s-1_depends_on\s+1\fP
need not be global to work.
.PP
Internal routine
\f2aoectlintr\fP
is registered with
\f(CW\s-1aoecomm\s+1\fP
as the global message receiver
if
\f(CW\s-1/dev/aoemon\s+1\fP
is open.
.SH
7.2.5.
Algorithms
.SH
7.2.5.1.
Open device
.PP
Open routine
\f2aoectlopen\fP
acts according to the minor device
opened:
.IP
.in -\n(PIu
\f(CW\s-1/dev/aoectl\s+1\fP
.in
Return success.
.IP
.in -\n(PIu
\f(CW\s-1/dev/aoemon\s+1\fP
.in
.RS
.IP \(ci
If there is already a buffer ring
(the device is already open),
return
\f(CW\s-1EBUSY\s+1\fP.
.IP \(ci
Call
\f(CW\s-1aoecomm_initlog(aoectlintr)\s+1\fP
to register
\f2aoectlintr\fP
as the logger.
If this fails,
return
\f(CW\s-1EBUSY\s+1\fP.
.IP \(ci
Allocate the ring buffer.
If integer property
\f(CW\s-1monbufs\s+1\fP
exists,
allocate that many messages;
the default is 32.
If this fails,
deregister the logger
(call
\f(CW\s-1aoecomm_initlog(NULL)\s+1\fP)
and return
\f(CW\s-1EBUSY\s+1\fP.
.RE
.IP
.in -\n(PIu
Any other device
.in
Return
\f(CW\s-1ENXIO\s+1\fP.
.SH
7.2.5.2.
Close device
.PP
Close routine
\f2aoectlclose\fP
acts according to the minor device closed:
.IP
.in -\n(PIu
\f(CW\s-1/dev/aoemon\s+1\fP
.in
.RS
.IP \(ci
Call
\f(CW\s-1aoecomm_initlog(NULL)\s+1\fP
to deregister the logger.
.IP \(ci
Free the buffer ring
and any STREAMS messages still stored there.
.IP \(ci
Return success.
.RE
.IP
.in -\n(PIu
Any other device
.in
Return success.
.SH
7.2.5.3.
Write to device
.PP
Write routine
\f2aoectlwrite\fP
acts according to the minor device written:
.IP
.in -\n(PIu
\f(CW\s-1/dev/aoectl\s+1\fP
.in
Fetch the user's buffer,
and parse it as if it contains
a single complete
AoE control protocol
message.
Process according to message type:
.RS
.IP
.in -\n(PIu
\f(CW\s-1ACINITCHAN\s+1\fP
.in
Extract
\f2chan\fP
and
\f2cookie\fP
values;
call
\f(CW\s-1aoecomm_initchan(\s+1\fP\f2chan\fP\f(CW\s-1,\s+1\fP\0\f2cookie\fP\f(CW\s-1)\s+1\fP.
On success,
return as if all bytes were successfully written;
on failure,
return
\f(CW\s-1ENXIO\s+1\fP.
Only the channel and cookie values
are used here;
other parameters are ignored.
.IP
.in -\n(PIu
\f(CW\s-1ACSEND\s+1\fP
.in
Extract channel number
and message contents,
allocate a STREAMS buffer,
and copy the message there.
Call
\f(CW\s-1aoecomm_send(\s+1\fP\f2chan\fP\f(CW\s-1,\s+1\fP\0\f2message\fP\f(CW\s-1)\s+1\fP.
If all is well,
return success;
if no STREAMS buffer was available,
\f(CW\s-1ENOMEM\s+1\fP;
if
\f2aoecomm_send\fP
failed,
\f(CW\s-1ENXIO\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1ACDEVENAB\s+1\fP
.in
Extract
\f2chan\fP,
\f2maj\fP,
\f2min\fP,
and
\f2cmd\fP
values.
Call
\f(CW\s-1aoecomm_devenab(\s+1\fP\f2chan\fP\f(CW\s-1,\s+1\fP\0\f2maj\fP\f(CW\s-1,\s+1\fP\0\f2min\fP\f(CW\s-1,\s+1\fP\0\f2cmd\fP\f(CW\s-1)\s+1\fP.
If the result is
\f(CW\s-1ADENAB\s+1\fP,
return as if all bytes were written;
if
\f(CW\s-1ADDISAB\s+1\fP,
return zero (no error, but nothing written).
.RE
If the message is ill-formed
or has an unknown type,
return
\f(CW\s-1ENXIO\s+1\fP.
.IP
.in -\n(PIu
Any other device
.in
Return
\f(CW\s-1ENODEV\s+1\fP.
.SH
7.2.5.4.
Read from device
.PP
Read routine
\f2aoectlread\fP
acts according to the minor device read:
.IP
.in -\n(PIu
\f(CW\s-1/dev/aoemon\s+1\fP
.in
.RS
.IP \(ci
If
\f(CW\s-1aoectlstate.rmon\0\s+1\fP\f(CW\s-1==\0\s+1\fP\f(CW\s-1aoectlstate.wmon\s+1\fP,
set the process-sleeping flag,
wait on the condition variable,
and try again.
If a signal arrives while waiting,
return
\f(CW\s-1EINTR\s+1\fP.
.IP \(ci
If
\f(CW\s-1aoectlstate.rmon\0\s+1\fP\f(CW\s-1!=\0\s+1\fP\f(CW\s-1aoectlstate.wmon\s+1\fP
and the user's buffer has room for the message stored in
\f(CW\s-1\(**\s+1\fP\f(CW\s-1aoectlstat.rmon\s+1\fP,
compose an
\f(CW\s-1ACLOG\s+1\fP
control-message header including the channel number,
log-message type,
and lost-message count
stored in that
\f(CW\s-1Aoebuf\s+1\fP;
use
\f2uiomove\fP(9F) to
copy the header,
then the data contents of the message,
to the user's buffer.
Free the STREAMS message;
clear the lost-message counter in
\f(CW\s-1\(**\s+1\fP\f(CW\s-1aoectlstate.rmon\s+1\fP;
set
\f(CW\s-1aoectlstate.rmon\s+1\fP
to
\f(CW\s-1aoectlstate.rmon->next\s+1\fP.
.IP \(ci
Loop until
\f(CW\s-1aoectlstate.rmon\0\s+1\fP\f(CW\s-1==\0\s+1\fP\f(CW\s-1aoectlstate.wmon\s+1\fP,
the user's buffer has no room for the next message,
or
\f2uiomove\fP
returns an error.
In the last case
return the same error as
\f2uiomove\fP;
otherwise return the number of bytes read.
.RE
Notice that if the first available message
is longer than the user's buffer,
\f2read\fP
returns zero.
.IP
.in -\n(PIu
Any other device
.in
Return
\f(CW\s-1ENODEV\s+1\fP.
.SH
7.2.5.5.
Message received
.KS
.RS
\f(CW\s-1static void aoectlintr(chan, code, mp)\s+1\fP
.br
\f(CW\s-1int chan, code;\s+1\fP
.br
\f(CW\s-1mblk_t *mp;\s+1\fP
.br
.RE
.KE
.PP
If there's room in the ring,
store
\f2mp\fP,
\f2code\fP
(the log-message type),
and
\f2chan\fP
in
\f(CW\s-1\(**\s+1\fP\f(CW\s-1aoectlstate.wmon\s+1\fP
and set
\f(CW\s-1aoectlstate.wmon\s+1\fP
to
\f(CW\s-1aoectlstate.wmon->next\s+1\fP.
If the ring is full,
increment the lost-messages counter in
\f(CW\s-1\(**\s+1\fP\f(CW\s-1aoectlstate.wmon\s+1\fP
and free
\f2mp\fP.
In either case,
if the process-sleeping flag is set,
signal the condition variable,
awakening any process blocked reading
\f(CW\s-1/dev/aoemon\s+1\fP.
.SH
7.3.
\f(CW\s-1aoed\s+1\fP
.SH
7.3.1.
Configuration
.PP
Each instance of
\f(CW\s-1aoed\s+1\fP
represents one AoE disk target,
with
13 or 21 block and 13 or 21 character
minor devices:
.IP \(bu
Eight (SPARC) or 16 (x86)
devices accessing the standard Solaris disk partitions.
.IP \(bu
Four devices,
present even on SPARC systems,
accessing the direct DOS partitions
if a DOS label is present.
.IP \(bu
A final partition accessing the whole disk
regardless of partition tables,
reserved cylinders,
and other operating-system impedimenta.
.PP
Links in directories
\f(CW\s-1/dev/dsk\s+1\fP
and
\f(CW\s-1/dev/rdsk\s+1\fP
use
fixed `controller' name
\f(CW\s-1ca\s+1\fP.
.PP
These devices are created for
\f(CW\s-1aoed\s+1\fP:
.IP
.in -\n(PIu
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:a\s+1\fP (\f(CW\s-1/dev/dsk/cad\s+1\fP\f2inst\fP\f(CW\s-1s0\s+1\fP)
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\0\0\0\0\&.\|.\|.
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:p\s+1\fP (\f(CW\s-1/dev/dsk/cad\s+1\fP\f2inst\fP\f(CW\s-1s15\s+1\fP)
.in
Block devices for instance
\f2inst\fP,
partitions
\f(CW\s-1a\s+1\fP-\f(CW\s-1h\s+1\fP
(and on to
\f(CW\s-1p\s+1\fP
on x86)
in
\f(CW\s-1/devices\s+1\fP-speak,
or
\f(CW\s-1s0\s+1\fP-\f(CW\s-1s7\s+1\fP
(\f(CW\s-1s15\s+1\fP)
in
\f(CW\s-1/dev\s+1\fP-speak.
.IP
.in -\n(PIu
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:r\s+1\fP (\f(CW\s-1/dev/dsk/cad\s+1\fP\f2inst\fP\f(CW\s-1p1\s+1\fP)
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\0\0\0\0\&.\|.\|.
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:u\s+1\fP (\f(CW\s-1/dev/dsk/cad\s+1\fP\f2inst\fP\f(CW\s-1p4\s+1\fP)
.in
Block devices for instance
\f2inst\fP,
partitions
\f(CW\s-1r\s+1\fP-\f(CW\s-1u\s+1\fP
in
\f(CW\s-1/devices\s+1\fP-speak,
or
\f(CW\s-1p1\s+1\fP-\f(CW\s-1p4\s+1\fP
in
\f(CW\s-1/dev\s+1\fP-speak.
.IP
.in -\n(PIu
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:wd\s+1\fP (\f(CW\s-1/dev/dsk/cad\s+1\fP\f2inst\fP)
.in
Whole-disk block device for
\f2inst\fP.
.IP
.in -\n(PIu
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:a,raw\s+1\fP (\f(CW\s-1/dev/rdsk/cad\s+1\fP\f2inst\fP\f(CW\s-1s0\s+1\fP)
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\0\0\0\0\&.\|.\|.
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:h,raw\s+1\fP (\f(CW\s-1/dev/rdsk/cad\s+1\fP\f2inst\fP\f(CW\s-1s7\s+1\fP)
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:r,raw\s+1\fP (\f(CW\s-1/dev/rdsk/cad\s+1\fP\f2inst\fP\f(CW\s-1p1\s+1\fP)
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\0\0\0\0\&.\|.\|.
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:u,raw\s+1\fP (\f(CW\s-1/dev/rdsk/cad\s+1\fP\f2inst\fP\f(CW\s-1p4\s+1\fP)
.IP
.in -\n(PIu
\f(CW\s-1/devices/pseudo/aoed@\s+1\fP\f2inst\fP\f(CW\s-1:wd,raw\s+1\fP (\f(CW\s-1/dev/rdsk/cad\s+1\fP\f2inst\fP)
.in
Standard and whole-disk character devices for
\f2inst\fP.
.PP
Sun's standard device drivers create the DOS-partition
devices only for certain kinds of disk
and only on Solaris/x86;
when the DOS devices exist,
the whole-disk device in
\f(CW\s-1/dev\s+1\fP
is called
\f(CW\s-1p0\s+1\fP.
The AoE driver supports DOS partition tables
on Solaris/SPARC as well,
but omits the
\f(CW\s-1p0\s+1\fP
name to avoid a bug in tools like
\f(CW\s-1format\s+1\fP(1M).
.PP
The driver calls
\f2ddi_create_minor_node\fP(9F)
with node type
\f(CW\s-1DDI_NT_PSEUDO\s+1\fP,
rather than the
\f(CW\s-1DDI_NT_BLOCK\s+1\fP
type normally used for disks.
AoE-specific entries in
\f(CW\s-1/dev/devlink.tab\s+1\fP
create the
\f(CW\s-1/dev/dsk\s+1\fP
and
\f(CW\s-1/dev/rdsk\s+1\fP
links.
This circumvents a
bug in
\f2devfsadm\fP(1M).
.PP
Every device instance must be listed in
in driver configuration file
\f(CW\s-1aoed.conf\s+1\fP,
read by the system when the
\f(CW\s-1aoed\s+1\fP
driver module is loaded.
\f2Driver.conf\fP(4D)
gives the file format.
Each entry must declare
\f(CW\s-1name="aoed"\s+1\fP,
\f(CW\s-1parent="pseudo"\s+1\fP,
and these integer-valued properties:
.IP
.in -\n(PIu
\f(CW\s-1instance=\s+1\fP\f2inst\fP
.in
Use instance number
\f2inst\fP
internally to identify this device to the driver;
display it in device names
and log messages.
.IP
.in -\n(PIu
\f(CW\s-1aoechan=\s+1\fP\f2c\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1aoemaj=\s+1\fP\f2MM\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1aoemin=\s+1\fP\f2mm\fP
.in
Bind AoE target address
\f2c\fP/\f2MM\fP/\f2mm\fP
to this instance:
\f2c\fP
identifies an
\f(CW\s-1aoecomm\s+1\fP
channel,
\f2MM\fP
and
\f2mm\fP
give the AoE protocol major and minor addresses.
.PP
The copy of
\f(CW\s-1aoed.conf\s+1\fP
installed with the subsystem
declares 30 devices:
instances 0-14 and 100-114
for
\f(CW\s-1aoechan\s+1\fP
0,
\f(CW\s-1aoemaj\s+1\fP
0 and 1,
\f(CW\s-1aoemin\s+1\fP
0-14.
This is enough for two EtherDrive shelves,
each with 15 slots.
The system administrator may edit
the file to add or remove devices
(with
\f2aoemkconf\fP
or by hand)
or to apply his own numbering convention,
as explained in the
Installation and Operation Guide.
.PP
By convention,
decimal instance number
\f(CW\s-1instance=\s+1\fP\f2cMMmm\fP
corresponds to target address
\f(CW\s-1aoechan=\s+1\fP\f2c\fP,
\f(CW\s-1aoemaj=\s+1\fP\f2MM\fP,
\f(CW\s-1aoemin=\s+1\fP\f2mm\fP.
Whatever mapping appears in the entries in
\f(CW\s-1aoed.conf\s+1\fP
is honoured,
however;
the only program that knows about it
is the
\f2aoemkconf\fP
program for generating configuration entries.
Any desired instance-to-address mapping will work
so long as instance numbers are not reused
and none is too big.
.PP
In 32-bit versions of Solaris,
18 bits are available for minor device numbers.
13 (21 on x86) minor devices are created for each instance,
allowing
262144/13 = 20164
(262144/21 = 12483) instances.
In 64-bit systems,
32 bits are available,
hence there may be
4294967296/13 = 330382099
(4294967296/21 = 204522252).
.PP
These optional integer-valued properties may be included:
.IP
.in -\n(PIu
\f(CW\s-1timeout=\s+1\fP\f2ms\fP
.in
If an I/O operation hasn't completed within
\f2ms\fP
milliseconds,
assume it never will;
send the command again,
or after several retries
give up and report an error.
The default is 200 milliseconds.
.IP
.in -\n(PIu
\f(CW\s-1hd=\s+1\fP\f2nhead\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1sec=\s+1\fP\f2nsec\fP
.in
Assume this device has cylinders of
\f2nhead\fP
heads,
and tracks of
\f2nsec\fP
sectors,
ignoring any information supplied by the hardware
or that invented by
\f2fudgegeom\fP.
Honoured only if both properties are included;
if only one is given
it is ignored.
.IP
.in -\n(PIu
\f(CW\s-1maxdata=\s+1\fP\f2len\fP
.in
Allow a data segment of at most
\f2len\fP
bytes,
overriding the default:
the minimum of
that offered in the target's
Query-Config response message
(1024 if no value returned),
and that configured
for the specified
\f2aoechan\fP.
.IP
.in -\n(PIu
\f(CW\s-1maxbuf=\s+1\fP\f2nbuf\fP
.in
Limit the maximum buffer count
(number of concurrent messages the target can handle)
to
\f2nbuf\fP
rather than the default 256.
The value is in any case bounded above
by that suggested by the target.
.IP
.in -\n(PIu
\f(CW\s-1disksort=\s+1\fP\f2enable\fP
.in
If
\f2enable\fP
is 1,
call the standard
\f2disksort\fP(9F)
kernel routine
to sort queued disk operations;
if zero,
just use FIFO ordering.
.PP
On Solaris 9 and earlier versions,
changes to
\f(CW\s-1aoed.conf\s+1\fP
take effect only when the
\f(CW\s-1aoed\s+1\fP
kernel module is loaded.
If AoE disks have already been used,
the file will not be effective
until the driver is unloaded,
which means every AoE disk device
must be unmounted and closed.
Often it is simplest just to reboot the system.
.PP
On Solaris 10,
\f2update_drv\fP(1M)
updates the configuration.
With option
\f(CW\s-1-f\s+1\fP
it can do so without a module unload.
Device instances presently in use cannot be changed
on the fly,
but new devices may be added
without rebooting.
.SH
7.3.2.
Source code
.PP
The
\f(CW\s-1aoed\s+1\fP
driver source code
is broken into several files:
.IP
.in -\n(PIu
\f(CW\s-1aoed.c\s+1\fP
.in
Main program:
official driver entry points
and data structures.
.IP
.in -\n(PIu
\f(CW\s-1addadk.c\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1aduscsi.c\s+1\fP
.in
Handle ATA- and SCSI- (sic)
specific
\f2ioctl\fP
calls.
.IP
.in -\n(PIu
\f(CW\s-1adio.c\s+1\fP
.in
Generate and send AoE commands
for an I/O request,
or as needed during device initialization.
Process AoE replies.
Retransmit commands whose answers don't arrive in
a specified time;
cancel commands that have suffered too many retries.
.IP
.in -\n(PIu
\f(CW\s-1addos.c\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1adefi.c\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1advtoc.c\s+1\fP
.in
Read and write disk labels and partition tables;
separate implementations for
each of the several
label formats.
.IP
.in -\n(PIu
\f(CW\s-1adcmd.c\s+1\fP
.in
Allocate,
free,
and manage lists of
pending AoE commands.
.IP
.in -\n(PIu
\f(CW\s-1adsubr.c\s+1\fP
.in
Miscellaneous subroutines:
find the
\f(CW\s-1Adrive\s+1\fP
structure associated with a Solaris device;
error-message helpers.
.IP
.in -\n(PIu
\f(CW\s-1adtrace.c\s+1\fP
.in
Simple event-tracing mechanism;
used only when debugging the driver.
.IP
.in -\n(PIu
\f(CW\s-1aoed.h\s+1\fP
.in
Header file declaring data types and instances
and procedure prototypes
used in more than one source file.
.IP
.in -\n(PIu
\f(CW\s-1dadkio32.h\s+1\fP
.in
Header file declaring a 32-bit version of certain
data structures used by the Solaris
\f(CW\s-1DKIOCTL_RWCMD\s+1\fP
\f2ioctl\fP.
For some reason Sun's header files leave these out.
.PP
Many subroutines and data objects are meant to be shared among
\f(CW\s-1aoed\s+1\fP
source files but not with the rest of the kernel.
The system provides no known way to do exactly that.
(Perhaps
\f2ld\fP(1)
has an option to do the trick,
but we have yet to discover it.)
If every such symbol began with a prefix unique to
\f(CW\s-1aoed\s+1\fP
all would be well,
but that would clutter the source code.
As a compromise,
such names are overlaid with macros in
\f(CW\s-1aoed.h\s+1\fP,
in effect prepending
\f(CW\s-1aoed_\s+1\fP.
For example,
the prototype for
\f2acalloc\fP
is preceded by
.RS
\f(CW\s-1#define	acalloc	aoed_acalloc\s+1\fP
.br
.RE
.PP
To make it easier to
include optional code only as needed
(\f2trace\fP
in particular),
all object files except main program
\f(CW\s-1aoed.o\s+1\fP
are collected into object library
\f(CW\s-1adlib.a\s+1\fP.
The module binary is generated by running
.RS
\f(CW\s-1ld -r -o aoed aoed.o adlib.a\s+1\fP
.br
.RE
.SH
7.3.3.
Data structures
.PP
None is used outside the
\f(CW\s-1aoed\s+1\fP
module.
.SH
7.3.3.1.
\f(CW\s-1Adrive\s+1\fP
.PP
An
\f(CW\s-1Adrive\s+1\fP
structure
is allocated for each device instance
when it is attached,
freed when it is detached,
containing:
.IP \(bu
An integer state with one of the following values:
.RS
.IP
.in -\n(PIu
\f(CW\s-1ADCLOSED\s+1\fP
.in
No process has this device open;
I/O forbidden.
.IP
.in -\n(PIu
\f(CW\s-1ADWAOE\s+1\fP
.in
First stage in device setup:
waiting for AoE Query-Config response.
.IP
.in -\n(PIu
\f(CW\s-1ADWATA\s+1\fP
.in
Second stage in device setup:
Query-Config response received,
awaiting ATA IDENTIFY response.
MAC address, max-buffer,
and max-data-segment values valid.
.IP
.in -\n(PIu
\f(CW\s-1ADGATA\s+1\fP
.in
Third stage in device setup:
ATA IDENTIFY response received,
number of sectors valid.
I/O is now allowed,
but only to the
\f(CW\s-1wd\s+1\fP
partition.
.IP
.in -\n(PIu
\f(CW\s-1ADWPTAB\s+1\fP
.in
Fourth stage in device setup:
waiting for disk label to be read.
.IP
.in -\n(PIu
\f(CW\s-1ADREADY\s+1\fP
.in
Setup complete;
regular I/O allowed
to any partition defined by the label,
whether real or synthetic.
Disk label, geometry,
and
partition table valid,
as allowed by the label type.
.RE
State values are ordered
as above:
for example,
\f(CW\s-1ap->state\0\s+1\fP\f(CW\s-1>=\0\s+1\fP\f(CW\s-1ADGATA\s+1\fP
is true only if IDENTIFY information has been received.
.IP \(bu
A drive-is-closing flag,
used to ensure that no new I/O is begun
during
\f2adclose\fP.
.IP \(bu
Bit-masks (of integral type
\f(CW\s-1Pbits\s+1\fP
to hide the number of bits required)
indicating which block and which character minor devices
are open;
a count of the outstanding layered opens.
Used to decide whether this is the first open
or last close.
.IP \(bu
Static configuration values:
.RS
.IP \(ci
Target address
(\f2aoechan\fP, \f2aoemaj\fP, \f2aoemin\fP)
of the associated AoE device;
determined from properties
initialized from
\f(CW\s-1aoed.conf\s+1\fP.
.IP \(ci
Optional property values from
\f(CW\s-1aoed.conf\s+1\fP:
I/O timeout value,
substitute head and sector counts.
.IP \(ci
Solaris instance number
and
\f(CW\s-1dev_info_t\s+1\fP
pointer.
.IP \(ci
Solaris major device number,
discovered when the device is first opened;
needed for a few kernel service routines
that require a complete
\f(CW\s-1dev_t\s+1\fP
as argument.
.IP \(ci
Other static flags:
maximum data-segment size
was set by an configuration property
and should not be overridden;
this system uses the older,
incompatible argument to the
\f(CW\s-1DKIOCGETEFI\s+1\fP
and
\f(CW\s-1DKIOCSETEFI\s+1\fP
\f2ioctl\fPs
\&.
.RE
.IP \(bu
Configuration properties of the AoE target:
.RS
.IP \(ci
Target's Ethernet MAC address.
.IP \(ci
Maximum buffer count,
i.e. number of concurrent messages
the target can handle.
.IP \(ci
Maximum data-segment size,
i.e. the longest read or write transfer
allowed in an ATA command.
.RE
.IP \(bu
Geometry,
partition,
and other information about the target disk:
.RS
.IP \(ci
Number of logical blocks in the target disk.
.IP \(ci
Dynamic flags,
including the label type:
\f(CW\s-1AVLABVTOC\s+1\fP,
\f(CW\s-1AVLABEFI\s+1\fP,
\f(CW\s-1AVLABDOS\s+1\fP,
or
\f(CW\s-1(AVLABDOS|AVLABVTOC)\s+1\fP
(when the VTOC label is
encapsulated in a DOS partition).
.IP \(ci
Cylinder size;
often a fake value
invented by
\f2fudgegeom\fP
or
\f2adreceive\fP.
.IP \(ci
Solaris
\f(CW\s-1dk_geom\s+1\fP
structure describing disk geometry;
meaningful only with label type
\f(CW\s-1AVLABDOS\s+1\fP
or
\f(CW\s-1AVLABVTOC\s+1\fP.
.IP \(ci
Solaris
\f(CW\s-1vtoc\s+1\fP
(volume table-of-contents,
i.e. on-disk label;
valid only under label type
\f(CW\s-1LABVTOC\s+1\fP.
.IP \(ci
Starting sector
and length
of each partition,
including
the whole-disk partition.
An invalid partition has length zero.
.IP \(ci
Starting sector and length of the Solaris-labelled disk area.
If the disk has a DOS partition encapsulating a Solaris label,
these are the starting sector and length of that partition;
otherwise zero and the length of the disk.
.IP \(ci
Serial-number,
disk firmware version,
and disk model strings
returned by the ATA IDENTIFY command.
.RE
.IP \(bu
Information about pending and active I/O requests:
.RS
.IP \(ci
List of
\f(CW\s-1Acmd\s+1\fP
structures
available for allocation by
\f2acalloc\fP.
.IP \(ci
Tag value to be used in the next command
sent to this target.
.IP \(ci
Count of active commands,
those in the list of pending commands
in this channel's
\f(CW\s-1Achan\s+1\fP.
.IP \(ci
Solaris
\f(CW\s-1diskhd\s+1\fP
structure
containing a queue of
\f(CW\s-1buf\s+1\fP
structures for pending I/O requests,
suitable for use by
\f2disksort\fP(9F).
.IP \(ci
Pointers to the head and tail of
the list of active
\f(CW\s-1buf\s+1\fP
structures for this device,
i.e. those whose transfers have been at least partly begun.
.RE
.IP \(bu
A
\f(CW\s-1kmutex_t\s+1\fP
lock to prevent concurrent access to non-static values in this
\f(CW\s-1Adrive\s+1\fP.
.IP \(bu
A
\f(CW\s-1kcondvar_t\s+1\fP
condition variable
used while waiting for initialization to finish.
.IP \(bu
Structures used to collect statistics
displayed by
\f2iostat\fP
and
\f2kstat\fP(1M):
a
\f(CW\s-1kstat_t\s+1\fP
of type
\f(CW\s-1KSTAT_TYPE_IO\s+1\fP
for the whole drive,
another for each partition
(including the whole-disk partition),
one named
\f(CW\s-1aoed\s+1\fP\f2inst\fP\f(CW\s-1,stats\s+1\fP
for a handful of per-drive counters
specific to
\f(CW\s-1aoed\s+1\fP,
another named
\f(CW\s-1aoed\s+1\fP\f2inst\fP\f(CW\s-1,conf\s+1\fP
to display per-drive configuration values.
.SH
7.3.3.2.
\f(CW\s-1Acmd\s+1\fP
.PP
Each
\f(CW\s-1Adrive\s+1\fP
has a pool of
\f(CW\s-1Acmd\s+1\fP
structures,
allocated when the device is first opened,
freed on last close.
The pool is initialized with as many
\f(CW\s-1Acmd\s+1\fPs
as the target allows concurrent commands.
When a command is composed and sent,
an
\f(CW\s-1Acmd\s+1\fP
is taken from the pool;
while the command awaits a response,
the
\f(CW\s-1Acmd\s+1\fP
is kept in a per-channel pending-command list;
when a response is received
or the command is abandoned after a timeout,
the
\f(CW\s-1Acmd\s+1\fP
is returned to the pool.
If the pool is empty
no new commands may be sent
until an outstanding command has completed or timed out
and its
\f(CW\s-1Acmd\s+1\fP
has been returned to the pool.
.PP
Each
\f(CW\s-1Acmd\s+1\fP
contains:
.IP \(bu
The AoE command message to be sent,
excluding any data sectors to be written.
.IP \(bu
A pointer to the associated
\f(CW\s-1Adrive\s+1\fP.
.IP \(bu
A pointer to the associated
\f(CW\s-1buf\s+1\fP
structure.
If
\f(CW\s-1NULL\s+1\fP
this is not an I/O command.
.IP \(bu
A pointer to the associated
\f(CW\s-1kcondvar_t\s+1\fP
condition variable;
\f(CW\s-1NULL\s+1\fP
if none.
.IP \(bu
The AoE tag value for this command.
.IP \(bu
How long to wait for a response
before declaring that this command has timed out;
how many retries have occurred,
and how many are allowed.
.IP \(bu
When this command was first started,
and when it next times out,
in absolute clock ticks
as returned by
\f2ddi_get_lbolt\fP(9F).
.IP \(bu
If an I/O command,
the
absolute sector number of the first sector of this transfer,
and the byte offset and length
of the segment handled by this command.
.IP \(bu
Next-\f(CW\s-1Acmd\s+1\fP
and
previous-\f(CW\s-1Acmd\s+1\fP
pointers,
for the several lists in which
\f(CW\s-1Acmd\s+1\fPs
may be stored.
.IP \(bu
A state value,
used only for sanity checks:
value
\f(CW\s-1CSFREE\s+1\fP
if this
\f(CW\s-1Acmd\s+1\fP
is in the
drive's
free-command pool;
\f(CW\s-1CSCHAN\s+1\fP
if on the channel's
active-command list;
\f(CW\s-1CSLOOSE\s+1\fP
otherwise.
If an
\f(CW\s-1Acmd\s+1\fP
in the pool or on the list
is found to be in the wrong state,
or if an
\f(CW\s-1Acmd\s+1\fP
being added to either is not in state
\f(CW\s-1CSLOOSE\s+1\fP,
the system panics.
.LP
The state value was introduced when tracking
a subtle bug that freed an
\f(CW\s-1Acmd\s+1\fP
that was still on the active-channel list.
It remains because it's cheap insurance:
if another such bug creeps in,
it will be caught early and found more easily.
.SH
7.3.3.3.
\f(CW\s-1Achan\s+1\fP
.PP
Each
\f2aoechan\fP
named at least once in
\f(CW\s-1aoed.conf\s+1\fP
has an
\f(CW\s-1Achan\s+1\fP
structure.
\f(CW\s-1Achan\s+1\fPs
are allocated as needed as devices are attached,
in a dynamic array handled by
\f2ddi_soft_state_init\fP
and
\f2ddi_get_soft_state\fP(9F).
The whole array is freed
when the module is unloaded.
.PP
Each
\f(CW\s-1Achan\s+1\fP
contains:
.IP \(bu
Pointers to the head and tail of a list
of
\f(CW\s-1Acmd\s+1\fPs
representing commands awaiting responses
from this
\f2aoechan\fP.
.IP \(bu
A
\f(CW\s-1timeout_id_t\s+1\fP
identifying this channel's instance of
\f2chantimer\fP,
if any.
.IP \(bu
A channel-is-active flag.
.IP \(bu
A
\f(CW\s-1kmutex_t\s+1\fP
lock preventing concurrent access to the
\f(CW\s-1Achan\s+1\fP.
.SH
7.3.3.4.
\f(CW\s-1Abuf\s+1\fP
.PP
Every
\f(CW\s-1buf\s+1\fP
structure for a pending or active I/O transfer
has an associated
\f(CW\s-1Abuf\s+1\fP
structure,
allocated when the
\f(CW\s-1buf\s+1\fP
is accepted by
\f2adstrategy\fP.
The address of the
\f(CW\s-1Abuf\s+1\fP
is stored in
\f2bp\fP\f(CW\s-1->b_private\s+1\fP.
The
\f(CW\s-1Abuf\s+1\fP
is freed before
\f2biodone\fP(9F)
is called.
.PP
Macro
\f(CW\s-1bptoabuf(bp)\s+1\fP
is a shorthand for
\f(CW\s-1(Abuf\0\s+1\fP\f(CW\s-1\(**\s+1\fP\f(CW\s-1)(bp->b_private)\s+1\fP.
.PP
Each
\f(CW\s-1Abuf\s+1\fP
contains:
.IP \(bu
The number of AoE commands outstanding
for this
\f(CW\s-1buf\s+1\fP.
.IP \(bu
The absolute logical sector number
to be used in the next AoE command
generated for this transfer.
.IP \(bu
The offset within the transfer
to the first byte to be read or written
by the next command.
.IP \(bu
The total number of bytes to be transferred.
.LP
The transfer associated with a
\f(CW\s-1buf\s+1\fP
structure
is complete when the offset value equals the total-bytes value
and the number of outstanding commands
becomes zero,
or when an error occurs.
.SH
7.3.4.
Exports
.PP
Standard Solaris loadable-module and device-driver
entry points and data structures are supplied:
in particular
\f2_init\fP,
\f2_fini\fP,
and
\f2_info\fP
routines,
a
\f(CW\s-1modlinkage\s+1\fP
structure (and its many children),
\f(CW\s-1dev_ops\s+1\fP
and
\f(CW\s-1cb_ops\s+1\fP
structures.
A
\f(CW\s-1_depends_on\s+1\fP
string declares a dependency on
\f(CW\s-1aoecomm\s+1\fP,
since
\f(CW\s-1aoed\s+1\fP
calls procedures
supplied by that module.
.PP
Only
\f2_init\fP,
\f2_fini\fP,
and
\f2_info\fP
are global.
\f2_init\fP
calls
\f2mod_install\fP
to tell the system where to find the
\f(CW\s-1modlinkage\s+1\fP
structure
through which the other data structures and routines
can be located.
Apparently
\f(CW\s-1_depends_on\s+1\fP
need not be global to work.
.PP
Internal routine
\f2adreceive\fP
is registered with
\f(CW\s-1aoecomm\s+1\fP
as the message receiver for each
\f2aoechan\fP
that has been used to send at least one message.
Internal routine
\f2chantimer\fP
runs as a self-renewing timer,
with one instance per
\f2aoechan\fP.
.SH
7.3.5.
Algorithms
.SH
7.3.5.1.
Attach device instance
.PP
Attach routine
\f2adattach\fP
works as follows:
.IP \(bu
Allocate an
\f(CW\s-1Adrive\s+1\fP
for this device instance.
Initialize it with constants at hand:
instance number and
\f(CW\s-1dev_info_t\s+1\fP
pointer;
\f(CW\s-1aoechan\s+1\fP,
\f(CW\s-1aoemaj\s+1\fP,
\f(CW\s-1aoemin\s+1\fP,
other
\f(CW\s-1aoed.conf\s+1\fP-entry
properties,
fetched with
\f2ddi_get_prop_int\fP(9F).
If any required property is missing,
return failure.
.IP \(bu
Call
\f2solversion\fP;
if this system is earlier than Solaris 10,
set the flag indicating an old-style argument
to the
\f(CW\s-1DKIOCGETEFI\s+1\fP
and
\f(CW\s-1DKIOCSETEFI\s+1\fP
\f2ioctl\fPs.
.IP \(bu
Call
\f2ddi_create_minor_node\fP(9F)
to create each of the eighteen minor devices
for this instance.
.IP \(bu
Initialize locks and condition variables
in this
\f(CW\s-1Adrive\s+1\fP.
.IP \(bu
Call
\f2ddi_report_dev\fP(9F)
to inform the system that the instance is ready.
.LP
\f2Adattach\fP
does not attempt to access the AoE target.
The
\f2aoechan\fP
may not yet be active.
Even if it is,
the target may not exist;
there's no point in stalling for several seconds for each
non-existent device configured in
\f(CW\s-1aoed.conf\s+1\fP.
.SH
7.3.5.2.
Open device partition
.PP
Open routine
\f2adopen\fP
works as follows:
.IP \(bu
Call
\f2aoecomm_devenab\fP
to check that the AoE target
has been enabled.
If not,
immediately return
\f(CW\s-1ENXIO\s+1\fP.
.IP \(bu
Create
\f(CW\s-1kstat_t\s+1\fP
structures
for the drive
and for the partition being opened,
as necessary.
.IP \(bu
If this is a non-blocking
\f2open\fP
(\f(CW\s-1FNDELAY\s+1\fP
or
\f(CW\s-1FNONBLOCK\s+1\fP
set in
\f2open\fP
flags)
and the
\f(CW\s-1Adrive\s+1\fP
state is
\f(CW\s-1ADCLOSED\s+1\fP,
call
\f2driveinit\fP
to start the initialization process.
.IP \(bu
If neither
\f(CW\s-1FNDELAY\s+1\fP
nor
\f(CW\s-1FNONBLOCK\s+1\fP
was set,
call
\f2openwait\fP
to start initialization if necessary
and wait for completion.
If
\f2openwait\fP
fails,
return the error reported.
Check whether the partition being opened
is of length zero;
if so,
return
\f(CW\s-1ENXIO\s+1\fP.
.IP \(bu
Mark this partition open according to the type of open call:
for block or character open,
set the appropriate bit
in the appropriate open-subdebvice bitmask;
for a layered open,
increment the layered-open count.
.PP
Drive initialization comprises these steps:
.IP
.in -\n(PIu
1.
\f2Driveinit\fP
called;
drive state is
\f(CW\s-1ADCLOSED\s+1\fP
.in
Allocate a single
\f(CW\s-1Acmd\s+1\fP
for this drive;
broadcast an AoE Query-Config command
with the desired
\f2aoemaj\fP
and
\f2aoemin\fP
addresses to the desired AoE channel,
to locate the target.
Set drive state to
\f(CW\s-1ADWAOE\s+1\fP.
.IP
.in -\n(PIu
2.
Query-Config response arrives
matching the pending
\f(CW\s-1Acmd\s+1\fP;
drive state is
\f(CW\s-1ADWAOE\s+1\fP
.in
Store the
\f2maxbuf\fP
value reported in the Query-Config response
(limited by the maximum configured for this drive),
and the MAC address whence it came.
Call
\f2acinit\fP
to top up this drive's
\f(CW\s-1Acmd\s+1\fP
pool to
\f2maxbuf\fP
entries.
If the maximum data-segment size
wasn't specified as a configuration property,
extract that in the response message,
interpreting zero as the default value 1024;
compare it with the value configured for the channel,
returned by
\f2aoecomm_maxdata\fP;
store the lesser in
\f2ep\fP.
Compose and send an ATA IDENTIFY command.
Set state to
\f(CW\s-1ADWATA\s+1\fP.
.IP
.in -\n(PIu
3.
ATA response message arrives;
drive state is
\f(CW\s-1ADWATA\s+1\fP
.in
Extract size and geometry information
and device-type strings
from the presumed IDENTIFY response.
If the disk is
VTOC-proper,
initialize the
\f(CW\s-1dk_geom\s+1\fP
structure in
\f2ep\fP
and call
\f2fudgegeom\fP
to tweak the numbers if necessary;
if
VTOC-improper,
invent a cylinder size
such that the largest possible cylinder number
(given the size of this drive)
is bounded by
\f(CW\s-1ULONG_MAX\s+1\fP,
for
\f2disksort\fP(9F).
Set up whole-disk partition limits.
Fill in the configuration
\f(CW\s-1kstat_t\s+1\fP
structure,
now that dynamically-determined values
like
\f(CW\s-1maxdata\s+1\fP
and
\f(CW\s-1mibsize\s+1\fP
are known.
Set state to
\f(CW\s-1ADGATA\s+1\fP.
Awaken any processes sleeping in
\f2openwait\fP
for this drive.
.IP
.in -\n(PIu
4.
Drive state
\f(CW\s-1ADGATA\s+1\fP
discovered within
\f2openwait\fP
.in
Compose a device ID object
(\f2ddi_devid_register\fP(9F))
and several string properties;
the former may be used by some Solaris programs,
the latter are an invention and may disappear
in the future.
Set state to
\f(CW\s-1ADWPTAB\s+1\fP.
Call
\f2readptab\fP
to read the disk label,
filling in all remaining partition entries,
the label type,
and possibly
other label-specific data
in the
\f(CW\s-1Adrive\s+1\fP.
When all is well,
set state to
\f(CW\s-1ADREADY\s+1\fP.
.LP
If any step fails,
initialization is completely abandoned:
the drive state is reset to
\f(CW\s-1ADCLOSED\s+1\fP,
the
\f(CW\s-1Acmd\s+1\fP
pool is freed.
The next
\f2open\fP
attempt will start over.
.PP
Locks are used to assure that at most one process at a time
is working on initializing a given drive.
Other processes desiring to access the drive
sleep until initialization succeeds
or is abandoned due to an error.
.SH
7.3.5.3.
Close device partition
.PP
Close routine
\f2adclose\fP
works as follows:
.IP \(bu
If a block or character close,
clear the open-subdevices bit for the appropriate partition and type;
if a layered-open close,
decrement the counter.
.IP \(bu
If at least one partition for this drive remains open
or there is at least one outstanding layered open,
return success now.
.IP \(bu
Set the drive-is-closing flag.
Loop,
sleeping if necessary on the
\f(CW\s-1Adrive\s+1\fP's
condition variable,
until both the active- and
pending-\f(CW\s-1buf\s+1\fP
lists have drained
and the active-command count is zero.
.IP \(bu
Call
\f2acclear\fP
to free the
\f(CW\s-1Acmd\s+1\fP
pool.
Clear the drive-is-closing flag.
Set drive state to
\f(CW\s-1ADCLOSED\s+1\fP.
.SH
7.3.5.4.
Detach device instance
.PP
Detach routine
\f2addetach\fP
works as follows.
.IP \(bu
Discard all minor devices for all device instances
(\f2ddi_remove_minor_node\fP(9F)
with second argument
\f(CW\s-1NULL\s+1\fP).
.IP \(bu
If the list of active
\f(CW\s-1buf\s+1\fP
structs for this drive
is not empty,
log an error message
and return failure.
(This shouldn't happen.)
.IP \(bu
Discard any remaining
\f(CW\s-1Acmd\s+1\fPs
for this drive;
clear out the
\f(CW\s-1Acmd\s+1\fP
pool.
.IP \(bu
Free any
\f(CW\s-1kstat_t\s+1\fP
structures associated with this drive,
including those for all partitions.
.IP \(bu
Destroy the
\f(CW\s-1kmutex_t\s+1\fP
locks and the
\f(CW\s-1kcondvar_t\s+1\fP
condition variable.
.IP \(bu
Free the
\f(CW\s-1Adrive\s+1\fP.
.PP
The system calls
\f2addetach\fP
only after all references to this device instance
have been closed;
\f2adclose\fP
blocks until no pending I/O operations remain.
Hence
\f2adddetach\fP
need show no mercy to any I/O
still unfinished.
.SH
7.3.5.5.
Read or write device
.PP
Read routine
\f2adread\fP
and write routine
\f2adwrite\fP,
called only for the character (raw) device,
just call
\f2physio\fP(9F)
which allocates a
\f(CW\s-1buf\s+1\fP
structure and calls
\f2adstrategy\fP.
Block-device I/O,
whether through the file system
or by direct
\f2read\fP
or
\f2write\fP
call,
is handled by the system;
the device driver sees only a call to
\f2adstrategy\fP
for each block.
.PP
All I/O transfers
are therefore queued by strategy routine
\f2adstrategy\fP,
which works as follows:
.IP \(bu
If the drive-is-closing flag is set
for this drive,
abort the transfer with
\f(CW\s-1ENXIO\s+1\fP.
.IP \(bu
Check that this is a reasonable transfer;
in particular that the length is
a multiple of
\f(CW\s-1DEV_BSIZE\s+1\fP
(512).
If not,
abort the transfer with
\f(CW\s-1ENXIO\s+1\fP.
.IP \(bu
If the drive state is
<\0\f(CW\s-1ADGATA\s+1\fP,
call
\f2openwait\fP
to wait until I/O is possible;
evidently a non-blocking open is in effect.
If
\f2openwait\fP
reports an error,
abort the transfer
with that error code.
(Why
\f(CW\s-1ADGATA\s+1\fP
rather than
\f(CW\s-1ADREADY\s+1\fP?
So the normal I/O path may be used
to read the disk label.)
.IP \(bu
Check that the requested transfer is within
the current partition:
.RS
.IP \(ci
If it begins after the end of the partition,
abort with
\f(CW\s-1ENXIO\s+1\fP.
.IP \(ci
If it begins exactly at the end of the partition,
set
\f2bp\fP\f(CW\s-1->b_resid\s+1\fP
set to
\f2bp\fP\f(CW\s-1->b_bcount\s+1\fP
(so
\f2read\fP
will return zero),
call
\f2biodone\fP(9F),
and return.
.IP \(ci
If it spans the end,
shorten
\f2bp\fP\f(CW\s-1->b_bcount\s+1\fP
to fit.
.RE
.IP \(bu
Allocate an
\f(CW\s-1Abuf\s+1\fP
for this
\f(CW\s-1buf\s+1\fP
structure;
store its address in
\f2bp\fP\f(CW\s-1->b_private\s+1\fP.
Initialize the
count and absolute starting block number.
.IP \(bu
Store the cylinder number
in which this transfer begins
in
\f2bp\fP\f(CW\s-1->b_resid\s+1\fP;
call
\f2disksort\fP(9F)
to insert this transfer in the drive's queue of pending requests.
.IP \(bu
Call
\f2adstart\fP
to start I/O if possible.
.SH
7.3.5.6.
Process
\f2ioctl\fP
.PP
\f2Ioctl\fP
routine
\f2adioctl\fP
calls
\f2openwait\fP
to check that initialization is complete
(in case this was a non-blocking open),
then switches on the
\f2ioctl\fP
command code.
.PP
The data-model rules for
\f2ioctl\fP
in Solaris
complicate matters.
When a 32-bit program
makes
\f2ioctl\fP
calls under a 64-bit kernel,
conversions may be necessary,
depending on the data used by the particular
\f2ioctl\fP
command.
Sometimes the 32- and 64-bit forms of a data structure
have the same size and layout;
sometimes they differ,
and the appropriate version must be filled in and returned.
In one case they differ
but Sun doesn't ship a header file declaring the 32-bit form
even though it is needed for
\f2format\fP(1M).
See Writing Device Drivers for more about this mess.
.PP
In another case,
handled by
\f2efiioc\fP,
the argument format differs
according to the Solaris version.
This is why
\f2adattach\fP
checks the version and sets a flag.
.PP
The following standard disk-device commands are supported.
Many,
but not all,
are listed in the Writing Device Drivers book
and described in somewhat more detail
in
\f2dkio\fP(7D).
.IP
.in -\n(PIu
\f(CW\s-1DKIOCINFO\s+1\fP
.in
Fetch information about the disk controller.
Returns constant controller type
\f(CW\s-1DKC_DIRECT\s+1\fP,
with a constant large controller number
(in the hope that it won't conflict with a real one),
the instance number as the disk unit number,
slave number zero.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGAPART\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSAPART\s+1\fP
.in
Fetch or set the starting block number and length
of all eight standard Solaris partitions
on this drive.
Only the in-core
\f(CW\s-1Adrive\s+1\fP
is written,
not the disk label.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCPARTINFO\s+1\fP
.in
Fetch the starting block number and length
of the partition
open on this file descriptor.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGGEOM\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCG_PHYGEOM\s+1\fP
.in
Return the drive's current
\f(CW\s-1dk_geom\s+1\fP
structure.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCSGEOM\s+1\fP
.IP
.in -\n(PIu
Update the
\f(CW\s-1dk_geom\s+1\fP
structure in
\f2ep\fP,
without changing the on-disk label.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGMEDIAINFO\s+1\fP
.in
Return a
\f(CW\s-1dk_minfo\s+1\fP
structure containing
the media type
(always
\f(CW\s-1DK_FIXED_DISK\s+1\fP),
sector size
(always
\f(CW\s-1DEV_BSIZE\s+1\fP),
and total sector count
of the disk.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCREMOVABLE\s+1\fP
.in
Is this a removable-media device?
Always returns `no.'
.IP
.in -\n(PIu
\f(CW\s-1DKIOCSETEFI\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSETEFI\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCPARTITION\s+1\fP
.in
EFI-label-specific calls
handled by
\f2efiioc\fP.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGVTOC\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSVTOC\s+1\fP
.in
VTOC-label-specific calls
handled by
\f2vtocioc\fP.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGMBOOT\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSMBOOT\s+1\fP
.in
DOS-label-specific calls
handled by
\f2dosioc\fP.
.IP
.in -\n(PIu
\f(CW\s-1DIOCTL_RWCMD\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DIOCTL_GETMODEL\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DIOCTL_GETSERIAL\s+1\fP
.in
ATA-specific calls
handled by
\f2dadkioctl\fP.
.IP
.in -\n(PIu
\f(CW\s-1USCSICMD\s+1\fP
.in
SCSI-specific (sic) call
handled by
\f2aduscsi\fP.
.SH
7.3.5.7.
Send AoE command
.PP
To send an AoE command
from within the driver:
.IP \(bu
Call
\f2acalloc\fP
to allocate an
\f(CW\s-1Acmd\s+1\fP
from the pool for this drive.
If none is available,
no more commands may be sent for now.
\f2Acalloc\fP
initializes the
\f(CW\s-1Adrive\s+1\fP
pointer,
header-length field,
and the common part of the AoE header
(AoE command,
\f2aoemaj\fP
and
\f2aoemin\fP,
version code and other flags,
unique tag value),
and zeroes all other fields.
.IP \(bu
Fill in the expiry interval and maximum retry count.
Fill in the remainder of the AoE command,
e.g. ATA register values for an ATA command,
but not data sectors to be written.
If this is a transfer,
set the
\f(CW\s-1buf\s+1\fP
pointer,
the starting block number,
and the transfer length and offset.
.IP \(bu
Call
\f2adstcmd\fP
to send the command
and place the
\f(CW\s-1Acmd\s+1\fP
on the pending-command list
for this
\f2aoechan\fP.
.PP
\f2Chantimer\fP
is called at regular intervals
to discover commands
that must be retransmitted
or expired.
.SH
7.3.5.8.
AoE message arrives
.PP
AoE messages arrive by calls to
\f2adreceive\fP,
which calls
\f2adlookup\fP
to locate the corresponding
\f(CW\s-1Acmd\s+1\fP,
then processes the message
according to its command code.
If the message is garbled
or otherwise questionable,
the
\f(CW\s-1Acmd\s+1\fP
is returned to the channel's pending-command list;
otherwise it is freed to the pool for its
\f(CW\s-1Adrive\s+1\fP.
A garbled, erroneous, or unsolicited (no
\f(CW\s-1Acmd\s+1\fP)
message is
logged
and discarded.
.SH
7.3.6.
Internal interfaces
.PP
Here is a summary
of major internal procedures,
grouped by source file.
Unless otherwise noted,
each is global within the
\f(CW\s-1aoed\s+1\fP
module,
but is not meant to be called from elsewhere in the system;
hence
\f(CW\s-1aoed.h\s+1\fP
supplies both a prototype
and a name-hiding macro,
as explained above.
.SH
7.3.6.1.
\f(CW\s-1aoed.c\s+1\fP
.KS
.RS
\f(CW\s-1static int openwait(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ap;\s+1\fP
.br
.RE
.KE
.PP
Return zero when
\f2ep\fP
has reached
state
\f(CW\s-1ADREADY\s+1\fP,
blocking if necessary;
or return a nonzero
\f2errno\fP
value if initialization fails.
.PP
If
\f2ep\fP
is in state
\f(CW\s-1ADCLOSED\s+1\fP
(initialization hasn't started yet),
call
\f2driveinit\fP
to get things started.
At each subsequent step prior to
\f(CW\s-1ADREADY\s+1\fP,
call
\f2cv_wait_sig\fP(9S)
to block
on the condition variable in
\f2ep\fP.
If
\f2cv_wait_sig\fP
is interrupted by a
UNIX\u\s-11\s+1\d\u\s-1\s+1\d
signal,
.FS
1.
UNIX is a registered trademark of the Open Group.
.FE
return
\f(CW\s-1EINTR\s+1\fP.
.PP
\f2Openwait\fP
is local to
\f(CW\s-1aoed.c\s+1\fP.
.KS
.RS
\f(CW\s-1static int readptab(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.KE
.PP
Search device
\f2ep\fP
for a valid disk label:
.IP 1.
If the drive state is
\f(CW\s-1<\0\s+1\fP\f(CW\s-1ADGATA\s+1\fP
or the disk size is not known,
return \-1.
.IP 2.
If
\f2fetchvtoc\fP
finds a proper VTOC label,
return 1,
leaving the label-type flags set to
\f(CW\s-1AVLABVTOC\s+1\fP.
.IP 3.
If
\f2fetchefi\fP
finds a proper EFI label,
return 1,
leaving the label-type flags set to
\f(CW\s-1AVLABEFI\s+1\fP.
.IP 4.
If
\f2fetchdos\fP
finds a proper DOS label:
.RS
.IP \(ci
If a DOS partition is marked as possibly
containing an encapsulated VTOC,
and this is a Solaris/x86 system,
call
\f2fetchvtoc\fP
\f(CW\s-1.\s+1\fP
If no proper VTOC label is found,
call
\f2fakevtoc\fP
to invent one.
In either case return 1,
leaving the label-type flags
set to
\f(CW\s-1AVFLABDOS|AVLABVTOC\s+1\fP.
.IP \(ci
If no DOS partition has the encapsulated-VTOC type,
or in any case on Solaris/SPARC,
return 1,
leaving the label flags set to
\f(CW\s-1AVLABDOS\s+1\fP.
.RE
.IP 5.
If
\f2fakevtoc\fP
can make a fake VTOC label,
return 0,
leaving the label-type flags set to
\f(CW\s-1AVLABVTOC\s+1\fP.
.IP 6.
Otherwise return 1,
with no label-type flags set.
.PP
\f2Readptab\fP
is local to
\f(CW\s-1aoed.c\s+1\fP.
.KS
.RS
\f(CW\s-1void fudgegeom(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.KE
.PP
Invent consistent geometry values for drive
\f2ep\fP
if necessary.
At present this means:
.IP \(bu
If
\f(CW\s-1aoed.conf\s+1\fP
supplied both
\f(CW\s-1hd\s+1\fP
and
\f(CW\s-1sec\s+1\fP
values
for this drive,
adopt them.
Compute the number of cylinders
by dividing
\f(CW\s-1hd\(**\s+1\fP\f(CW\s-1sec\s+1\fP
by the drive's total size
(number of user-accessible sectors)
as reported by ATA IDENTIFY.
.IP \(bu
If ATA IDENTIFY
returned track size,
head count,
and cylinder count values
that appear reasonable
(their product is at least the reported total size),
adopt those values.
This is very unlikely to occur.
.IP \(bu
Adopt fixed track-size and head-count values
compiled into the driver;
compute the number of cylinders as when
the values are specified in
\f(CW\s-1aoed.conf\s+1\fP.
At present these values are
\f(CW\s-1hd=64\s+1\fP
\f(CW\s-1sec=890\s+1\fP;
the resulting very-large cylinders
are required to avoid bugs in
some versions of
\f2newfs\fP(1M)
when making very large file systems.
.LP
The geometry values adopted are stored in the
\f(CW\s-1dk_geom\s+1\fP
structure in
\f2ep\fP.
.KS
.RS
\f(CW\s-1int zeroblock(ep, bno)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.KE
.PP
Write zeroes to absolute sector
\f2bno\fP
on device
\f2ep\fP;
return zero for success,
nonzero
\f2errno\fP
value on failure.
Calls
\f2adrwkern\fP
to do the real work.
.KS
.RS
\f(CW\s-1int adrwkern(ep, rw, bno, buf, len)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int rw;\s+1\fP
.br
\f(CW\s-1dev_t dev;\s+1\fP
.br
\f(CW\s-1diskaddr_t bno;\s+1\fP
.br
\f(CW\s-1void *buf;\s+1\fP
.br
\f(CW\s-1int len;\s+1\fP
.br
\f(CW\s-1\s+1\fP
.br
\f(CW\s-1int adrwuser(ep, rw, bno, buf, len, resp)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int rw;\s+1\fP
.br
\f(CW\s-1dev_t dev;\s+1\fP
.br
\f(CW\s-1diskaddr_t bno;\s+1\fP
.br
\f(CW\s-1void *buf;\s+1\fP
.br
\f(CW\s-1int len;\s+1\fP
.br
\f(CW\s-1int *resp;\s+1\fP
.br
.RE
.KE
.PP
Read
(\f2rw\fP\f(CW\s-1==B_READ\s+1\fP)
or write
(\f(CW\s-1B_WRITE\s+1\fP)
\f2len\fP
bytes to or from kernel or user address
\f2buf\fP
from or to absolute sector
\f2bno\fP
on drive
\f2ep\fP.
Return zero for success,
nonzero
\f2errno\fP
value on failure.
.PP
If the transfer finished without error
but without reading or writing the whole buffer,
\f2adrwkern\fP
returns nonstandard error value
\f(CW\s-1EXDEV\s+1\fP;
\f2adrwuser\fP
returns zero,
but if
\f2resp\fP
is nonzero,
stores the number of bytes not transferred in
\f(CW\s-1\(**\s+1\fP\f2resp\fP.
.PP
\f2Adrwkern\fP
makes a synthetic buffer header
and calls
\f2adstrategy\fP;
\f2adrwuser\fP
makes a
\f(CW\s-1struct\0\s+1\fP\f(CW\s-1uio\s+1\fP
and calls
\f2physio\fP(9F).
Both operate on the whole-disk partition.
.SH
7.3.6.2.
\f(CW\s-1addadk.c\s+1\fP
.KS
.RS
\f(CW\s-1int dadkioctl(ep, cmd, arg, mode)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int cmd;\s+1\fP
.br
\f(CW\s-1intptr_t arg;\s+1\fP
.br
\f(CW\s-1int mode;\s+1\fP
.br
.RE
.KE
.PP
Implement these undocumented
ATA-specific
\f2ioctl\fP
calls.
Some are used by
\f2format\fP(1M).
.IP
.in -\n(PIu
\f(CW\s-1DIOCTL_RWCMD\s+1\fP
.in
Read from
or write to an absolute address
on an ATA disk,
regardless of the partition table.
Uses
\f2adrwuser\fP
to do the dirty work.
\&.
.IP
.in -\n(PIu
\f(CW\s-1DIOCTL_GETMODEL\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DIOCTL_GETSERIAL\s+1\fP
.in
Return the ATA device-model
and serial-number strings,
learned from an ATA INQUIRY command
and stored in the
\f(CW\s-1Adrive\s+1\fP
during device initialization.
.LP
\f2Dadkioctl\fP
returns zero if all is well,
an
\f2errno\fP
value if an error occurred.
.SH
7.3.6.3.
\f(CW\s-1aduscsi.c\s+1\fP
.KS
.RS
\f(CW\s-1int aduscsi(ep, cmd, arg, mode)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int cmd;\s+1\fP
.br
\f(CW\s-1intptr_t arg;\s+1\fP
.br
\f(CW\s-1int mode;\s+1\fP
.br
.RE
.KE
.PP
Implement the
SCSI-specific
\f(CW\s-1USCSICMD\s+1\fP
\f2ioctl\fP,
faking
the SCSI
INQUIRY,
TEST UNIT READY,
READ CAPACITY,
and READ BLOCK LIMITS
commands.
In all cases this comprises
encoding and returning
various data stored in the
\f(CW\s-1Adrive\s+1\fP.
.PP
\f2Aduscsi\fP
is needed
to work around
bugs in some versions of
\f2format\fP(1M).
.PP
Those familiar with SCSI may note that the SCSI standard
doesn't define READ BLOCK LIMITS
for disk devices.
\f2Format\fP
calls it anyway.
.PP
The ATA IDENTIFY command returns
a device-model string;
SCSI INQUIRY
returns separate vendor and model strings.
The two SCSI strings together afford
less than half the maximum length of the single ATA string,
and in fact some ATA devices return strings
too long to fit in the SCSI message.
The faking code does the best it can.
.SH
7.3.6.4.
\f(CW\s-1addos.c\s+1\fP
.KS
.RS
\f(CW\s-1int fetchdos(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.KE
.PP
.PP
Examine the boot sector (sector 0) of drive
\f2ep\fP
for a valid DOS partition table.
If none is found,
return 0.
Otherwise
scan the four direct partitions
for one of type
\f(CW\s-1SYSID_SUNOS\s+1\fP
or
\f(CW\s-1SYSID_SUNOS2\s+1\fP.
If such a partition is found,
set the encapsulated-label partition
start and length to those of the partition,
and record fake geometry parameters in
\f2ep\fP.
If there is more than one such partition,
use the highest-numbered.
Whether an encapsulated-label partition
was found or not,
set the
\f(CW\s-1AVLABDOS\s+1\fP
label-type flag
and return 1.
.KS
.RS
\f(CW\s-1void zapdos(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.KE
Invalidate any existing DOS label on drive
\f2ep\fP
by zeroing the boot sector;
clear the
\f(CW\s-1AVLABDOS\s+1\fP
label-type flag.
.KS
.RS
\f(CW\s-1int dosioc(ep, cmd, arg, mode)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int cmd;\s+1\fP
.br
\f(CW\s-1intptr_t arg;\s+1\fP
.br
\f(CW\s-1int mode;\s+1\fP
.br
.RE
.KE
.PP
Handle DOS-specific disk
\f2ioctl\fP
\f2cmd\fP,
with the given
\f2arg\fP
and call
\f2mode\fP:
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGMBOOT\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSMBOOT\s+1\fP
.in
Call
\f2adrwkern\fP
to
read or write the boot sector
(where the DOS label lives)
on the disk,
to or from the buffer pointed to by
\f2arg\fP.
.LP
\f2Dosioc\fP
returns zero if all was well,
an
\f2errno\fP
value if an error occurred.
.SH
7.3.6.5.
\f(CW\s-1adefi.c\s+1\fP
.KS
.RS
\f(CW\s-1int fetchefi(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.KE
.PP
Search drive
\f2ep\fP
for a valid
EFI label,
trying the primary (sector 1)
and backup (last sector) GPT locations as necessary.
If a label is found,
store the starting sector address and length
of the first eight partitions
in the drive's partition table,
set label-type flag to
\f(CW\s-1AVLABEFI\s+1\fP,
and return 1.
If there is no label,
return 0;
if an I/O error occurred,
return \-1.
.KS
.RS
\f(CW\s-1void zapefi(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.KE
.PP
Invalidate any existing EFI label on the drive
by zeroing the primary and backup GPT sectors;
clear label-type flag
\f(CW\s-1AVLABEFI\s+1\fP.
.KS
.RS
\f(CW\s-1int efiioc(ep, cmd, arg, mode)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int cmd;\s+1\fP
.br
\f(CW\s-1intptr_t arg;\s+1\fP
.br
\f(CW\s-1int mode;\s+1\fP
.br
.RE
.KE
.PP
Handle EFI-specific disk
\f2ioctl\fP
\f2cmd\fP,
with the given
\f2arg\fP
and call
\f2mode\fP:
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGETEFI\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSETEFI\s+1\fP
.in
Read or write absolute sectors of the drive,
as specified by the user-space
\f(CW\s-1dk_efi_t\s+1\fP
structure addressed by
\f2arg\fP,
via calls to
\f2adrwuser\fP.
Take care to account for the different format
the structure had in Solaris 9.
For
\f(CW\s-1DKIOCSETEFI\s+1\fP
only,
call
\f2zaplabels\fP
with flag argument
\f(CW\s-1AVLABEFI\s+1\fP,
then set that flag
in the label-type flags of
\f2ep\fP.
The Solaris
\f2libefi\fP(3LIB)
library uses these calls;
the real label processing is done by the library,
not the kernel.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCPARTITION\s+1\fP
.in
Read the EFI partition descriptor
for the partition
specified in the
\f(CW\s-1partition64\s+1\fP
structure
addressed by
\f2arg\fP
directly from the disk
(not from the driver's in-core partition table).
Copy the partition's starting sector,
length,
and UUID
to the
\f(CW\s-1partition64\s+1\fP
and copy it back to the user.
.LP
\f2Efiioc\fP
returns zero if all was well,
an
\f2errno\fP
value if an error occurred.
.SH
7.3.6.6.
\f(CW\s-1advtoc.c\s+1\fP
.KS
.RS
\f(CW\s-1int fetchvtoc(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.KE
.PP
Search the
Solaris-labelled part of the disk
for a valid
VTOC label,
checking the primary location
(relative sector 0)
and the backups
(sectors 1, 3, 5, 7, and 9
in the last track)
as necessary.
If a valid label is found,
use its contents to fill in the drive's
\f(CW\s-1vtoc\s+1\fP
and
\f(CW\s-1dk_geom\s+1\fP
structures and its partition table;
set the drive's label type to
\f(CW\s-1LABVTOC\s+1\fP;
and return 1.
If no label is found
or an I/O error occurs,
return \-1.
.KS
.RS
\f(CW\s-1void zapvtoc(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.KE
Invalidate any existing VTOC label on the drive
by zeroing the primary sector and all backups.
.KS
.RS
\f(CW\s-1int fakevtoc(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.KE
If the drive is too large
to accommodate a VTOC label,
return 0.
Otherwise
synthesize one using whatever geometry
and disk-model information is at hand;
invent a partition table in which
partitions 0 and 2
(\f(CW\s-1a\s+1\fP and \f(CW\s-1c\s+1\fP)
map all of the disk but the reserved cylinders
(usually the last two);
set the label type to
\f(CW\s-1LABVTOC\s+1\fP;
and return 1.
.KS
.RS
\f(CW\s-1int vtocioc(ep, cmd, arg, mode)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int cmd;\s+1\fP
.br
\f(CW\s-1intptr_t arg;\s+1\fP
.br
\f(CW\s-1int mode;\s+1\fP
.br
.RE
.KE
.PP
If the drive's label type is not
\f(CW\s-1LABVTOC\s+1\fP,
return error
\f(CW\s-1ENOTSUP\s+1\fP.
Otherwise
handle VTOC-specific
\f2ioctl\fP
\f2cmd\fP,
with the given
\f2arg\fP
and call
\f2mode\fP:
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGVTOC\s+1\fP
.in
Return the
\f(CW\s-1vtoc\s+1\fP
structure stored in
\f2ep\fP.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCSVTOC\s+1\fP
.in
Update the drive's partition table
from the
\f(CW\s-1vtoc\s+1\fP
structure addressed by
\f2arg\fP;
copy that
\f(CW\s-1vtoc\s+1\fP
to
\f2ep\fP;
then,
using that
\f(CW\s-1vtoc\s+1\fP
and the drive's current
\f(CW\s-1dk_geom\s+1\fP,
update the primary copy and all backups
of the on-disk VTOC label.
Call
\f2zaplabels\fP
with argument type
\f(CW\s-1AVLABVTOC\s+1\fP.
.LP
\f2Vtocioc\fP
returns zero if all was well,
an
\f2errno\fP
value if an error occurred.
.SH
7.3.6.7.
\f(CW\s-1adio.c\s+1\fP
.KS
.RS
\f(CW\s-1int driveinit(ep, ndelay)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int ndelay;\s+1\fP
.br
.RE
.KE
.PP
Start initialization for drive
\f2ep\fP:
.IP 1.
If
\f2ndelay\fP
is nonzero
(a non-blocking
\f2open\fP
is in progress),
reset the MAC address for this drive
to the global broadcast address
and return success.
The real work will be done
when
\f2driveinit\fP
is called
by a subsequent
\f2openwait\fP
call.
.IP 2.
Confirm that the target is
enabled,
and that the target channel
is active.
.IP 3.
Register
\f2adreceive\fP
as the receiver for the target channel.
.IP 4.
Start
\f2chantimer\fP
for this channel if necessary.
.IP 5.
Call
\f2acalloc\fP
to allocate a single
\f(CW\s-1Acmd\s+1\fP,
and immediately use it to send
an AoE Query-Config command.
.PP
Initially
\f2ep\fP
should be in state
\f(CW\s-1ADCLOSED\s+1\fP.
If all is well,
return 1
with
\f2ep\fP
in state
\f(CW\s-1ADWAOE\s+1\fP;
if an error occurs,
return \-1
with the state unchanged.
.KS
.RS
\f(CW\s-1static void chantimer(acp)\s+1\fP
.br
\f(CW\s-1void *acp;\s+1\fP
.br
.RE
.KE
.PP
\f2Acp\fP
really points to an
\f(CW\s-1Achan\s+1\fP,
but the argument type must be
\f(CW\s-1void\0\s+1\fP\f(CW\s-1*\s+1\fP
to satisfy the rules of
\f2timeout\fP(9F).
.PP
Examine each
\f(CW\s-1Acmd\s+1\fP
awaiting a response from the
\f2aoechan\fP
associated with
\f2ap\fP.
If an
\f(CW\s-1Acmd\s+1\fP
has expired
but another retry is allowed,
retransmit it.
If no more retries are allowed,
cancel the command:
.IP \(bu
If there is an associated
\f(CW\s-1buf\s+1\fP
structure,
pass it to
\f2biodone\fP(9F)
with error
\f(CW\s-1ETIME\s+1\fP,
and
call
\f2adkillbuf\fP
to free any other
\f(CW\s-1Acmd\s+1\fPs
associated with the same
\f(CW\s-1buf\s+1\fP.
.IP \(bu
If there is an associated
condition variable,
call
\f2cv_broadcast\fP(9F)
to awaken processes waiting on it.
.LP
In any case
call
\f2acfree\fP
to return the
\f(CW\s-1Acmd\s+1\fP
to the pool for the corresponding
\f(CW\s-1Adrive\s+1\fP.
.PP
A copy of
\f2chantimer\fP
normally runs every
\f(CW\s-1TMO_CLOCK\s+1\fP
ticks
(20 ms)
for each AoE channel that has been used at least once
by the
\f(CW\s-1aoed\s+1\fP
module.
If a command was retransmitted,
\f2chantimer\fP
quits,
and the next run begins in
\f(CW\s-1TMO_CLOCKHOLD\s+1\fP
ticks
(10 ms),
to avoid flooding a congested network
with retries.
.PP
\f2Chantimer\fP
is local to
\f(CW\s-1adio.c\s+1\fP,
though it is called from outside the module
via
\f2timeout\fP.
.KS
.RS
\f(CW\s-1static void adreceive(chan, mp)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1mblk_t *mp;\s+1\fP
.br
.RE
.KE
.PP
AoE message
\f2mp\fP
has arrived via channel
\f2chan\fP:
.IP \(bu
If
\f2mp\fP
is
\f(CW\s-1NULL\s+1\fP,
the channel is shutting down.
Locate the corresponding
\f(CW\s-1Achan\s+1\fP,
clear the channel-is-active flag,
and return.
.IP \(bu
If the message is obviously malformed
(too short,
invalid version,
not an AoE response),
call
\f2aoecomm_log\fP
to log it
with code
\f(CW\s-1ACLILLFORM\s+1\fP;
return.
.IP \(bu
Call
\f2adlookup\fP
to locate the
\f(CW\s-1Acmd\s+1\fP
for which this is a response,
and to remove it from the active-command list.
If none is found,
log it
with code
\f(CW\s-1ACLUNSOL\s+1\fP
and return.
.IP \(bu
If the message is a well-formed Query-Config response
and the drive state is
\f(CW\s-1ADWAOE\s+1\fP,
copy the MAC address and
\f2maxbuf\fP
value to the
\f(CW\s-1Adrive\s+1\fP;
adjust the
maximum data-segment size if required;
call
\f2acfree\fP
to free the
\f(CW\s-1Acmd\s+1\fP;
and move to the
next initialization step.
.IP \(bu
If the message is a well-formed ATA-command response:
.RS
.IP \(ci
In drive state
\f(CW\s-1ADWATA\s+1\fP,
this is the response to an ATA IDENTIFY command:
store data;
free the
\f(CW\s-1Acmd\s+1\fP;
and
move to the
next initialization step.
.IP \(ci
In drive state
\f(CW\s-1ADGATA\s+1\fP,
\f(CW\s-1ADWPTAB\s+1\fP,
or
\f(CW\s-1ADREADY\s+1\fP,
this is the response to an ATA READ or WRITE request.
Locate the corresponding
\f(CW\s-1buf\s+1\fP
structure;
free the
\f(CW\s-1Acmd\s+1\fP.
Store read data if any;
if the transfer is now complete
or an error occurred,
free the
\f(CW\s-1Abuf\s+1\fP
and call
\f2biodone\fP(9F).
If the device-is-closing flag is set
and both the active-transfer list
and the pending-transfer queue
are empty,
signal the condition variable
to awaken
\f2adclose\fP.
In any case call
\f2adstart\fP
to start more I/O if any remains pending,
whether for this transfer or another another.
.RE
.IP \(bu
If the message is ill-formed
or the drive in an unexpected state,
log the message
with code
\f(CW\s-1ACLILLFORM\s+1\fP
or
\f(CW\s-1ACLUNSOL\s+1\fP,
and return the
\f(CW\s-1Acmd\s+1\fP
to the active-command list.
.PP
\f2Adreceive\fP
is local to
\f(CW\s-1adio.c\s+1\fP,
but is usually registered with
\f(CW\s-1aoecomm\s+1\fP
as the receiver
for one or more channels.
.RS
\f(CW\s-1void adstart(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.PP
Start I/O transfers
on drive
\f2ep\fP,
looping until no
further
\f(CW\s-1Acmd\s+1\fP
is available
(\f2acalloc\fP
returns
\f(CW\s-1NULL\s+1\fP)
or no
\f(CW\s-1buf\s+1\fP
structure remains queued.
.PP
Depending on the transfer length
and the maximum data-segment size for this drive,
several commands may be required
for a single
\f(CW\s-1buf\s+1\fP
structure.
\f2Adstart\fP
starts as much as it can.
If it was forced to stop
(ran out of
\f(CW\s-1Acmd\s+1\fPs)
partway through a
\f(CW\s-1buf\s+1\fP,
the next call to
\f2adstart\fP
continues with that buffer
before starting another.
.RS
\f(CW\s-1int adstcmd(dp)\s+1\fP
.br
\f(CW\s-1Acmd *dp;\s+1\fP
.br
.RE
.PP
Compose the AoE command
described in
\f2dp\fP
in a new STREAMS buffer,
and call
\f2aoecomm_send\fP
to send it.
On success,
call
\f2adstore\fP
to add
\f2dp\fP
to the channel's active-command list
and return 1.
On error, return \-1:
unreasonable
\f2dp\fP
contents,
\f2aoecomm_send\fP
failed.
.PP
Before calling
\f2aoecomm_send\fP
\f2adstcmd\fP
locates the
\f(CW\s-1Achan\s+1\fP
for the target channel
and checks the channel-is-active flag.
If the flag is clear,
\f2aoecomm_initdriver\fP
is called to re-register
\f2adreceive\fP
as the receiver for this channel;
if
\f2aoecomm_initdriver\fP
succeeds,
the flag is then set.
This affords recovery when a channel is shut down
and then restarted:
\f2adreceive\fP
clears the flag when
\f(CW\s-1aoecomm\s+1\fP
reports the shutdown;
\f2adstcmd\fP
re-registers the receiver when next possible,
and sets the flag when it happens.
Pending messages presumably time out and are retried.
If the channel comes back within the timeout,
no data are lost;
if not,
an I/O error is reported as for any other timeout.
.PP
These fields must have been filled in in
\f2dp\fP:
.IP
.in -\n(PIu
\f2dp\fP\f(CW\s-1->ep\s+1\fP
.in
\f(CW\s-1Adrive\s+1\fP
to which the command should be sent.
Both the target address
(\f2aoechan\fP, \f2aoemaj\fP, \f2aoemin\fP)
and Ethernet MAC address
are significant;
the latter may be the broadcast address.
.IP
.in -\n(PIu
\f2dp\fP\f(CW\s-1->tag\s+1\fP
.in
Tag value for this command.
.IP
.in -\n(PIu
\f2dp\fP\f(CW\s-1->aoehd\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f2dp\fP\f(CW\s-1->aoelen\s+1\fP
.in
AoE command to be sent,
excluding data to be written.
\f(CW\s-1aoehd\s+1\fP
is preallocated,
with room for
\f(CW\s-1AOEHEADLEN\s+1\fP
(32)
bytes,
of which
\f(CW\s-1aoelen\s+1\fP
are used by this command.
.IP
.in -\n(PIu
\f2dp\fP\f(CW\s-1->exptime\s+1\fP
.in
Minimum time,
in clock ticks,
allowed
before declaring that this command has expired.
Zero means never.
.IP
.in -\n(PIu
\f2dp\fP\f(CW\s-1->maxtries\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f2dp\fP\f(CW\s-1->ntries\s+1\fP
.in
This command may be sent at most
\f(CW\s-1maxtries\s+1\fP
times,
i.e. at most
\f(CW\s-1maxtries-1\s+1\fP
retries;
it has already been sent
\f(CW\s-1ntries\s+1\fP
times.
.IP
.in -\n(PIu
\f2dp\fP\f(CW\s-1->bp\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f2dp\fP\f(CW\s-1->len\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f2dp\fP\f(CW\s-1->off\s+1\fP
.in
If
\f(CW\s-1bp\s+1\fP
is not
\f(CW\s-1NULL\s+1\fP
and
\f(CW\s-1B_READ\s+1\fP
is not set in its flags,
append
\f(CW\s-1len\s+1\fP
bytes of data
from that buffer,
starting at offset
\f(CW\s-1off\s+1\fP
within the buffer.
.PP
\f2Dp\fP\f(CW\s-1->expires\s+1\fP
is set to the current time in clock ticks
(read from
\f2ddi_get_lbolt\fP(9F))
plus the expiry interval,
with the latter lengthened a little
for retries:
.RS
\f(CW\s-1dp->expires = ddi_get_lbolt() + (dp->exptime * (1 + dp->ntries))\s+1\fP
.RE
.SH
7.3.6.8.
\f(CW\s-1adcmd.c\s+1\fP
.PP
.RS
\f(CW\s-1Acmd *adlookup(chan, p)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1char *p;\s+1\fP
.br
.RE
.PP
Extract the tag value and target-address numbers
from the AoE message pointed to by
\f2p\fP.
Search the active-command list for channel
\f2chan\fP
for an
\f(CW\s-1Acmd\s+1\fP
with a matching tag,
associated with an
\f(CW\s-1Adrive\s+1\fP
with a matching target address.
If a match is found,
remove the
\f(CW\s-1Acmd\s+1\fP
from the active-command list,
set its
state
to
\f(CW\s-1CSLOOSE\s+1\fP,
decrement the active-command count
in the
\f(CW\s-1Adrive\s+1\fP,
and return the address of the
\f(CW\s-1Acmd\s+1\fP.
If there is no match,
return
\f(CW\s-1NULL\s+1\fP.
.RS
\f(CW\s-1void adstore(chan, dp)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1Acmd *dp;\s+1\fP
.br
.RE
.PP
Add
\f2dp\fP
to the active-command list for channel
\f2chan\fP;
increment the active-command count
in the corresponding
\f(CW\s-1Adrive\s+1\fP.
The
state
of
\f2dp\fP
must be
\f(CW\s-1CSLOOSE\s+1\fP;
it is set to
\f(CW\s-1CSCHAN\s+1\fP.
.RS
\f(CW\s-1void adremove(chan, dp)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1Acmd *dp;\s+1\fP
.br
\f(CW\s-1\s+1\fP
.br
\f(CW\s-1void adcremove(cp, dp)\s+1\fP
.br
\f(CW\s-1Achan *cp;\s+1\fP
.br
\f(CW\s-1Acmd *dp;\s+1\fP
.br
.RE
.PP
If
\f2dp\fP
is on the active-command list for channel
\f2chan\fP
or
\f2cp\fP,
remove it;
set its
state
is set to
\f(CW\s-1CSLOOSE\s+1\fP;
decrement the active-command count
of the corresponding
\f(CW\s-1Adrive\s+1\fP.
.PP
\f2Cp\fP
must be locked before calling
\f2adcremove\fP.
.RS
\f(CW\s-1void adkilldrive(chan, ep)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.PP
For each
\f(CW\s-1Acmd\s+1\fP
on the active-command list for channel
\f2chan\fP
and associated with
\f2ep\fP:
remove and free
(\f2acfree\fP)
the
\f(CW\s-1Acmd;\s+1\fP
decrement the active-command counter for
\f2ep\fP.
.RS
\f(CW\s-1void adkillbuf(chan, bp)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1struct buf *bp;\s+1\fP
.br
.RE
.PP
For each
\f(CW\s-1Acmd\s+1\fP
on the active-command list for channel
\f2chan\fP
and associated with
\f2bp\fP:
remove and free
(\f2acfree\fP)
the
\f(CW\s-1Acmd;\s+1\fP
decrement the active-command counter for
the corresponding
\f2Adrive\fP.
.RS
\f(CW\s-1Acmd *acalloc(ep, aoecmd, hdlen)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int aoecmd, hdlen;\s+1\fP
.br
.RE
.PP
Allocate an
\f(CW\s-1Acmd\s+1\fP
from the pool associated with
\f2ep\fP,
and
initialize its contents as follows:
.IP \(bu
Set the
\f(CW\s-1Adrive\s+1\fP
pointer to
\f2ep\fP.
.IP \(bu
Store
\f2ep\fP\f(CW\s-1->nexttag & ~TAGOFFSET\s+1\fP
in the
\f(CW\s-1tag\s+1\fP
field of the
\f(CW\s-1Acmd\s+1\fP,
taking care to skip the value zero
(used by AoE targets
for unsolicited broadcast replies).
Increment
\f2ep\fP\f(CW\s-1->nexttag\s+1\fP
for next time.
.IP \(bu
Set the AoE command length
to
\f2hdlen\fP.
Fill in the common header:
standard protocol-version and flag values,
command type
\f2aoecmd\fP,
\f2aoemaj\fP
and
\f2aoemin\fP
from
\f2ep\fP,
tag from that just invented.
.IP \(bu
Set the
state
to
\f(CW\s-1CSLOOSE\s+1\fP.
.IP \(bu
Zero all other fields.
.LP
Return the address of the new
\f(CW\s-1Acmd\s+1\fP,
or
\f(CW\s-1NULL\s+1\fP
if none was available.
.PP
Messages with tag values greater than or equal to
\f(CW\s-1TAGOFFSET\s+1\fP
(\f(CW\s-10x80000000\s+1\fP)
are reserved for non-kernel diagnostic programs.
There are none such yet in the Solaris implementation.
Originally the sense was inverted,
with
\f(CW\s-1TAGOFFSET\s+1\fP
set in messages generated by the Solaris kernel driver,
lesser values reserved for non-kernel use;
this was changed in version 1.3.3
for consistency with the Linux AoE implementation.
.RS
\f(CW\s-1void acfree(ep, dp)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1Acmd *dp;\s+1\fP
.br
.RE
.PP
Return
\f2dp\fP
to the pool associated with
\f2ep\fP,
setting its state to
\f(CW\s-1CSFREE\s+1\fP.
.RS
\f(CW\s-1void acinit(ep, nfree)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int nfree;\s+1\fP
.br
.RE
.PP
Allocate new
\f(CW\s-1Acmd\s+1\fPs
from system memory,
initialize them with
state
\f(CW\s-1CSFREE\s+1\fP,
and add them to the pool associated with
\f2ep\fP
until the pool contains at least
\f2nfree\fP
entries
or the system runs out of memory.
.PP
If the pool is not empty,
but has fewer than
\f2nfree\fP
entries,
it is topped up to
\f2nfree\fP.
If it is already larger than
\f2nfree\fP
the system panics.
.RS
\f(CW\s-1void acclear(ep)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
.RE
.PP
Empty the
\f(CW\s-1Acmd\s+1\fP
pool associated with
\f2ep\fP,
returning memory to the system.
.SH
7.3.6.9.
\f(CW\s-1adsubr.c\s+1\fP
.RS
\f(CW\s-1Adrive *devtodrive(dev, s)\s+1\fP
.br
\f(CW\s-1dev_t dev;\s+1\fP
.br
\f(CW\s-1char *s;\s+1\fP
.br
.RE
.PP
Return a pointer to the
\f(CW\s-1Adrive\s+1\fP
corresponding to Solaris device
\f2dev\fP.
If none exists,
log an error message
(including
\f2s\fP
if
non-\f(CW\s-1NULL\s+1\fP)
and return
\f(CW\s-1NULL\s+1\fP.
.KS
.RS
\f(CW\s-1char *aoeerrstr(aoecode, buf, len)\s+1\fP
.br
\f(CW\s-1int aoecode;\s+1\fP
.br
\f(CW\s-1char *buf;\s+1\fP
.br
\f(CW\s-1int len;\s+1\fP
.br
\f(CW\s-1\s+1\fP
.br
\f(CW\s-1char *ataerrstr(atacode, buf, len)\s+1\fP
.br
\f(CW\s-1int atacode;\s+1\fP
.br
\f(CW\s-1char *buf;\s+1\fP
.br
\f(CW\s-1int len;\s+1\fP
.br
.RE
.KE
.PP
\f2Aoeerrstr\fP
returns a string encoding
\f2aoecode\fP
as a decimal number,
with a string explaining its meaning as an AoE error code
if one is known.
.PP
\f2Ataerrstr\fP
returns a string encoding
\f2atacode\fP
as an eight-bit hexadecimal number,
with a string containing the standard abbreviation
describing each bit that is set.
.PP
Either routine stores the string in
\f2buf\fP,
truncating safely after
\f2len\fP
bytes if necessary.
The return value is
\f2buf\fP.
.KS
.RS
\f(CW\s-1char *admsg(dev, ep, fmt, ...)\s+1\fP
.br
\f(CW\s-1dev_t dev;\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1char *fmt;\s+1\fP
.br
.RE
.KE
.PP
Write the message given by
\f2printf\fP-like
format string
\f2fmt\fP
and any following arguments
to
\f2syslog\fP
and the console,
prepending
a string of the form
.RS
\f(CW\s-1aoed\s+1\fP\f2inst\fP
\f(CW\s-1s\s+1\fP\f2part\fP
\f2chan\fP\f(CW\s-1/\s+1\fP\f2aoemaj\fP\f(CW\s-1/\s+1\fP\f2aoemin\fP
.RE
describing the device identified by
\f2dev\fP
and
\f2ep\fP.
.PP
If
\f2ep\fP
is
\f(CW\s-1NULL\s+1\fP,
the
\f(CW\s-1Adrive\s+1\fP
is determined from
\f2dev\fP
if possible.
If no
\f(CW\s-1Adrive\s+1\fP
can be found,
the target address is omitted.
If
\f2dev\fP
is
\f(CW\s-1NODEV\s+1\fP,
the partition name is omitted.
.PP
A newline is appended to the message;
\f2fmt\fP
should contain none.
.PP
The message will be truncated safely at 300 characters.
.KS
.RS
\f(CW\s-1char *adpname(part)\s+1\fP
.br
\f(CW\s-1int part;\s+1\fP
.br
.RE
.KE
.PP
Return a short string name
for partition
\f2part\fP,
in the style used for names in
\f(CW\s-1/devices\s+1\fP:
\f(CW\s-1a\s+1\fP
for
\f2part\fP
0,
\f(CW\s-1b\s+1\fP
for 1,
and so on to
\f(CW\s-1h\s+1\fP
for 7;
\f(CW\s-1wd\s+1\fP
for 8;
\f(CW\s-1???\s+1\fP
for anything else.
.KS
.RS
\f(CW\s-1void zaplabels(ep, f)\s+1\fP
.br
\f(CW\s-1Adrive *ep;\s+1\fP
.br
\f(CW\s-1int f;\s+1\fP
.br
.RE
.KE
Erase
from
\f2ep\fP
disk labels of all types
except those specified in label-flag value
\f2f\fP:
.IP \(bu
If
\f(CW\s-1(\s+1\fP\f2f\fP\f(CW\s-1&AVLABDOS) == 0\s+1\fP,
call
\f2zapdos\fP.
.IP \(bu
If
\f(CW\s-1(\s+1\fP\f2f\fP\f(CW\s-1&AVLABVTOC) == 0\s+1\fP,
call
\f2zapvtoc\fP.
.IP \(bu
If
\f(CW\s-1(\s+1\fP\f2f\fP\f(CW\s-1&AVLABEFI) == 0\s+1\fP,
call
\f2zapefi\fP.
.KS
.RS
\f(CW\s-1int solversion();\s+1\fP
.br
.RE
.KE
Return the integer Solaris version number
of the running system;
e.g. 10
for Solaris 10
(aka Solaris 2.10, or SunOS 5.10).
If the version cannot be determined,
return 0.
.PP
\f2Solversion\fP
works by examining global value
\f(CW\s-1utsname.release\s+1\fP.
This is admittedly a hack,
as is the very notion of this routine,
which should be used only
as a last resort.
.KS
.RS
\f(CW\s-1int putllprop(dev, dip, name, val)\s+1\fP
.br
\f(CW\s-1dev_t dev;\s+1\fP
.br
\f(CW\s-1dev_info_t *dip;\s+1\fP
.br
\f(CW\s-1char *name;\s+1\fP
.br
\f(CW\s-1uint64_t val;\s+1\fP
.br
.RE
.KE
Create or update a device property with a 64-bit-integer value.
Solaris library routine
\f2ddi_prop_update_int64\fP
(\f2ddi_prop_update\fP(9F))
does this,
but is present only in Solaris 9 and later versions;
if
\f(CW\s-1aoed\s+1\fP
called it directly
the module wouldn't load on Solaris 7 or 8.
\f2putllprop\fP
does the work through undocumented
but apparently-stable interfaces that exist
even in older systems.
.PP
The
\f(CW\s-1Nblocks\s+1\fP
and
\f(CW\s-1Size\s+1\fP
properties,
at least one of which is required by ZFS,
have 64-bit integer values.
.SH
7.3.6.10.
\f(CW\s-1adtrace.c\s+1\fP
.RS
\f(CW\s-1void trace(id, p0, i0, i1)\s+1\fP
.br
\f(CW\s-1int id;\s+1\fP
.br
\f(CW\s-1void *p0;\s+1\fP
.br
\f(CW\s-1int i0, i1;\s+1\fP
.br
.RE
.PP
Store the arguments
and a sequence number
in the next slot
in a circular buffer of
1024 trace records.
There is sufficient locking
to prevent concurrent calls from using the same slot.
.PP
\f2Trace\fP
is meant for collecting real-time event traces
during debugging.
.PP
No tools are supplied to read the trace buffer;
use
\f2adb\fP
or
\f2mdb\fP(1).
.PP
A
\f(CW\s-1kmutex_t\s+1\fP
lock is initialized when
\f2trace\fP
is first called,
but never destroyed.
Hence if
\f2trace\fP
is used and the
\f(CW\s-1aoed\s+1\fP
module is reloaded,
the system loses a little memory
until the next boot.
Since
\f2trace\fP
is used mostly to gather information
just before a reproducible crash or hang,
this is unlikely to cause trouble in practice.
.SH
8.
User-mode component details
.PP
This is a summary of function and implementation;
see the manual pages for details of usage.
.PP
Each program has a single source file,
but
C programs use the
library
described in its own section below.
Most C programs
also use some of the
network data structure include files
from the
\f(CW\s-1include\s+1\fP
directory.
.SH
8.1.
\f2Aoestart\fP
.PP
\f2Aoestart\fP
starts an AoE channel
according to its arguments.
A network device and channel number
must be given.
Optional parameters include Ethernet protocol type
and maximum data-segment size,
with defaults taken from the AoE protocol spec
and derived from the device's MTU setting.
.PP
\f2Aoestart\fP
calls
\f2comminit\fP
to open the Ethernet device,
configure it,
and perform the
channel startup handshake
with the
\f(CW\s-1aoecomm\s+1\fP
and
\f(CW\s-1aoectl\s+1\fP
kernel modules.
If all is well,
it calls
\f2achattach\fP
to keep the channel open.
By default it then writes an
\f(CW\s-1ACSEND\s+1\fP
command to
\f(CW\s-1/dev/aoectl\s+1\fP
to broadcast a Query-Config command,
in the hope that every AoE target will respond,
and that
\f2aoemon\fP
will hear the responses and enable all the devices.
.PP
The source code is file
\f(CW\s-1aoestart.c\s+1\fP.
.SH
8.2.
\f2Aoestop\fP
.PP
\f2Aoestop\fP
calls
\f2achdetach\fP
to close the AoE channels named by its arguments.
.PP
The source code is file
\f(CW\s-1aoestop.c\s+1\fP.
.SH
8.3.
\f2Aoectl\fP
.PP
\f2Aoectl\fP
opens
\f(CW\s-1/dev/aoectl\s+1\fP
and writes commands
according to its arguments:
.IP
.in -\n(PIu
\f(CW\s-1probe\s+1\fP
.in
Compose an AoE Query-Config command
for each target named;
use
\f(CW\s-1ACSEND\s+1\fP
to send to the broadcast address
on the channel named.
.IP
.in -\n(PIu
\f(CW\s-1list\s+1\fP
.in
Send one or more
\f(CW\s-1ACDEVENAB\s+1\fP
commands with subcommand
\f(CW\s-1ADQUERY\s+1\fP.
If a target has wildcards,
first let the system discover whether any matching device
is enabled;
if so,
iterate over possible values to find out what they are.
On older hardware this takes a few seconds.
.IP
.in -\n(PIu
\f(CW\s-1enable\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1disable\s+1\fP
.in
Send one or more
\f(CW\s-1ACDEVENAB\s+1\fP
commands with subcommand
\f(CW\s-1ADENAB\s+1\fP
or
\f(CW\s-1ADDISAB\s+1\fP.
Wildcards are handled by the system.
.PP
The source code is file
\f(CW\s-1aoectl.c\s+1\fP.
.SH
8.4.
\f2Aoemon\fP
.PP
\f2Aoemon\fP
opens
\f(CW\s-1/dev/aoemon\s+1\fP
and loops forever
reading it.
An
\f(CW\s-1ACLOG\s+1\fP
message of type
\f(CW\s-1ACLUNSOL\s+1\fP
containing an AoE Query-Config response
causes the responding target device to be enabled,
and the action reported with
\f2logmsg\fP.
Any other message is just logged with
\f2logpkt\fP.
.PP
The source code is file
\f(CW\s-1aoemon.c\s+1\fP.
.SH
8.5.
Library routines
.PP
Common code used by more than one user-mode component
or isolating some implementation dependency
is compiled into object library
\f(CW\s-1libcmd.a\s+1\fP,
used with all the programs listed above.
Header file
\f(CW\s-1aoecmd.h\s+1\fP
(in the
\f(CW\s-1user\s+1\fP
directory,
since no kernel component needs it)
declares prototypes for all library routines,
as well as a few parameter values.
.PP
Here is a list of library routines,
organized by source file.
.SH
8.5.1.
\f(CW\s-1comm.c\s+1\fP
.PP
Communication-channel startup.
Used by
\f2aoestart\fP.
.KS
.RS
\f(CW\s-1int comminit(chan, name, proto, maxdata, retaddr)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
\f(CW\s-1char *name;\s+1\fP
.br
\f(CW\s-1int proto;\s+1\fP
.br
\f(CW\s-1int maxdata;\s+1\fP
.br
\f(CW\s-1char retaddr[LEN_EADDR];\s+1\fP
.br
.RE
.KE
.PP
Open Ethernet device
\f2name\fP.
Configure it to receive AoE messages 
with Ethernet protocol type
\f2proto\fP
and maximum data-segment size
\f2maxdata\fP
(if
\f2maxdata\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1<=\0\s+1\fP\f(CW\s-10\s+1\fP,
a size computed from the device's current MTU).
Make the resulting file into AoE channel
\f2chan\fP.
Copy the Ethernet MAC address of the device
to
\f2retaddr\fP,
and return the resulting file descriptor;
or return \-1 if an error occurred.
.PP
If
\f2maxdata\fP
is zero,
fetch the device MTU,
compute the corresponding AoE maximum-data size,
and so inform
\f(CW\s-1aoecomm\s+1\fP.
.PP
Hidden inside
\f2comminit\fP
are many Solaris DLPI calls
and the
AoE startup dance.
The channel is initialized,
but not attached to its file system mount point;
see
\f2achattach\fP.
.SH
8.5.2.
\f(CW\s-1achan.c\s+1\fP
.PP
.KS
.RS
\f(CW\s-1int achattach(fd, chan)\s+1\fP
.br
\f(CW\s-1int fd, chan;\s+1\fP
.br
.RE
.KE
.PP
Arrange that file descriptor
\f2fd\fP
will remain open,
and the corresponding AoE channel active,
even if the calling program closes
\f2fd\fP
or exists.
Remember that
\f2fd\fP
it is associated with channel
\f2chan\fP
so the arrangements may be undone by
\f2achdetach\fP.
.PP
Return > 0 for success,
< 0 for failure
or if
\f2chan\fP
has an unreasonable value.
.KS
.RS
\f(CW\s-1int achdetach(chan)\s+1\fP
.br
\f(CW\s-1int chan;\s+1\fP
.br
.RE
.KE
.PP
Undo what
\f2achattach\fP
did,
closing the file descriptor and shutting down AoE channel
\f2chan\fP.
If
\f2chan\fP
is negative,
do this for every active channel.
.PP
Return > 0 if this was done;
zero if there is no file descriptor associated with
\f2chan\fP
(or none with any chan if
zero if no such channel
\f2chan\fP
is negative);
< 0 if an error occurred.
.PP
\f2Achattach\fP
calls
\f2fattach\fP(3)
to attach
\f2fd\fP
to file
\f(CW\s-1/etc/aoe/ch\s+1\fP\f2nn\fP
(\f2nn\fP
the channel number,
expressed as a two-digit decimal number),
after creating the file if necessary.
\f2Achdetach\fP
calls
\f2fdetach\fP(3)
on
\f(CW\s-1/etc/aoe/ch\s+1\fP\f2nn\fP.
.SH
8.5.3.
\f(CW\s-1trdwr.c\s+1\fP
.KS
.RS
\f(CW\s-1int tread(fd, buf, len, ms)\s+1\fP
.br
\f(CW\s-1int fd;\s+1\fP
.br
\f(CW\s-1void *buf;\s+1\fP
.br
\f(CW\s-1int len, ms;\s+1\fP
.br
\f(CW\s-1\s+1\fP
.br
\f(CW\s-1int twrite(fd, buf, len, ms)\s+1\fP
.br
\f(CW\s-1int fd;\s+1\fP
.br
\f(CW\s-1void *buf;\s+1\fP
.br
\f(CW\s-1int len, ms;\s+1\fP
.br
.RE
.KE
.PP
Call
\f2read\fP
or
\f2write\fP,
but return \-1
with
\f2errno\fP
set to
\f(CW\s-1EINTR\s+1\fP
if the operation hasn't completed in
\f2ms\fP
milliseconds.
.SH
8.5.4.
\f(CW\s-1log.c\s+1\fP
.PP
Write to an error log,
using
\f2syslog\fP(3)
with facility
\f(CW\s-1LOG_DAEMON\s+1\fP
and a severity level specified as an abstract argument.
.PP
Why not just call
\f2syslog\fP
directly?
Partly to make it easier to fine-tune the implementation
(e.g. the best mapping from internal severity levels
to those of
\f2syslog\fP
depends on
\f(CW\s-1syslog.conf\s+1\fP
conventions);
partly to avoid stepping on the buffer-overflow
and format-string-trust problems
in some versions of
\f2syslog\fP.
.PP
.KS
.RS
\f(CW\s-1void loginit(name, tostderr)\s+1\fP
.br
\f(CW\s-1char *name;\s+1\fP
.br
\f(CW\s-1int tostderr;\s+1\fP
.br
.RE
.KE
.PP
Initialize the logging mechanism.
Should be called before any other logging call.
\f2Name\fP
is a string name to be included in messages.
If
\f2tostderr\fP
is nonzero,
messages are copied to the standard error file
as well as to the log.
.KS
.RS
\f(CW\s-1void logmsg(type, format, ...)\s+1\fP
.br
\f(CW\s-1int type;\s+1\fP
.br
\f(CW\s-1char *format;\s+1\fP
.br
.RE
.KE
.PP
Write a message to the log.
\f2Format\fP
is a format string of the sort accepted by
\f2printf\fP(3);
it may be followed by arguments.
\f2Type\fP
has one of the following values,
listed here from most important to least:
.IP
.in -\n(PIu
\f(CW\s-1LE\s+1\fP
.in
Error message:
something isn't working right.
Uses
\f2syslog\fP
severity
\f(CW\s-1LOG_ERROR\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1LN\s+1\fP
.in
Notice:
something unusual has happened,
but perhaps all is still well.
\f2Syslog\fP
severity
\f(CW\s-1LOG_WARNING\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1LI\s+1\fP
.in
Status information;
nothing is wrong.
\f2Syslog\fP
severity
\f(CW\s-1LOG_NOTICE\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1LD\s+1\fP
.in
Debugging chatter.
\f2Syslog\fP
severity
\f(CW\s-1LOG_DEBUG\s+1\fP.
.LP
Flag
\f(CW\s-1LS\s+1\fP
may be or-ed into
\f2type\fP
to request that the message be copied to standard error
regardless of the
\f2tostderr\fP
argument in
\f2loginit\fP.
.PP
The mapping between
\f2type\fP
codes and
\f2syslog\fP
severity values is not quite the obvious one
because the default
\f(CW\s-1/etc/syslog.conf\s+1\fP
file supplied with Solaris
throws away
\f(CW\s-1daemon.notice\s+1\fP
messages.
.PP
Each
\f2logmsg\fP
call produces
exactly one line of log text.
Messages accumulated in dribs and drabs
should be composed separately.
A terminating newline is neither necessary nor desired.
A message longer than 250 characters
is silently (but safely) truncated.
.KS
.RS
\f(CW\s-1void logpkt(type, buf, len)\s+1\fP
.br
\f(CW\s-1int type;\s+1\fP
.br
\f(CW\s-1char *buf;\s+1\fP
.br
\f(CW\s-1int len;\s+1\fP
.br
.RE
.KE
.PP
Log the contents of
\f2buf\fP,
interpreting it
as an Ethernet datagram
containing an AoE message.
If the message appears truncated
just go as far as possible;
if ill-formed,
interpret as far as possible,
then log the remainder with
\f2loghex\fP.
.KS
.RS
\f(CW\s-1void loghex(type, buf, len)\s+1\fP
.br
\f(CW\s-1int type;\s+1\fP
.br
\f(CW\s-1char *buf;\s+1\fP
.br
\f(CW\s-1int len;\s+1\fP
.br
.RE
.KE
.PP
Log the contents of
\f2buf\fP
as hexadecimal bytes,
no more than 16 to a line.
.KS
.RS
\f(CW\s-1char *ethertoa(addr, buf)\s+1\fP
.br
\f(CW\s-1char *addr;\s+1\fP
.br
\f(CW\s-1char *buf;\s+1\fP
.br
.RE
.KE
.PP
Return a printable string representing
the six bytes at address
\f2addr\fP
as an Ethernet MAC address.
If
\f2buf\fP
is nonzero,
put the string there;
at least
\f(CW\s-1LEN_EADDR*3\s+1\fP
bytes should be available.
If
\f2buf\fP
is
\f(CW\s-1NULL\s+1\fP,
use a static buffer.
.SH
8.5.5.
\f(CW\s-1debug.c\s+1\fP
.KS
.RS
\f(CW\s-1int verbose;\s+1\fP
.br
\f(CW\s-1int debug;\s+1\fP
.br
.RE
.KE
.PP
If
\f2verbose\fP
has a nonzero value,
some programs and library routines chatter a little
(via
\f2logmsg\fP)
as they work.
If
\f2debug\fP
is nonzero,
chatter is more copious and more detailed.
Normally these are set by command-line options.
.SH
8.6.
\f2Aoelabinit\fP
.PP
\f2Aoelabinit\fP
writes an initial disk label to one or more disks.
By default the label is VTOC format
if the disk size allows that,
EFI otherwise,
and a label is written
only if none (of either format)
already exists;
different choices may be specified.
Writing a label of one type
invalidates any existing label of the other.
.PP
This program exists only as a workaround
for a
bug in
\f2format\fP(1M),
which is unable to cope with an unlabelled ATA disk
large enough to require EFI labelling.
Probably Sun will fix this eventually,
but
\f2aoelabinit\fP
will remain
both to avoid stranding those with older systems
and because it seems like a useful tool in its own right.
.PP
\f2Aoelabinit\fP
uses the Solaris
\f2libefi\fP(3LIB)
library to read and write EFI labels.
Since this is a shared library
present only since Solaris 9 4/03,
\f2aoelabinit\fP
can be run only on newer systems.
Static linking isn't practical
because the library uses a kernel interface
that differs in Solaris 9 and Solaris 10.
.SH
8.7.
\f2Aoeunlabel\fP
.PP
\f2Aoeunlabel\fP
zeroes the sectors conventionally used
for EFI and VTOC labels and their backups.
Something of the sort must be done
before a disk with an EFI label
may be repartitioned on a pre-EFI system,
e.g. when a disk used as part of a ZFS pool
is recycled to ordinary use on a Solaris 8 system.
.PP
\f2Aoeunlabel\fP
uses none of the special Solaris label-access libraries;
it just overwrites sectors directly.
Hence it works even on a pre-EFI system,
unlike
\f2aoelabinit\fP.
.SH
8.8.
\f2Aoemkconf\fP
.PP
\f2Aoemkconf\fP
emits device-configuration entries
suitable for inclusion in
\f(CW\s-1aoed.conf\s+1\fP.
.PP
An argument of the form
\f(CW\s-10/11/9\s+1\fP
names a blade for which an entry is desired;
the
\f2aoemin\fP
part may be an inclusive range,
like
\f(CW\s-10/11/9-11\s+1\fP
as a shorthand for
\f(CW\s-10/11/9\s+1\fP
\f(CW\s-10/11/10\s+1\fP
\f(CW\s-10/11/11\s+1\fP.
Entries contain only the required properties:
\f(CW\s-1name="aoed"\s+1\fP,
\f(CW\s-1parent="pseudo"\s+1\fP,
\f(CW\s-1aoechan\s+1\fP
and
\f(CW\s-1aoemaj\s+1\fP
and
\f(CW\s-1aoemin\s+1\fP
as specified,
\f(CW\s-1instance\s+1\fP
computed in
the standard way.
.PP
Any other argument names an existing kernel-configuration file;
entries that would duplicate an instance number
already declared in the file
are omitted.
.PP
For example:
.RS
\f(CW\s-1aoemkconf\0\s+1\fP\f(CW\s-1/usr/kernel/drv/aoed.conf\0\s+1\fP\f(CW\s-10/11/0-14 0/13/0-14\s+1\fP
.RE
prints entries for two 15-slot EtherDrive shelves
at
\f2aoemaj\fP
addresses 11 and 13,
but suppresses any the system has already been told about;
.RS
\f(CW\s-1aoemkconf\0\s+1\fP\f(CW\s-1/usr/kernel/drv/aoed.conf\0\s+1\fP\f(CW\s-1`aoectl\0\s+1\fP\f(CW\s-1list`\s+1\fP
.RE
prints an entry for every device that has made itself known
on the network but is not yet configured;
and,
for those who enjoy living on the edge,
.RS
\f(CW\s-1aoemkconf\0\s+1\fP\f(CW\s-1/usr/kernel/drv/aoed.conf\0\s+1\fP\f(CW\s-1`aoectl\0\s+1\fP\f(CW\s-1list`\0\s+1\fP\f(CW\s-1\s+1\fP\f(CW\s-1\s+1\fP
.br
\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1\0\s+1\fP\f(CW\s-1>>/usr/kernel/drv/aoed.conf\s+1\fP
.br
.RE
updates the configuration file in place.
.PP
\f2Aoemkconf\fP
is an
\f2awk\fP
program.
There are no explicit hooks for customization,
but it ought to be easy to adapt it to
local ideas of instance numbers
or to supply additional device properties.
.SH
8.9.
\f(CW\s-1/etc/init.d/aoe\s+1\fP,
\f(CW\s-1/lib/svc/method/device-aoe\s+1\fP
.PP
Startup/shutdown shell script,
with the same contents under either name:
\f(CW\s-1/etc/init.d/aoe\s+1\fP
for use in the
\f2init.d\fP(4)
mechanism on a non-SMF system,
\f(CW\s-1/lib/svc/method/device-aoe\s+1\fP
for use with SMF.
.PP
The script
acts according to its first argument:
.IP
.in -\n(PIu
\f(CW\s-1aoe \s+1\fP\f(CW\s-1start\s+1\fP
.in
Start
\f2aoemon\fP
if necessary.
Massage the contents of
\f(CW\s-1/etc/aoe.conf\s+1\fP
to produce a collection of
\f2aoestart\fP
commands,
one for each channel to be started,
and execute them.
In spirit this is just
.RS
\f(CW\s-1sed </etc/aoe.conf 's;^;/sbin/aoestart ;' | sh\s+1\fP
.br
.RE
but practical details (comments, blank lines)
add a little complexity.
.IP
.in -\n(PIu
\f(CW\s-1aoe \s+1\fP\f(CW\s-1stop\s+1\fP [ \f2contract\fP ]
.in
Call
\f2aoestop\fP
to shut down all channels;
kill
\f2aoemon\fP.
If SMF is active,
\f2contract\fP
is the SMF contract number
under which
\f2aoemon\fP
was run.
.IP
.in -\n(PIu
\f(CW\s-1aoe \s+1\fP\f(CW\s-1restart\s+1\fP [ \f2contract\fP ]
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1aoe \s+1\fP\f(CW\s-1refresh\s+1\fP [ \f2contract\fP ]
.in
Equivalent to
\f(CW\s-1aoe \s+1\fP\f(CW\s-1stop; \s+1\fP\f(CW\s-1aoe \s+1\fP\f(CW\s-1start\s+1\fP
except that
\f2aoemon\fP
is not restarted.
.PP
On a non-SMF system,
\f(CW\s-1/etc/init.d/aoe\s+1\fP
is linked to
\f(CW\s-1/etc/rc2.d/S00aoe\s+1\fP
so that
\f(CW\s-1aoe\0\s+1\fP\f(CW\s-1start\s+1\fP
will be called very early in the system startup process.
There is no
\f(CW\s-1K\s+1\fP\f2xx\fP\f(CW\s-1aoe\s+1\fP
link:
there's no need
to call
\f(CW\s-1aoe\0\s+1\fP\f(CW\s-1stop\s+1\fP
during a normal shutdown.
.PP
On an SMF system,
AoE is installed as service
\f(CW\s-1svc:/device/aoe\s+1\fP,
and should be started and stopped
(enabled and disabled)
with
\f2svcadm\fP(1M).
The service manifest is
\f(CW\s-1user/aoe.xml\s+1\fP
in the source-code tree,
\f(CW\s-1/opt/CORDaoe/lib/aoe.xml\s+1\fP
and
\f(CW\s-1/var/svc/manifest/device/aoe.xml\s+1\fP
in the installed package.
The
\f(CW\s-1aoe\s+1\fP
service depends on
\f(CW\s-1svc:/filesystems/root\s+1\fP
and declares
\f(CW\s-1svc:/filesystems/usr\s+1\fP
as an optional dependent.
Thus any file system but the root or
\f(CW\s-1/usr\s+1\fP,
and any swap area,
may be placed on an AoE disk.
.PP
The choice of dependencies
derives from Solaris implementation details:
.IP \(bu
\f(CW\s-1svc:/filesystems/usr\s+1\fP
enables swap areas,
so
\f(CW\s-1svc:/device/aoe\s+1\fP
must be enabled first
to allow swapping to AoE disks.
.IP \(bu
The dependency is optional
so that a failure to start AoE
won't interfere with other startup,
except to the extent caused by
failure to mount AoE-disk file systems.
.IP \(bu
It sounds impossible for AoE to start
before
\f(CW\s-1svc:/filesystems/usr\s+1\fP,
because AoE drivers and tools are stored in
\f(CW\s-1/usr/kernel\s+1\fP
and
\f(CW\s-1/usr/sbin\s+1\fP.
In fact,
if
\f(CW\s-1/usr\s+1\fP
is a separate file system
(not so common any more)
\f(CW\s-1filesystems/root\s+1\fP
mounts
\f(CW\s-1/usr\s+1\fP
read-only,
presumably because so much of Solaris itself
is stored in
\f(CW\s-1/usr\s+1\fP.
.PP
If file
\f(CW\s-1/etc/default/aoe.options\s+1\fP
exists,
its contents are interpolated
into the
\f(CW\s-1aoe\s+1\fP
script
(with the shell's
\f(CW\s-1.\s+1\fP
operator)
before anything else is done.
Many parameters such as
the location of
\f2aoestart\fP,
\f2aoestop\fP,
and
\f2aoemon\fP,
the directory where channel files are attached
to keep them open,
and
the name of the configuration file
are set within
\f(CW\s-1aoe\s+1\fP
by shell variables;
settings in
\f(CW\s-1aoe.options\s+1\fP
override the default values.
.PP
SMF or
\f2init.d\fP
startup is selected
when the
\f(CW\s-1CORDaoe\s+1\fP
package is installed:
.IP \(bu
\f(CW\s-1/etc/init.d/aoe\s+1\fP
is installed in any case.
.IP \(bu
If the target system uses SMF,
a
\f2postinstall\fP
script:
.RS
.IP 1.
Copies
\f(CW\s-1/etc/init.d/aoe\s+1\fP
to
\f(CW\s-1/lib/svc/method/device-aoe\s+1\fP.
.IP 2.
Copies
\f(CW\s-1/opt/CORDaoe/lib/aoe.xml\s+1\fP
to
\f(CW\s-1/var/svc/manifest/device/aoe.xml\s+1\fP,
then runs
\f2svccfg\fP(1M)
to import the latter.
.IP 3.
Calls
\f(CW\s-1svccfg\0\s+1\fP\f(CW\s-1refresh\0\s+1\fP\f(CW\s-1/system/filesystem/usr\s+1\fP
as a workaround for a bug in
early versions of Solaris 10,
which sometimes didn't register
\f(CW\s-1dependent\s+1\fP
declarations when a new service was added.
.RE
.IP \(bu
If the target is a non-SMF system,
\f(CW\s-1/etc/init.d/aoe\s+1\fP
is linked to
\f(CW\s-1/etc/rc2.d/S00aoe\s+1\fP.
.SH
9.
Design details,
compromises,
bugs,
and other concessions to reality
.PP
Here is a collection of notes
about design decisions,
problems encountered with Solaris or elsewhere
and how they have been papered over (or not),
and so on.
.PP
Some of the problems described here
will, we hope,
be mended in future versions of the subsystem,
or eased by future versions of Solaris,
though the latter is no panacea
since older Solaris systems must not be abandoned lightly.
.SH
9.1.
\f2Gcc\fP
versus
Sun C
.PP
Currently all the code is compiled with
Sun's C compiler,
available at no cost
from
\f(CW\s-1www.opensolaris.org\s+1\fP.
.PP
Earlier versions of the driver were built with
\f2gcc\fP,
which proved unsatisfactory:
.IP \(bu
\f2Gcc-\fPspecific
runtime-library routines
were required to support certain 64-bit operations
on 32-bit machines.
Only certain 64-bit operations were affected,
apparently only in certain contexts.
The Solaris kernel environment doesn't offer these routines;
fetching and compiling the
\f2gcc\fP
runtime-library source code just for those
compiler-support routines
seemed prohibitively cumbersome.
As a workaround,
the
\f(CW\s-1aoed\s+1\fP
driver was modified to keep certain values
as 32-bit numbers
even though 64 bits would have been more appropriate;
in particular,
the size of an AoE disk was limited to 32 bits' worth of sectors,
i.e. 2 tebibytes.
.IP \(bu
Variadic procedures are implemented differently in
\f2gcc\fP
and Sun C;
to supply a new variadic function
requires more
\f2gcc\fP-specific
runtime support.
This forbade the present
\f2admsg\fP
routine,
making error logging cumbersome.
.PP
These problems were observed with
\f2gcc\fP
version 3;
perhaps newer versions are better.
Sun's C compiler works fine,
no longer costs an arm and a leg,
and does stricter type-checking
(which has helped prevent a few bugs);
we plan to stick with it.
The compromises required for
\f2gcc\fP
have been removed from the code
and will not be reinstated.
.SH
9.2.
Solaris version differences
.PP
The same
\f(CW\s-1CORDaoe\s+1\fP
binary package may be installed on
Solaris 7, 8, 9, or 10.
Surprisingly little magic is required
to make this work:
despite marked changes in the underlying operating system,
the interfaces visible to device drivers
have been quite stable since Solaris 7.
.PP
Care is needed in a few places:
.IP \(bu
Newer versions of Solaris require
new disk-specific
\f2ioctl\fP
calls,
especially for
EFI-format label support.
It does no harm for the corresponding AoE code
to be present in older versions,
but it does make it difficult to compile the AoE subsystem
on an older system.
.IP \(bu
The argument structure to the
\f(CW\s-1DKIOCGETEFI\s+1\fP
and
\f(CW\s-1DKIOCSETEFI\s+1\fP
\f2ioctl\fPs,
introduced in Solaris 9,
was changed for Solaris 10,
apparently for the convenience of the library code
in which Sun use that call.
This call is used by
\f2prtvtoc\fP
and
\f2format\fP(1M),
so it is important that it work properly.
To paper this over,
the
\f(CW\s-1aoed\s+1\fP
module makes a somewhat-hacky explicit
runtime Solaris-version test.
.IP \(bu
In Solaris 9,
\f2ddi_create_minor_node\fP(9F)
gained some flags for Sun Cluster support;
those flag values disappeared from the header files
in Solaris 10.
The driver avoids them.
This may make it incompatible with Sun Clusters;
we haven't had a chance to test.
.IP \(bu
The package must
adapt when installed
to use SMF if the target supports it,
\f2init.d\fP(4)
otherwise.
.IP \(bu
A device to be added to a ZFS pool
must have the 64-bit integer property
\f(CW\s-1Nblocks\s+1\fP
defined.
A call to define a 64-bit integer property
wasn't added to the Sun driver interface
until Solaris 9.
A
custom interface routine
is required to paper over this,
so that the driver may support ZFS
but will still load on Solaris 7 and 8.
.PP
The AoE package currently compiles without error
only on Solaris 10,
though the resulting binaries may be run on any supported system.
.SH
9.3.
Disk labels
.PP
The original Solaris disk-label format
has limits that have recently become troublesome:
in particular,
it is unable to handle multi-terabyte disks.
A mid-life update to Solaris 9
introduced a new label format,
which removes the old limitations
at the cost of considerable extra complexity.
.PP
To top it off,
Solaris/SPARC and Solaris/x86
use different forms of old-style disk label,
and the x86 implementation allows an old-style label
to be encapsulated within a specially-designated DOS partition.
.PP
Here is a summary of what Sun did
and how it affects the AoE driver,
derived from Sun documentation,
experiment,
and some analysis of Open Solaris source code.
.SH
9.3.1.
VTOC (old-style) labels
.PP
The original Solaris
volume table of contents (VTOC) disk label
comprises a single sector (512 bytes)
of label information.
The VTOC label records
storing disk geometry information
(in particular cylinder, head, and sector counts),
a string label or two,
and an array of eight or sixteen partition descriptors.
Values are stored as 16- and 32-bit
native integers.
The label is protected by a magic number
and a simple-minded checksum.
.PP
The eight- and sixteen-partition label formats
differ quite a bit;
they are not meant to interoperate.
SPARC systems use the eight-partition variant,
x86 systems the sixteen-partition one.
.PP
The primary VTOC label is stored
in the first sector of the disk.
Backups are kept at sector offsets
1, 3, 5, 7, and 9
in the last track.
.PP
Many limitations of the VTOC format
are more obvious now than when
the scheme was adopted in the early 1990s:
.IP \(bu
The format is inherently architecture-dependent,
both because it contains native integers
and because of the 8-partition/16-partition divide.
Hence it is unreasonably messy to share disks
between SPARC and x86 systems.
That has become more likely with the spread of
shared-host storage interconnects
like AoE and Fibre Channel.
.IP \(bu
The labelling scheme depends to some extent
on the outdated assumption that
a disk is composed of an array of
equal-sized tracks and cylinders,
and the equally obsolete belief
that the disk will report such size numbers.
.IP \(bu
Partition sizes are stored as signed 32-bit numbers;
in effect a partition may be at most
2\u\s-131\s+1\d
sectors,
or 1TiB.
Storage arrays affording multi-tebibyte logical disks
are now common,
and often the point is to be able to
have a single large file system.
.SH
9.3.2.
DOS (fdisk) labels
.PP
Solaris/x86 allows
(in some cases requires)
a disk to have a DOS partition label.
.PP
A DOS label is stored in the first sector of the disk;
it contains four partition descriptors,
each including the starting and ending sector of the partition
and a partition type (`system ID').
The label is protected by a magic number.
Integer values in the label are always in
Intel (little-endian) order.
There is no backup label.
.PP
The four partitions described in the label
are called primary partitions.
There is a scheme to allow a primary partition
to encapsulate a logical disk
containing another DOS label,
affording additional logical partitions.
.PP
Solaris supports only primary partitions.
It has its own encapsulation scheme,
however:
partition type 130 or 191 (decimal)
indicates a logical disk with a Sun VTOC label.
Solaris makes the (relative) partitions
described in the VTOC
available through the conventional subdevices
\f(CW\s-1s0\s+1\fP-\f(CW\s-1s15\s+1\fP;
DOS primary partitions are accessed through
new devices
\f(CW\s-1p1\s+1\fP-\f(CW\s-1p4\s+1\fP,
whether or not there is an encapsulated VTOC.
.SH
9.3.3.
EFI (new-style) labels
.PP
Sun addressed the problems with
the old VTOC label scheme by adding support for EFI labels,
borrowed from the
Intel Extensible Firmware Interface
standard.
EFI support first appeared in Solaris 9 4/03.
Patches are available to add support to older copies of Solaris 9,
but not to Solaris 8 or older releases.
.PP
An EFI label comprises two data structures:
the GUID Partition Table header (GPT)
and a GUID Partition Entry (GPE) array.
.IP \(bu
The GPT is exactly one 512-byte sector.
It contains
header data,
including the size of the user-accessible part of the disk
(a contiguous area not occupied
by EFI data structures);
the disk address and length
of the GPE table;
the disk address of this copy of the GPT,
and of the other
(normally there are two);
and a magic number and checksum.
.IP \(bu
A GPE is 128 bytes.
Each descriptor includes the starting and ending
disk address of the partition,
a 128-bit unique partition identifier,
and an 128-bit partition-type code.
GPEs are stored in a contiguous array of sectors,
which must be at least 16kiB
(32 sectors, room for 128 GPEs)
but may be longer.
Not every GPE in the array need be active.
.PP
One copy of the GPT is stored in the second sector
of the disk,
immediately followed by the GPE array
in contiguous sectors.
A backup GPT is stored in the last sector,
immediately preceded by a backup GPE array.
Sectors from the end of the primary GPE array
to the beginning of the backup
may be allocated to partitions.
EFI partitions may not overlap one another,
nor may an EFI partition overlap the GPT or GPE or sector 0.
.PP
The EFI label contains no information
about cylinders, heads, or tracks;
the disk is treated as a simple linear array of sectors.
Sector addresses and counts are stored as 64-bit unsigned integers,
allowing for disks and individual partitions
as large as 8 zebibytes
(more than 8 billion tebibytes).
All integer values
are stored in a fixed byte order
defined by the EFI standard;
hence a label written by a big-endian SPARC system
may be read without difficulty
by a little-endian IA32 system
or vice versa.
.SH
9.3.4.
Bigger file systems
.PP
The original Solaris UFS file system format
also allows only 31 bits
for sector numbers.
In Solaris 9/03
(with corresponding patches
for earlier editions of Solaris 9),
Sun added a `multi-terabyte file system' variant.
.SH
9.3.5.
Sun implementation details
.PP
When there were only VTOC labels,
a Solaris disk driver was expected to support
these label-related
\f2ioctl\fP
commands:
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGGEOM\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSGEOM\s+1\fP
.in
Fetch or set a
\f(CW\s-1struct \s+1\fP\f(CW\s-1dk_geom\s+1\fP
containing disk-geometry information.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGVTOC\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSVTOC\s+1\fP
.in
Fetch or set a
\f(CW\s-1struct \s+1\fP\f(CW\s-1vtoc\s+1\fP
containing disk-partition information.
The on-disk VTOC contains a merger of
\f(CW\s-1struct \s+1\fP\f(CW\s-1vtoc\s+1\fP
and
\f(CW\s-1struct dk_geom\s+1\fP.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGAPART\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSAPART\s+1\fP
.in
Fetch or set a
\f(CW\s-1struct \s+1\fP\f(CW\s-1dk_allmap\s+1\fP
containing the starting cylinder number
and size in sectors
of all eight or sixteen partitions.
.PP
DOS-label support added these
\f2ioctl\fPs:
.IP
.in -\n(PIu
\f(CW\s-1DKIOCG_PHYGEOM\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCG_VIRTGEOM\s+1\fP
.in
Fetch physical or virtual geometry.
Physical is that of the whole disk;
virtual (apparently) that of the encapsulated-VTOC logical disk.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCPARTINFO\s+1\fP
.in
Fetch starting sector and length
of the current partition.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGMBOOT\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSMBOOT\s+1\fP
.in
Fetch or set the contents of the DOS label;
equivalent to reading or writing the first sector
of the physical disk.
.PP
To a Sun EFI-compliant driver,
a disk is
\f2VTOC-proper\fP
if its size is
no greater than
\f(CW\s-1DK_MAX_BLOCKS\s+1\fP
sectors
(\f(CW\s-10xffffffff\s+1\fP,
one less than a tebibyte);
such a disk may have either sort of label,
but defaults to VTOC.
A larger disk is
\f2VTOC-improper\fP:
either sort of label may be read,
but the default is EFI;
an existing VTOC label will be read
but cannot be written.
.PP
An EFI-compliant driver
supports the VTOC-label and
(if DOS labels are implemented)
DOS-label
\f2ioctl\fPs,
but returns error
\f(CW\s-1ENOTSUP\s+1\fP
if the disk has an EFI label,
or if
\f(CW\s-1DKIOCSVTOC\s+1\fP
is called on a VTOC-improper disk.
An EFI-compliant driver
also supports these
\f2ioctl\fPs:
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGETEFI\s+1\fP
.nr F! \n(PD
.nr PD 0
.IP
.in -\n(PIu
.nr PD \n(F!
\f(CW\s-1DKIOCSETEFI\s+1\fP
.in
Fetch or set EFI partition info.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCPARTITION\s+1\fP
.in
Fetch a
\f(CW\s-1struct \s+1\fP\f(CW\s-1partition64\s+1\fP
containing the starting sector address,
length,
and 128-bit partition-type code
for a designated partition.
.IP
.in -\n(PIu
\f(CW\s-1DKIOCGMEDIAINFO\s+1\fP
.in
Fetch a
\f(CW\s-1struct \s+1\fP\f(CW\s-1dk_minfo\s+1\fP
giving the sector size in bytes,
device size in sectors,
and a device-type code
(removable disk,
fixed disk,
CD-ROM,
CD-R or CD-RW,
floppy,
etc.).
.PP
The
\f(CW\s-1DKIOCGETEFI\s+1\fP
and
\f(CW\s-1DKIOCSETEFI\s+1\fP
\f2ioctl\fPs
really just perform I/O to
absolute disk addresses,
regardless of the partition table.
.PP
In an EFI-compliant version of Solaris,
utility programs like
\f2format\fP
and
\f2prtvtoc\fP(1M)
use new
\f2libefi\fP
and
\f2libvtoc\fP(3LIB)
libraries
to read and write disk labels.
\f2Libvtoc\fP
is a simple wrapper around the old
\f2ioctl\fPs.
\f2Libefi\fP
is more complicated:
when reading a label
it validates all the checksums
(device drivers just check the magic number);
when writing,
it computes correct checksums and validates other GPT and GPE values,
in particular enforcing the EFI-standard rule
that partitions may not overlap
and a Sun-specific rule that exactly one partition
must be of special
\f(CW\s-1reserved\s+1\fP
type,
presumably as a stand-in for the reserved cylinders
in the old VTOC scheme.
.PP
To find the label on a disk,
Sun's EFI-compliant drivers
search as follows:
.IP 1.
If sector 0
contains a valid VTOC label,
use it.
If the disk is not VTOC-proper,
print a warning.
There is apparently no attempt
to locate a backup VTOC label.
.IP 2.
If sector 1
or the last (backup-label) sector
contains a valid EFI label,
use it.
Sun's drivers use only the first eight or sixteen partitions.
Sun's drivers also ignore the eighth partition described in the label,
replacing partition
\f(CW\s-1:h\s+1\fP
with one called
\f(CW\s-1:wd\s+1\fP
mapping the entire physical disk,
including the label areas.
.IP 3.
If the driver supports DOS labels,
examine sector 0.
If it is a valid DOS label,
it.
If one of the DOS partitions
has an encapsulated-VTOC type,
use the VTOC label too.
If more than one DOS partition
has such a type,
ignore all but the highest-numbered.
.IP 4.
If no label can be found
but the disk is VTOC-proper,
invent a default partition table
in which partitions
\f(CW\s-1:a\s+1\fP
and
\f(CW\s-1:c\s+1\fP
span the whole disk.
.IP 5.
If no label can be found
but the disk is VTOC-improper,
set up no partition table at all,
not even the
\f(CW\s-1:wd\s+1\fP
partition.
Apparently it is expected that
\f2format\fP
will be used to label the disk before use.
(But what if the disk was written
by another operating system
with its own label scheme?)
.SH
9.3.6.
AoE implementation details
.PP
The
\f(CW\s-1aoed\s+1\fP
driver
recognizes VTOC, DOS (and encapsulated-VTOC), and EFI labels,
but with some differences from the Sun convention:
.IP \(bu
All
\f2ioctl\fP
calls are implemented.
All VTOC calls return
\f(CW\s-1ENOTSUP\s+1\fP
unless the disk already has a VTOC label,
or has no label and is VTOC-proper.
An existing VTOC label
may be updated with
\f(CW\s-1DKIOCSVTOC\s+1\fP
even if the disk is VTOC-improper;
Sun's drivers don't allow this.
.IP \(bu
The EFI
\f(CW\s-1:wd\s+1\fP
partition
uses a ninth (or seventeenth) partition,
rather than stealing one of the normal ones.
In fact that partition is valid
regardless of label type.
.IP \(bu
The label-search algorithm is similar to Sun's,
except that
\f(CW\s-1aoed\s+1\fP
checks for backup VTOC labels
if none is found in sector 0.
See the description of
\f2readptab\fP
for the full scoop.
.IP \(bu
Because there is a single AoE binary package per architecture
for all supported versions of Solaris,
\f(CW\s-1aoed\s+1\fP
always supports DOS and EFI labels
even when the surrounding system doesn't.
In practice this means an older system
may access existing partitions on a disk
that already has an EFI label,
or a SPARC system may access DOS partitions,
but tools like
\f2prtvtoc\fP
and
\f2format\fP
cannot print the partition table or update it.
Encapsulated VTOC labels are allowed only on Solaris/x86,
however,
so that the difference between SPARC and x86 VTOC-label formats
won't cause trouble.
.IP \(bu
AoE allows but does not require DOS labels,
regardless of system type.
.SH
9.4.
Concurrency issues
.PP
The Solaris kernel is pre-emptive:
all processing in the kernel
belongs to a scheduling thread
and may be pre-empted.
Thus locking is important
even on a single-processor system.
.SH
9.4.1.
Locks in
\f(CW\s-1aoecomm\s+1\fP
.PP
There are three global locks:
\f2chantlock\fP
protects the table that maps
\f2aoechan\fP
numbers to
\f(CW\s-1Aoechan\s+1\fP
structures;
\f2pendlock\fP
protects the pending-cookie table;
\f2loggerlock\fP
protects message-logger pointer.
Each of these locks is held for only a few lines of code at a time;
protected code sections contain no procedure calls,
and in particular no lock calls.
.PP
Each
\f(CW\s-1Aoechan\s+1\fP
structure
includes a lock,
protecting its contents.
These locks are also held only during short
code sequences
that cannot provoke other locks.
.PP
During normal operation,
the data protected by
\f2loggerlock\fP
and the
\f(CW\s-1Aoechan\s+1\fP
lock are written rarely
but read constantly.
These locks proved to be hot spots;
changing them from the
\f(CW\s-1kmutex_t\s+1\fP
type to
\f(CW\s-1krwlock_t\s+1\fP
made raw disk I/O measurably faster.
.SH
9.4.2.
Locks in
\f(CW\s-1aoectl\s+1\fP
.PP
There are two locks in
\f(CW\s-1aoectl\s+1\fP.
One serializes writes to the
\f(CW\s-1/dev/aoectl\s+1\fP
device;
the other prevents concurrent access to the buffer ring
feeding the
\f(CW\s-1/dev/aoemon\s+1\fP
device.
The two are entirely disjoint:
code using one lock
never calls code using the other.
.SH
9.4.3.
Locks in
\f(CW\s-1aoed\s+1\fP
.PP
Two locks are used throughout the
\f(CW\s-1aoed\s+1\fP
module:
one protecting the
\f(CW\s-1Adrive\s+1\fP
structure,
one the
\f(CW\s-1Achan\s+1\fP
structure.
To avoid nested-lock hangups,
there is a rule that
code in which an
\f(CW\s-1Adrive\s+1\fP
is locked
may lock
(or call code that locks)
an
\f(CW\s-1Achan\s+1\fP,
but not vice versa.
.PP
The driver's main entry points
locate and lock the relevant
\f(CW\s-1Adrive\s+1\fP
early on,
and unlock it just before returning.
Most subroutines
that take an
\f(CW\s-1Adrive\s+1\fP
argument assume it was already locked,
and leave it that way.
.PP
\f(CW\s-1Achan\s+1\fPs
are used
(hence locked)
only here and there,
and for short periods;
usually just for long enough to search
or to make a single change to
the active-command list.
Only rarely need an
\f(CW\s-1Adrive\s+1\fP
be accessed while an
\f(CW\s-1Achan\s+1\fP
is locked.
.PP
Timer routine
\f2chantimer\fP
contains an exception
that illustrates the rule.
\f2Chantimer\fP
locks an
\f(CW\s-1Achan\s+1\fP
while walking its active-command list
looking for expired commands.
If one is found,
the
\f(CW\s-1Achan\s+1\fP
is unlocked and the
\f(CW\s-1Adrive\s+1\fP
associated with the command locked
while the command is retransmitted or cancelled.
Then the
\f(CW\s-1Adrive\s+1\fP
is unlocked
and the
\f(CW\s-1Achan\s+1\fP
locked again;
and because the active-command list
may have changed while the
\f(CW\s-1Achan\s+1\fP
was unlocked,
\f2chantimer\fP
starts over from the beginning of the list.
.SH
9.5.
The startup dance
.PP
The channel-startup algorithm
is complicated by a potential security problem.
.PP
A STREAMS module has no associated permissions.
Anyone can open a stream device of some sort:
a pipe,
a network connection,
his own terminal.
Given an open stream,
anyone may push any module.
In particular,
anyone could push
\f(CW\s-1aoecomm\s+1\fP
onto one end of a pipe,
handle AoE messages on the other,
and the system would think it a valid
\f2aoechan\fP.
.PP
A malicious user could do this using a channel number
normally assigned to official disk devices.
An active channel number cannot be reused,
but a bad guy might attempt a race with the system administrator
or take advantage of a system problem.
.PP
Hence the algorithm that first requires
\f(CW\s-1/dev/aoectl\s+1\fP
to be opened:
that device file can be given whatever permissions
local policy dictates.
Someone who may open
\f(CW\s-1/dev/aoectl\s+1\fP
can set up AoE channels;
someone may not, cannot.
.SH
9.6.
How
\f(CW\s-1aoed\s+1\fP
fits into the system
.PP
The
\f(CW\s-1aoed\s+1\fP
driver is in a sense half-redundant.
Since the AoE protocol is just a way
to bundle ATA commands
into Ethernet packets,
one might think it would be possible
to write not a complete disk driver
but just an ATA host-adapter driver,
using the existing Solaris ATA code to do the rest of the work.
.PP
If it were SCSI-over-Ethernet rather than ATA,
that would be possible:
Sun expects third parties to write SCSI HBA drivers,
and documents the details.
Unfortunately that is not true for ATA.
Hence
\f(CW\s-1aoed\s+1\fP
is a standalone disk driver
attached to the
\f(CW\s-1pseudo\s+1\fP
nexus.
.PP
Perhaps this can be revisited in the future,
if Sun stabilize and publish their
ATA-adapter interface,
and once their ATA code has been fixed
to handle multi-terabyte disks.
Even then,
support for older Solaris versions would
require something like the current driver.
.SH
9.7.
Device-creation static
.PP
Device instances in Solaris are created in one of two ways:
.IP 1.
The system scans the available hardware.
When a device is discovered,
the corresponding driver's
\f2probe\fP
routine
is called to decide whether the device is usable.
If
\f2probe\fP
approves,
the driver's
\f2attach\fP
routine is called to create the software instance.
This works only if the system
(or some bus-adapter driver)
can generate a list of all existing devices,
and can somehow tell which device is of what type.
.IP 2.
Instances may be configured statically
in a driver configuration file.
The system reads the file,
and for each instance listed
creates a
\f(CW\s-1dev_info_t\s+1\fP
structure
and calls the driver
\f2attach\fP
routine.
.PP
Because AoE is a software construct
attached to the
\f(CW\s-1pseudo\s+1\fP
nexus,
only static configuration is allowed.
Some versions of Solaris even complain
if such a driver has a real
\f2probe\fP
entry point.
.PP
In Solaris 9 and earlier versions,
this means
devices can be attached
only when the
\f(CW\s-1aoed\s+1\fP
module is loaded.
To create more devices
requires shutting down the driver,
unloading the module,
and loading it afresh.
.PP
In Solaris 10,
a static device configuration can be updated
without unloading the driver;
\f2update_drv\fP(1M)
makes it happen.
Devices that are in use (open or mounted)
cannot be changed,
but new devices can be added
and idle devices removed
or given new properties.
At present,
the system administrator is expected to update
\f(CW\s-1aoed.conf\s+1\fP
and run
\f2update_drv\fP.
Perhaps devices can be detected automatically
in a future version,
though that might forbid
custom target-address-to-instance-number mappings.
.SH
9.8.
Device types,
and device-name horrors
.PP
It might seem natural for
\f(CW\s-1aoed\s+1\fP
to create device nodes by calling
\f2ddi_create_minor_node\fP(9F)
with node type
\f(CW\s-1DDI_NT_BLOCK_CHAN\s+1\fP,
using the
\f2aoemaj\fP
number
(or some combination of
\f2aoechan\fP
and
\f2aoemaj\fP)
as the device instance and
\f2aoemin\fP
as the target number.
\f(CW\s-1DDI_NT_BLOCK_CHAN\s+1\fP
is listed in the documentation,
but
nowhere do the manuals say how to supply a target number.
Apparently there are special hooks
for use by SCSI and ATA host-bus adapter drivers,
but no mechanism for general use.
Worse,
if one uses
\f(CW\s-1DDI_NT_BLOCK_CHAN\s+1\fP
anyway,
the system apparently picks a
garbage number out of uninitialized memory.
.PP
An early version of the driver used node type
\f(CW\s-1DDI_NT_BLOCK\s+1\fP.
This ran afoul of a bug in
\f2devfsadm\fP(1M)
in Solaris 7, 8, and 9.
Calling
\f2ddi_create_minor_node\fP
with the
\f2dev_info_t\fP
for instance 15
created devices named
\f(CW\s-1/devices/pseudo/aoed@15:*\s+1\fP,
as expected;
but the links to
\f(CW\s-1/dev/dsk\s+1\fP
and
\f(CW\s-1/dev/rdsk\s+1\fP
were named
\f(CW\s-1c\s+1\fP\f2N\fP\f(CW\s-1d21*\s+1\fP.
Apparently the code
that creates names in
\f(CW\s-1/devices\s+1\fP
believes instance numbers are decimal,
but the code that reads those names
and creates names in
\f(CW\s-1/dev\s+1\fP
believes the
\f(CW\s-1/devices\s+1\fP
names are hexadecimal
and `corrects' them.
.PP
To avoid these problems,
the
\f(CW\s-1aoed\s+1\fP
driver does everything the hard way.
Device nodes are created with type
\f(CW\s-1DDI_NT_PSEUDO\s+1\fP,
for which
\f2devfsadm\fP
does no automatic processing.
Installing the AoE subsystem
adds explicit rules to
\f(CW\s-1/etc/devlink.tab\s+1\fP
to generate
\f(CW\s-1/dev\s+1\fP
links
for
\f(CW\s-1aoed\s+1\fP:
.RS
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=a	dsk/cad\s+1\fP\f(CW\s-1A0s0\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=a,raw	rdsk/cad\s+1\fP\f(CW\s-1A0s0\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=b	dsk/cad\s+1\fP\f(CW\s-1A0s1\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=b,raw	rdsk/cad\s+1\fP\f(CW\s-1A0s1\s+1\fP
.br
\f(CW\s-1\&.\|.\|.\s+1\fP\f(CW\s-1\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=p	dsk/cad\s+1\fP\f(CW\s-1A0s15\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=p,raw	rdsk/cad\s+1\fP\f(CW\s-1A0s15\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=r	dsk/cad\s+1\fP\f(CW\s-1A0p1\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=r,raw	rdsk/cad\s+1\fP\f(CW\s-1A0p1\s+1\fP
.br
\f(CW\s-1\&.\|.\|.\s+1\fP\f(CW\s-1\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=u	dsk/cad\s+1\fP\f(CW\s-1A0p4\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=u,raw	rdsk/cad\s+1\fP\f(CW\s-1A0p4\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=wd	dsk/cad\s+1\fP\f(CW\s-1A0\s+1\fP
.br
\f(CW\s-1type=ddi_pseudo;name=aoed;minor=wd,raw	rdsk/cad\s+1\fP\f(CW\s-1A0\s+1\fP
.br
.RE
.PP
This approach reaps some minor benefits:
.IP \(bu
In older versions of Solaris,
\f2devfsadm\fP
doesn't know what to do with the
\f(CW\s-1wd\s+1\fP
partition.
The explicit rules generate a single consistent name.
.IP \(bu
If AoE were treated as just another disk,
Solaris would invent a numeric controller ID:
AoE disks might be named
\f(CW\s-1/dev/dsk/c1d\(**\s+1\fP\f(CW\s-1\s+1\fP
on one system,
\f(CW\s-1/dev/dsk/c2d\(**\s+1\fP\f(CW\s-1\s+1\fP
on another with different hardware or a different
Solaris version;
the names might even change after an OS reinstall.
Again,
the explicit rules
generate a single consistent name.
.PP
Rules are added for 16 Solaris partitions
even on systems supporting only eight Solaris partitions.
This makes installation easier
(one less special case),
and is harmless;
the system ignores any extra rules.
.SH
9.9.
\f2Format\fP
woes
.PP
In an early version of
\f2aoed\fP,
the
\f(CW\s-1DKIOCINFO\s+1\fP
\f2ioctl\fP
returned a customer controller type value
(\f(CW\s-1>=DKC_CUSTOMER\s+1\fP).
This causes
\f2format\fP(1M)
to misbehave quite badly:
.IP \(bu
When a customer-controller-type disk
is selected,
\f2format\fP
prompted for about a dozen subtle disk parameters,
most of them meaningless for modern ATA disks,
some already available via
\f(CW\s-1DKIOCINFO\s+1\fP.
.IP \(bu
When the disk label is updated,
\f2format\fP
attempts to read the backup labels
to verify that they were written correctly.
On a customer-type disk
it did this wrong:
it read mysteriously-chosen blocks
somewhere in the middle of the disk,
nowhere near the backup labels.
It couldn't read the backup labels anyway:
they are stored outside the user-data part of the disk.
.IP \(bu
On Solaris 9,
whren checking the label
or running tests on a
customer-type disk,
\f2format\fP
makes
\f2lseek\fP,
\f2read\fP,
and
\f2write\fP
calls with patently-invalid arguments
(as shown by
\f2truss\fP(1)).
Worse,
though the calls return errors as they should,
\f2format\fP
reports no trouble;
apparently the error returns are ignored.
.PP
To avoid this mess,
\f(CW\s-1aoed\s+1\fP
attempts to mimic a directly-attached ATA disk driver,
even though that is an undocumented interface.
\f(CW\s-1DKIOCINFO\s+1\fP
reports controller type
\f(CW\s-1DKC_DIRECT\s+1\fP;
the driver
implements a subset of
\f(CW\s-1DIOCTL_RWCMD\s+1\fP
\f2format\fP
uses for direct-address access to ATA disks.
The details are not officially documented;
they were
worked out by tracing
\f2format\fP,
reading
\f(CW\s-1<sys/dktp/dadkio.h>\s+1\fP,
and applying a mix of imagination and common sense.
.PP
The
EFI-label
code in
later Solaris 9 releases and in Solaris 10
introduced two new
\f2format\fP
embarrassments:
.IP \(bu
If an ATA disk is larger than 1TiB,
device-model information is fetched
with SCSI-specific
\f(CW\s-1ioctl\s+1\fP
calls.
This wouldn't work even with Sun's ATA-disk driver;
it's just a bug.
\f(CW\s-1aoed\s+1\fP
implements a fake
\f(CW\s-1USCSICMD\s+1\fP
\f2ioctl\fP.
Since ATA allows longer device-model strings than SCSI,
the string is sometimes truncated;
there's nothing we can do about that.
.IP \(bu
If an ATA disk is larger than 1TiB
and does not already have an EFI label,
\f2format\fP
cannot write one;
in fact
\f2format\fP
cannot do anything with such a disk.
The AoE subsystem includes a tool
\f2aoelabinit\fP
to write an initial label,
which
\f2format\fP
can then edit.
.PP
There is also at least one embarrassment in
DOS-label
support:
.IP \(bu
When a DOS label is present,
Solaris creates minor device
\f(CW\s-1:q\s+1\fP
(\f(CW\s-1p0\s+1\fP
in
\f(CW\s-1/dev\s+1\fP)
accessing the whole disk.
If this device exists,
\f2format\fP
insists that that the disk have a DOS label;
if none is present,
\f2format\fP
refuses even to print the existing partition table,
commanding
\f(CW\s-1Please use fdisk first\s+1\fP.
AoE therefore doesn't make the
\f(CW\s-1:q\s+1\fP/\f(CW\s-1p0\s+1\fP
device.
The
\f(CW\s-1:wd\s+1\fP
(no suffix in
\f(CW\s-1/dev\s+1\fP)
device,
created by AoE regardless of label type,
affords equivalent access if needed.
.LP
Some of these bugs may have been cured in
Solaris 10 11/06.
.SH
9.10.
Waiting for the network
.PP
When a Solaris Ethernet device is opened,
it sometimes takes a few seconds for the hardware
and software
to initialize.
During this interval,
DLPI attachment and configuration messages
are processed correctly,
and DLPI queries
report that the device is ready,
but it isn't:
messages written to the device
are silently thrown away.
Thus
\f2aoestart\fP
must somehow wait until the device is really ready
before broadcasting a Query-Config message
to discover which devices exist.
.PP
\f2Comminit\fP,
the library routine called by
\f2aoestart\fP
to open and initialize the device,
tries two schemes to wait for the hardware:
.IP 1.
Using the
\f2kstat\fP(3kstat)
library,
search for a
\f2kstat\fP
structure associated with the network device
bearing an integer-valued
\f(CW\s-1link_up\s+1\fP
parameter.
If found,
wait up to 30 seconds for the value
to become positive;
when it does,
return success.
.IP 2.
If no
\f2link_up\fP
parameter was found in a
\f2kstat\fP,
use magic calls equivalent to the innards of
\f2ndd\fP(1M)
to query the
\f(CW\s-1link_status\s+1\fP
value for the device.
Wait up to 30 seconds for the value
to become positive;
when it does,
return success.
A global state variable in the kernel must be changed
to select the device to be polled;
good citizenship suggests its original value should be saved
just before every test
and restored immediately after,
though races are still possible.
.LP
If neither test is possible
or the test chosen fails,
\f2comminit\fP
returns an error.
.PP
The
\f2kstat\fP
test is tried first because it's a bit faster
and rather less hacky in implementation
than the
\f2ndd\fP
test.
Recent network-device drivers
no longer support the
\f2ndd\fP
parameter,
but all Sun-supported network devices we know
support the
\f2kstat\fP
scheme,
except the old
\f(CW\s-1le\s+1\fP
(10Mbps LANCE Ethernet)
driver.
The latter device isn't a good choice for AoE,
but support will remain for now
because it is sometimes useful during our own testing.
.SH
9.11.
\f2Newfs\fP
woes
.PP
Solaris
\f2newfs\fP
gets into trouble when a large disk
has small cylinders,
producing complaints like
`Insufficient space in super block for rotational tables'
and
`inode blocks/cyl group >= data blocks.'
The details are not yet understood;
possibly
\f2newfs\fP
is incorrectly doing some bit of arithmetic
in too small a variable.
.PP
Empirically the trouble seems to vanish
when cylinders
are very large.
Hence the phony numbers generated by
\f2fudgegeom\fP,
which make cylinders as big as the system seems to be able to stand.
.PP
The
\f(CW\s-1-T\s+1\fP
(new multi-terabyte file system)
\f2newfs\fP
option in Solaris 10
and in newer editions of Solaris 9
also avoids the problem,
but that's no help in older versions of Solaris,
nor on 32-bit hardware.
.SH
9.12.
I/O timeouts
.PP
The default timeouts for I/O operations recorded in
\f2aoed.h\fP
are surprisingly long:
200ms for a read or write,
half a second for an AoE Query-Config
or ATA IDENTIFY.
Originally much smaller values were used,
but complicated devices like RAID controllers
really do take as long as 100-150ms
to finish some operations.
.PP
Longer timeouts shouldn't cause much grief anyway
except in special circumstances,
since lost messages are unlikely on modern
(switched, flow-controlled) networks
unless something is broken.
.PP
The large values may cause grief on
slow, congested networks,
e.g. 10Mbps or 100Mbps,
especially when repeaters or broadcast cables are used
rather than switches.
If that case comes up in real life,
just set the timeouts down manually.
If it happens often enough
it may make sense to invent a way
to set a per-channel I/O timeout default
rather than having to set it for each target.
